@@ -10,6 +10,5 @@ RED = '\x1b[31m'
 GREEN = '\x1b[32m'
 YELLOW = '\x1b[33m'
 BLUE = '\x1b[34m'
-
-#_MAGENTA = '\x1b[35m'
-#_CYAN = '\x1b[36m'
+MAGENTA = '\x1b[35m'
+CYAN = '\x1b[36m'
