@@ -260,10 +260,10 @@ class _ArgAction(_Action):
     def __init__(self, name, quit_parsing_flags, valid=None):
         # type: (str, bool, Optional[List[str]]) -> None
         """
-    Args:
-      quit_parsing_flags: Stop parsing args after this one.  for sh -c.
-        python -c behaves the same way.
-    """
+        Args:
+          quit_parsing_flags: Stop parsing args after this one.  for sh -c.
+            python -c behaves the same way.
+        """
         self.name = name
         self.quit_parsing_flags = quit_parsing_flags
         self.valid = valid
@@ -352,7 +352,6 @@ class SetToString(_ArgAction):
 
 
 class SetAttachedBool(_Action):
-    """This is the Go-like syntax of --verbose=1, --verbose, or --verbose=0."""
 
     def __init__(self, name):
         # type: (str) -> None
@@ -362,6 +361,12 @@ class SetAttachedBool(_Action):
         # type: (Optional[str], Reader, _Attributes) -> bool
         """Called when the flag matches."""
 
+        # TODO: Delete this part?  Is this eqvuivalent to SetToTrue?
+        #
+        # We're not using Go-like --verbose=1, --verbose, or --verbose=0
+        #
+        # 'attached_arg' is also used for -t0 though, which is weird
+
         if attached_arg is not None:  # '0' in --verbose=0
             if attached_arg in ('0', 'F', 'false',
                                 'False'):  # TODO: incorrect translation
@@ -625,7 +630,7 @@ def ParseMore(spec, arg_r):
             if action is None:
                 e_usage('got invalid flag %r' % arg, arg_r.Location())
 
-            # TODO: attached_arg could be 'bar' for --foo=bar
+            # Note: not parsing --foo=bar as attached_arg, as above
             action.OnMatch(None, arg_r, out)
             arg_r.Next()
             continue
