@@ -84,7 +84,7 @@ from osh import braces
 from osh import sh_expr_eval
 from osh import word_eval
 from mycpp import mylib
-from mycpp.mylib import log, switch, tagswitch
+from mycpp.mylib import log, probe, switch, tagswitch
 from ysh import expr_eval
 from ysh import func_proc
 from ysh import val_ops
@@ -118,7 +118,7 @@ def MakeBuiltinArgv(argv1):
     argv = ['']  # dummy for argv[0]
     argv.extend(argv1)
     missing = None  # type: CompoundWord
-    return cmd_value.Argv(argv, [missing] * len(argv), None, None, None)
+    return cmd_value.Argv(argv, [missing] * len(argv), None, None, None, None)
 
 
 class Deps(object):
@@ -327,14 +327,29 @@ class CommandEvaluator(object):
             e_die("Assignment builtin %r not configured" % cmd_val.argv[0],
                   cmd_val.arg_locs[0])
 
-        with vm.ctx_FlushStdout():
+        io_errors = []  # type: List[error.IOError_OSError]
+        with vm.ctx_FlushStdout(io_errors):
             with ui.ctx_Location(self.errfmt, cmd_val.arg_locs[0]):
                 try:
                     status = builtin_func.Run(cmd_val)
+                except (IOError, OSError) as e:
+                    # e.g. declare -p > /dev/full
+                    self.errfmt.PrintMessage(
+                        '%s builtin I/O error: %s' %
+                        (cmd_val.argv[0], pyutil.strerror(e)),
+                        cmd_val.arg_locs[0])
+                    return 1
                 except error.Usage as e:  # Copied from RunBuiltin
                     arg0 = cmd_val.argv[0]
                     self.errfmt.PrefixPrint(e.msg, '%r ' % arg0, e.location)
-                    status = 2  # consistent error code for usage error
+                    return 2  # consistent error code for usage error
+
+        if len(io_errors):  # e.g. declare -p > /dev/full
+            self.errfmt.PrintMessage(
+                '%s builtin I/O: %s' %
+                (cmd_val.argv[0], pyutil.strerror(io_errors[0])),
+                cmd_val.arg_locs[0])
+            return 1
 
         return status
 
@@ -760,7 +775,7 @@ class CommandEvaluator(object):
 
             num_lhs = len(node.lhs)
             if num_lhs == 1:
-                lvals = [self.expr_ev.EvalLhsExpr(node.lhs[0])]
+                lvals = [self.expr_ev.EvalLhsExpr(node.lhs[0], which_scopes)]
                 rhs_vals = [right_val]
             else:
                 items = val_ops.ToList(
@@ -776,7 +791,8 @@ class CommandEvaluator(object):
                 lvals = []
                 rhs_vals = []
                 for i, lhs_val in enumerate(node.lhs):
-                    lvals.append(self.expr_ev.EvalLhsExpr(lhs_val))
+                    lvals.append(
+                        self.expr_ev.EvalLhsExpr(lhs_val, which_scopes))
                     rhs_vals.append(items[i])
 
             for i, lval in enumerate(lvals):
@@ -823,13 +839,14 @@ class CommandEvaluator(object):
             # Checked in the parser
             assert len(node.lhs) == 1
 
-            aug_lval = self.expr_ev.EvalLhsExpr(node.lhs[0])
+            aug_lval = self.expr_ev.EvalLhsExpr(node.lhs[0], which_scopes)
             val = self.expr_ev.EvalExpr(node.rhs, loc.Missing)
 
             self.expr_ev.EvalAugmented(aug_lval, val, node.op, which_scopes)
 
     def _DoSimple(self, node, cmd_st):
         # type: (command.Simple, CommandStatus) -> int
+        probe('cmd_eval', '_DoSimple_enter')
         cmd_st.check_errexit = True
 
         # PROBLEM: We want to log argv in 'xtrace' mode, but we may have already
@@ -890,6 +907,7 @@ class CommandEvaluator(object):
         else:
             status = self._RunSimpleCommand(cmd_val, cmd_st, run_flags)
 
+        probe('cmd_eval', '_DoSimple_exit', status)
         return status
 
     def _DoExpandedAlias(self, node):
@@ -1002,8 +1020,21 @@ class CommandEvaluator(object):
         val = self.expr_ev.EvalExpr(node.e, loc.Missing)
 
         if node.keyword.id == Id.Lit_Equals:  # = f(x)
-            with vm.ctx_FlushStdout():
-                ui.PrettyPrintValue(val, mylib.Stdout())
+            io_errors = []  # type: List[error.IOError_OSError]
+            with vm.ctx_FlushStdout(io_errors):
+                try:
+                    ui.PrettyPrintValue(val, mylib.Stdout())
+                except (IOError, OSError) as e:
+                    self.errfmt.PrintMessage(
+                        'I/O error during = keyword: %s' % pyutil.strerror(e),
+                        node.keyword)
+                    return 1
+
+            if len(io_errors):  # e.g. disk full, ulimit
+                self.errfmt.PrintMessage(
+                    'I/O error during = keyword: %s' %
+                    pyutil.strerror(io_errors[0]), node.keyword)
+                return 1
 
         return 0
 
@@ -1383,7 +1414,11 @@ class CommandEvaluator(object):
         fnmatch_flags = FNM_CASEFOLD if self.exec_opts.nocasematch() else 0
 
         status = 0  # If there are no arms, it should be zero?
-        matched = False
+
+        done = False  # Should we try the next arm?
+
+        # For &; terminator - not just case fallthrough, but IGNORE the condition!
+        ignore_next_cond = False
 
         for case_arm in node.arms:
             with tagswitch(case_arm.pattern) as case:
@@ -1394,15 +1429,34 @@ class CommandEvaluator(object):
 
                     pat_words = cast(pat.Words, case_arm.pattern)
 
-                    for pat_word in pat_words.words:
-                        word_val = self.word_ev.EvalWordToString(
-                            pat_word, word_eval.QUOTE_FNMATCH)
+                    this_arm_matches = False
+                    if ignore_next_cond:  # Special handling for ;&
+                        this_arm_matches = True
+                        ignore_next_cond = False
+                    else:
+                        for pat_word in pat_words.words:
+                            word_val = self.word_ev.EvalWordToString(
+                                pat_word, word_eval.QUOTE_FNMATCH)
 
-                        if libc.fnmatch(word_val.s, to_match_str.s,
-                                        fnmatch_flags):
-                            status = self._ExecuteList(case_arm.action)
-                            matched = True  # TODO: Parse ;;& and for fallthrough and such?
-                            break
+                            if libc.fnmatch(word_val.s, to_match_str.s,
+                                            fnmatch_flags):
+                                this_arm_matches = True
+                                break  # Stop at first pattern
+
+                    if this_arm_matches:
+                        status = self._ExecuteList(case_arm.action)
+                        done = True
+
+                        # ;& and ;;& only apply to shell-style case
+                        if case_arm.right:
+                            id_ = case_arm.right.id
+                            if id_ == Id.Op_SemiAmp:
+                                # very weird semantic
+                                ignore_next_cond = True
+                                done = False
+                            elif id_ == Id.Op_DSemiAmp:
+                                # Keep going until next pattern
+                                done = False
 
                 elif case(pat_e.YshExprs):
                     pat_exprs = cast(pat.YshExprs, case_arm.pattern)
@@ -1414,7 +1468,7 @@ class CommandEvaluator(object):
                         if val_ops.ExactlyEqual(expr_val, to_match,
                                                 case_arm.left):
                             status = self._ExecuteList(case_arm.action)
-                            matched = True
+                            done = True
                             break
 
                 elif case(pat_e.Eggex):
@@ -1423,18 +1477,18 @@ class CommandEvaluator(object):
 
                     if val_ops.MatchRegex(to_match, eggex_val, self.mem):
                         status = self._ExecuteList(case_arm.action)
-                        matched = True
+                        done = True
                         break
 
                 elif case(pat_e.Else):
                     status = self._ExecuteList(case_arm.action)
-                    matched = True
+                    done = True
                     break
 
                 else:
                     raise AssertionError()
 
-            if matched:  # first match wins
+            if done:  # first match wins
                 break
 
         return status
@@ -1463,6 +1517,7 @@ class CommandEvaluator(object):
         # If we call RunCommandSub in a recursive call to the executor, this will
         # be set true (if strict_errexit is false).  But it only lasts for one
         # command.
+        probe('cmd_eval', '_Dispatch', node.tag())
         self.check_command_sub_status = False
 
         UP_node = node
