@@ -10,7 +10,7 @@ cmd_parse.py - Parse high level shell commands.
 from __future__ import print_function
 
 from _devbuild.gen import grammar_nt
-from _devbuild.gen.id_kind_asdl import Id, Id_t, Kind, Kind_str
+from _devbuild.gen.id_kind_asdl import Id, Id_t, Id_str, Kind, Kind_str
 from _devbuild.gen.types_asdl import lex_mode_e, cmd_mode_e, cmd_mode_t
 from _devbuild.gen.syntax_asdl import (
     loc,
@@ -154,12 +154,16 @@ def _MakeLiteralHereLines(
 
         # Maintain lossless invariant for STRIPPED tabs: add a Token to the
         # arena invariant, but don't refer to it.
+        #
+        # Note: We could use Lit_CharsWithoutPrefix for 'single quoted' EOF
+        # here docs, but it's more complex with double quoted EOF docs.
+
         if do_lossless:  # avoid garbage, doesn't affect correctness
-            arena.NewToken(Id.Ignored_HereTabs, 0, start_offset, src_line,
-                           None)
+            arena.NewToken(Id.Lit_CharsWithoutPrefix, start_offset, 0,
+                           src_line)
 
         t = arena.NewToken(Id.Lit_Chars, start_offset, len(src_line.content),
-                           src_line, src_line.content[start_offset:])
+                           src_line)
         tokens.append(t)
     return tokens
 
@@ -194,12 +198,12 @@ def _ParseHereDocBody(parse_ctx, r, line_reader, arena):
     # Maintain lossless invariant for STRIPPED tabs: add a Token to the
     # arena invariant, but don't refer to it.
     if parse_ctx.do_lossless:  # avoid garbage, doesn't affect correctness
-        arena.NewToken(Id.Ignored_HereTabs, 0, start_offset, end_line, None)
+        arena.NewToken(Id.Lit_CharsWithoutPrefix, start_offset, 0, end_line)
 
     # Create a Token with the end terminator.  Maintains the invariant that the
     # tokens "add up".
     h.here_end_tok = arena.NewToken(Id.Undefined_Tok, start_offset,
-                                    len(end_line.content), end_line, '')
+                                    len(end_line.content), end_line)
 
 
 def _MakeAssignPair(parse_ctx, preparsed, arena):
@@ -270,10 +274,9 @@ def _MakeAssignPair(parse_ctx, preparsed, arena):
     if offset == n:
         rhs = rhs_word.Empty  # type: rhs_word_t
     else:
-        # tmp2 is for intersection of C++/MyPy type systems
-        tmp2 = CompoundWord(parts[offset:])
-        word_.TildeDetectAssign(tmp2)
-        rhs = tmp2
+        w = CompoundWord(parts[offset:])
+        word_.TildeDetectAssign(w)
+        rhs = w
 
     return AssignPair(left_token, lhs, op, rhs)
 
@@ -303,11 +306,13 @@ def _AppendMoreEnv(preparsed_list, more_env):
         n = len(parts)
         offset = preparsed.part_offset
         if offset == n:
-            val = rhs_word.Empty  # type: rhs_word_t
+            rhs = rhs_word.Empty  # type: rhs_word_t
         else:
-            val = CompoundWord(parts[offset:])
+            w = CompoundWord(parts[offset:])
+            word_.TildeDetectAssign(w)
+            rhs = w
 
-        more_env.append(EnvPair(left_token, var_name, val))
+        more_env.append(EnvPair(left_token, var_name, rhs))
 
 
 def _SplitSimpleCommandPrefix(words):
@@ -607,8 +612,7 @@ class CommandParser(object):
         self.c_kind = Kind.Undefined
         self.c_id = Id.Undefined_Tok
 
-        self.pending_here_docs = [
-        ]  # type: List[Redir]  # should have HereLiteral arg
+        self.pending_here_docs = []  # type: List[Redir]
 
     def ResetInputObjects(self):
         # type: () -> None
@@ -631,7 +635,7 @@ class CommandParser(object):
 
     def _SetNextBrack(self):
         # type: () -> None
-        self.next_lex_mode = lex_mode_e.ShCommandBrack
+        self.next_lex_mode = lex_mode_e.ShCommandFakeBrack
 
     def _GetWord(self):
         # type: () -> None
@@ -659,6 +663,7 @@ class CommandParser(object):
             self.cur_word = w
 
             self.c_kind = word_.CommandKind(self.cur_word)
+            # Has special case for Id.Lit_{LBrace,RBrace,Equals}
             self.c_id = word_.CommandId(self.cur_word)
             self.next_lex_mode = lex_mode_e.Undefined
 
@@ -709,6 +714,9 @@ class CommandParser(object):
         # which would make the code below faster.  But small string optimization
         # would also speed it up, since redirects are small.
 
+        # One way to do this is with Kind.Redir and Kind.RedirNamed, and then
+        # possibly "unify" the IDs by subtracting a constant like 8 or 16?
+
         op_val = lexer.TokenVal(op_tok)
         if op_val[0] == '{':
             pos = op_val.find('}')
@@ -804,15 +812,29 @@ class CommandParser(object):
 
         YSH grammar:
 
-        simple_command = 
-            cmd_prefix* word+ typed_args? BraceGroup? cmd_suffix*
+        redirect = redir_op WORD
+        item = WORD | redirect
 
         typed_args =
           '(' arglist ')'
         | '[' arglist ']'
 
-        Notably, redirects shouldn't appear after between typed args and
+        simple_command = 
+            cmd_prefix* item+ typed_args? BraceGroup? cmd_suffix*
+
+        Notably, redirects shouldn't appear after typed args, or after
         BraceGroup.
+
+        Examples:
+
+        This is an assignment:
+           foo=1 >out
+
+        This is a command.Simple
+           >out
+
+        What about
+          >out (42)
         """
         redirects = []  # type: List[Redir]
         words = []  # type: List[CompoundWord]
@@ -824,37 +846,18 @@ class CommandParser(object):
         i = 0
         while True:
             self._GetWord()
-            if self.c_kind == Kind.Redir:
+
+            # If we got { }, change it to something that's not Kind.Word
+            kind2 = self.c_kind
+            if (kind2 == Kind.Word and self.parse_opts.parse_brace() and
+                    self.c_id in (Id.Lit_LBrace, Id.Lit_RBrace)):
+                kind2 = Kind.Op
+
+            if kind2 == Kind.Redir:
                 node = self.ParseRedirect()
                 redirects.append(node)
 
-            elif self.c_kind == Kind.Word:
-                if self.parse_opts.parse_brace():
-                    # Treat { and } more like operators
-                    if self.c_id == Id.Lit_LBrace:
-                        if self.allow_block:  # Disabled for if/while condition, etc.
-
-                            # allow x = 42
-                            self.hay_attrs_stack.append(first_word_caps)
-                            brace_group = self.ParseBraceGroup()
-
-                            # So we can get the source code back later
-                            lines = self.arena.SaveLinesAndDiscard(
-                                brace_group.left, brace_group.right)
-                            block = LiteralBlock(brace_group, lines)
-
-                            self.hay_attrs_stack.pop()
-
-                        if 0:
-                            print('--')
-                            block.PrettyPrint()
-                            print('\n--')
-                        break
-                    elif self.c_id == Id.Lit_RBrace:
-                        # Another thing: { echo hi }
-                        # We're DONE!!!
-                        break
-
+            elif kind2 == Kind.Word:
                 w = cast(CompoundWord, self.cur_word)  # Kind.Word ensures this
 
                 if i == 0:
@@ -868,6 +871,11 @@ class CommandParser(object):
                                 tok)
 
                     # Is the first word a Hay Attr word?
+                    #
+                    # Can we remove this StaticEval() call, and just look
+                    # inside Token?  I think once we get rid of SHELL nodes,
+                    # this will be simpler.
+
                     ok, word_str, quoted = word_.StaticEval(w)
                     # Foo { a = 1 } is OK, but not foo { a = 1 } or FOO { a = 1 }
                     if (ok and len(word_str) and word_str[0].isupper() and
@@ -877,43 +885,78 @@ class CommandParser(object):
 
                 words.append(w)
 
-            elif self.c_id == Id.Op_LParen:
-                # 1. Check that there's a preceding space
-                prev_byte = self.lexer.ByteLookBack()
-                if prev_byte not in (SPACE_CH, TAB_CH):
-                    if self.parse_opts.parse_at():
-                        p_die('Space required before (',
-                              loc.Word(self.cur_word))
-                    else:
-                        # inline func call like @sorted(x) is invalid in OSH, but the
-                        # solution isn't a space
-                        p_die(
-                            'Unexpected left paren (might need a space before it)',
-                            loc.Word(self.cur_word))
-
-                # 2. Check that it's not ().  We disallow this because it's a no-op and
-                #    there could be confusion with shell func defs.
-                # For some reason we need to call lexer.LookPastSpace, not
-                # w_parser.LookPastSpace.  I think this is because we're at (, which is
-                # an operator token.  All the other cases are like 'x=', which is PART
-                # of a word, and we don't know if it will end.
-                next_id = self.lexer.LookPastSpace(lex_mode_e.ShCommand)
-                if next_id == Id.Op_RParen:
-                    p_die('Empty arg list not allowed',
-                          loc.Word(self.cur_word))
-
-                typed_args = self.w_parser.ParseProcCallArgs(
-                    grammar_nt.ysh_eager_arglist)
-
-            elif self.c_id == Id.Op_LBracket:  # only when parse_bracket set
-                typed_args = self.w_parser.ParseProcCallArgs(
-                    grammar_nt.ysh_lazy_arglist)
-
             else:
                 break
 
             self._SetNextBrack()  # Allow bracket for SECOND word on
             i += 1
+
+        # my-cmd (x) or my-cmd [x]
+        self._GetWord()
+        if self.c_id == Id.Op_LParen:
+            # 1. Check that there's a preceding space
+            prev_byte = self.lexer.ByteLookBack()
+            if prev_byte not in (SPACE_CH, TAB_CH):
+                if self.parse_opts.parse_at():
+                    p_die('Space required before (', loc.Word(self.cur_word))
+                else:
+                    # inline func call like @sorted(x) is invalid in OSH, but the
+                    # solution isn't a space
+                    p_die(
+                        'Unexpected left paren (might need a space before it)',
+                        loc.Word(self.cur_word))
+
+            # 2. Check that it's not ().  We disallow this because it's a no-op and
+            #    there could be confusion with shell func defs.
+            # For some reason we need to call lexer.LookPastSpace, not
+            # w_parser.LookPastSpace.  I think this is because we're at (, which is
+            # an operator token.  All the other cases are like 'x=', which is PART
+            # of a word, and we don't know if it will end.
+            next_id = self.lexer.LookPastSpace(lex_mode_e.ShCommand)
+            if next_id == Id.Op_RParen:
+                p_die('Empty arg list not allowed', loc.Word(self.cur_word))
+
+            typed_args = self.w_parser.ParseProcCallArgs(
+                grammar_nt.ysh_eager_arglist)
+
+            self._SetNext()
+
+        elif self.c_id == Id.Op_LBracket:  # only when parse_bracket set
+            typed_args = self.w_parser.ParseProcCallArgs(
+                grammar_nt.ysh_lazy_arglist)
+
+            self._SetNext()
+
+        self._GetWord()
+
+        # Allow redirects after typed args, e.g.
+        #    json write (x) > out.txt
+        if self.c_kind == Kind.Redir:
+            redirects.extend(self._ParseRedirectList())
+
+        # my-cmd { echo hi }   my-cmd (x) { echo hi }   ...
+        if (self.parse_opts.parse_brace() and self.c_id == Id.Lit_LBrace and
+                # Disabled for if/while condition, etc.
+                self.allow_block):
+
+            # allow x = 42
+            self.hay_attrs_stack.append(first_word_caps)
+            brace_group = self.ParseBraceGroup()
+
+            # So we can get the source code back later
+            lines = self.arena.SaveLinesAndDiscard(brace_group.left,
+                                                   brace_group.right)
+            block = LiteralBlock(brace_group, lines)
+
+            self.hay_attrs_stack.pop()
+
+        self._GetWord()
+
+        # Allow redirects after block, e.g.
+        #    cd /tmp { echo $PWD } > out.txt
+        if self.c_kind == Kind.Redir:
+            redirects.extend(self._ParseRedirectList())
+
         return redirects, words, typed_args, block
 
     def _MaybeExpandAliases(self, words):
@@ -1057,8 +1100,7 @@ class CommandParser(object):
 
     def ParseSimpleCommand(self):
         # type: () -> command_t
-        """Fixed transcription of the POSIX grammar (TODO: port to
-        grammar/Shell.g)
+        """Fixed transcription of the POSIX grammar
 
         io_file        : '<'       filename
                        | LESSAND   filename
@@ -1302,7 +1344,7 @@ class CommandParser(object):
                          ;
         """
         words = []  # type: List[CompoundWord]
-        # The span_id of any semi-colon, so we can remove it.
+        # The token of any semi-colon, so we can remove it.
         semi_tok = None  # type: Optional[Token]
 
         while True:
@@ -1571,7 +1613,8 @@ class CommandParser(object):
         self._NewlineOk()
 
         self._GetWord()
-        if self.c_id not in (Id.Op_DSemi, Id.KW_Esac):
+        if self.c_id not in (Id.Op_DSemi, Id.Op_SemiAmp, Id.Op_DSemiAmp,
+                             Id.KW_Esac):
             c_list = self._ParseCommandTerm()
             action_children = c_list.children
         else:
@@ -1581,7 +1624,7 @@ class CommandParser(object):
         self._GetWord()
         if self.c_id == Id.KW_Esac:  # missing last ;;
             pass
-        elif self.c_id == Id.Op_DSemi:
+        elif self.c_id in (Id.Op_DSemi, Id.Op_SemiAmp, Id.Op_DSemiAmp):
             dsemi_tok = word_.AsOperatorToken(self.cur_word)
             self._SetNext()
         else:
@@ -1720,7 +1763,7 @@ class CommandParser(object):
         arms = []  # type: List[CaseArm]
         while True:
             self._GetWord()
-            if self.c_id == Id.KW_Esac:  # this is Kind.Word
+            if self.c_id == Id.KW_Esac:
                 break
             # case arm should begin with a pattern word or (
             if self.c_kind != Kind.Word and self.c_id != Id.Op_LParen:
@@ -1778,7 +1821,7 @@ class CommandParser(object):
             body = self.ParseBraceGroup()
             self._GetWord()
 
-            arm = IfArm(elif_kw, cond, None, body.children, [elif_kw.span_id])
+            arm = IfArm(elif_kw, cond, None, body.children, None)
             arms.append(arm)
 
         self._GetWord()
@@ -1789,8 +1832,8 @@ class CommandParser(object):
 
     def _ParseYshIf(self, if_kw, cond):
         # type: (Token, condition_t) -> command.If
-        """if test -f foo {
-
+        """
+        if test -f foo {
                      # ^ we parsed up to here
           echo foo
         } elif test -f bar; test -f spam {
@@ -1812,7 +1855,7 @@ class CommandParser(object):
         body1 = self.ParseBraceGroup()
         # Every arm has 1 spid, unlike shell-style
         # TODO: We could get the spids from the brace group.
-        arm = IfArm(if_kw, cond, None, body1.children, [if_kw.span_id])
+        arm = IfArm(if_kw, cond, None, body1.children, None)
 
         if_node.arms.append(arm)
 
@@ -1841,8 +1884,7 @@ class CommandParser(object):
             then_kw = word_.AsKeywordToken(ate)
 
             body = self._ParseCommandList()
-            arm = IfArm(elif_kw, cond, then_kw, body.children,
-                        [elif_kw.span_id, then_kw.span_id])
+            arm = IfArm(elif_kw, cond, then_kw, body.children, then_kw)
 
             arms.append(arm)
 
@@ -1875,8 +1917,8 @@ class CommandParser(object):
         if_node.if_kw = if_kw
         self._SetNext()  # past 'if'
 
-        if self.parse_opts.parse_paren() and self.w_parser.LookPastSpace(
-        ) == Id.Op_LParen:
+        if (self.parse_opts.parse_paren() and
+                self.w_parser.LookPastSpace() == Id.Op_LParen):
             # if (x + 1)
             enode = self.w_parser.ParseYshExprForCommand()
             cond = condition.YshExpr(enode)  # type: condition_t
@@ -1895,8 +1937,7 @@ class CommandParser(object):
         body = self._ParseCommandList()
 
         # First arm
-        arm = IfArm(if_kw, cond, then_kw, body.children,
-                    [if_kw.span_id, then_kw.span_id])
+        arm = IfArm(if_kw, cond, then_kw, body.children, then_kw)
         if_node.arms.append(arm)
 
         # 2nd to Nth arm
@@ -1968,13 +2009,17 @@ class CommandParser(object):
             return n5
 
         if self.c_id == Id.KW_DLeftBracket:
+            if not self.parse_opts.parse_dbracket():
+                p_die('Bash [[ not allowed in YSH (parse_dbracket)',
+                      loc.Word(self.cur_word))
             n6 = self.ParseDBracket()
             n6.redirects = self._ParseRedirectList()
             return n6
         if self.c_id == Id.Op_DLeftParen:
             if not self.parse_opts.parse_dparen():
-                p_die('You may want a space between parens (parse_dparen)',
-                      loc.Word(self.cur_word))
+                p_die(
+                    'Bash (( not allowed in YSH (parse_dparen, see OILS-ERR-14 for wart)',
+                    loc.Word(self.cur_word))
             n7 = self.ParseDParen()
             n7.redirects = self._ParseRedirectList()
             return n7
@@ -1984,8 +2029,9 @@ class CommandParser(object):
             return self.ParseTime()
 
         # Happens in function body, e.g. myfunc() oops
-        p_die('Unexpected word while parsing compound command',
-              loc.Word(self.cur_word))
+        p_die(
+            'Unexpected word while parsing compound command (%s)' %
+            Id_str(self.c_id), loc.Word(self.cur_word))
         assert False  # for MyPy
 
     def ParseFunctionDef(self):
@@ -2239,12 +2285,14 @@ class CommandParser(object):
 
                          # YSH extensions
                          | proc NAME ...
+                         | typed proc NAME ...
+                         | func NAME ...
                          | const ...
                          | var ...
                          | setglobal ...
                          | setref ...
                          | setvar ...
-                         | _ EXPR
+                         | call EXPR
                          | = EXPR
                          ;
 
@@ -2275,6 +2323,18 @@ class CommandParser(object):
                 p_die("proc is a YSH keyword, but this is OSH.",
                       loc.Word(self.cur_word))
 
+        if self.c_id == Id.KW_Typed:  # typed proc p () { ... }
+            self._SetNext()
+            self._GetWord()
+            if self.c_id != Id.KW_Proc:
+                p_die("Expected 'proc' after 'typed'", loc.Word(self.cur_word))
+
+            if self.parse_opts.parse_proc():
+                return self.ParseYshProc()
+            else:
+                p_die("typed is a YSH keyword, but this is OSH.",
+                      loc.Word(self.cur_word))
+
         if self.c_id == Id.KW_Func:  # func f(x) { ... }
             if self.parse_opts.parse_func():
                 return self.ParseYshFunc()
@@ -2351,12 +2411,12 @@ class CommandParser(object):
                 part0 = parts[0]
                 if part0.tag() == word_part_e.Literal:
                     tok = cast(Token, part0)
-                    if (match.IsValidVarName(tok.tval) and
+                    if (match.IsValidVarName(lexer.LazyStr(tok)) and
                             self.w_parser.LookPastSpace() == Id.Lit_Equals):
                         assert tok.id == Id.Lit_Chars, tok
 
-                        if len(self.hay_attrs_stack
-                               ) and self.hay_attrs_stack[-1]:
+                        if (len(self.hay_attrs_stack) and
+                                self.hay_attrs_stack[-1]):
                             # Note: no static var_checker.Check() for bare assignment
                             enode = self.w_parser.ParseBareDecl()
                             self._SetNext()  # Somehow this is necessary
@@ -2434,13 +2494,11 @@ class CommandParser(object):
     def ParseAndOr(self):
         # type: () -> command_t
         self._GetWord()
-        if self.c_id == Id.Word_Compound:
-            first_word_tok = word_.LiteralToken(self.cur_word)
-            if first_word_tok is not None and first_word_tok.id == Id.Lit_TDot:
-                # We got '...', so parse in multiline mode
-                self._SetNext()
-                with word_.ctx_Multiline(self.w_parser):
-                    return self._ParseAndOr()
+        if self.c_id == Id.Lit_TDot:
+            # We got '...', so parse in multiline mode
+            self._SetNext()
+            with word_.ctx_Multiline(self.w_parser):
+                return self._ParseAndOr()
 
         # Parse in normal mode, not multiline
         return self._ParseAndOr()
@@ -2536,8 +2594,9 @@ class CommandParser(object):
 
             else:
                 # e.g. echo a(b)
-                p_die('Invalid word while parsing command line',
-                      loc.Word(self.cur_word))
+                p_die(
+                    'Invalid word while parsing command line (%s)' %
+                    Id_str(self.c_id), loc.Word(self.cur_word))
 
             children.append(child)
 
@@ -2563,7 +2622,10 @@ class CommandParser(object):
           syntax_asdl.command
         """
         # Token types that will end the command term.
-        END_LIST = [self.eof_id, Id.Right_Subshell, Id.Lit_RBrace, Id.Op_DSemi]
+        END_LIST = [
+            self.eof_id, Id.Right_Subshell, Id.Lit_RBrace, Id.Op_DSemi,
+            Id.Op_SemiAmp, Id.Op_DSemiAmp
+        ]
 
         # NOTE: This is similar to _ParseCommandLine.
         #
@@ -2704,3 +2766,6 @@ class CommandParser(object):
             node = self.pending_here_docs[0]  # Just show the first one?
             h = cast(redir_param.HereDoc, node.arg)
             p_die('Unterminated here doc began here', loc.Word(h.here_begin))
+
+
+# vim: sw=4
