@@ -660,6 +660,19 @@ class VariablesAction(CompletionAction):
         f.write('VariablesAction ')
 
 
+class ExportedVarsAction(CompletionAction):
+    """compgen -e export."""
+
+    def __init__(self, mem):
+        # type: (Mem) -> None
+        self.mem = mem
+
+    def Matches(self, comp):
+        # type: (Api) -> Iterator[str]
+        for var_name in self.mem.GetExported():
+            yield var_name
+
+
 class ExternalCommandAction(CompletionAction):
     """Complete commands in $PATH.
 
@@ -1050,7 +1063,8 @@ class RootCompleter(object):
                 # readline splits at ':' so we have to prepend '-$' to every completed
                 # variable name.
                 self.comp_ui_state.display_pos = t2.col + 1  # 1 for $
-                to_complete = t2.tval[1:]
+                # computes s[1:] for Id.VSub_DollarName
+                to_complete = lexer.LazyStr(t2)
                 n = len(to_complete)
                 for name in self.mem.VarNames():
                     if name.startswith(to_complete):
@@ -1061,7 +1075,7 @@ class RootCompleter(object):
             # echo ${P
             if t2.id == Id.VSub_Name and IsDummy(t1):
                 self.comp_ui_state.display_pos = t2.col  # no offset
-                to_complete = t2.tval
+                to_complete = lexer.LazyStr(t2)
                 n = len(to_complete)
                 for name in self.mem.VarNames():
                     if name.startswith(to_complete):
@@ -1072,7 +1086,7 @@ class RootCompleter(object):
             # echo $(( VAR
             if t2.id == Id.Lit_ArithVarLike and IsDummy(t1):
                 self.comp_ui_state.display_pos = t2.col  # no offset
-                to_complete = t2.tval
+                to_complete = lexer.LazyStr(t2)
                 n = len(to_complete)
                 for name in self.mem.VarNames():
                     if name.startswith(to_complete):
