@@ -21,6 +21,7 @@ from core import error
 from core import process
 from core.error import e_die, e_die_status
 from core import pyos
+from core import pyutil
 from core import state
 from core import ui
 from core import vm
@@ -134,6 +135,7 @@ class ShellExecutor(vm._Executor):
         self.ext_prog = ext_prog
         self.waiter = waiter
         self.tracer = tracer
+        self.multi_trace = tracer.multi_trace
         self.job_control = job_control
         # sleep 5 & puts a (PID, job#) entry here.  And then "jobs" displays it.
         self.job_list = job_list
@@ -186,6 +188,7 @@ class ShellExecutor(vm._Executor):
         thunk = process.SubProgramThunk(self.cmd_ev,
                                         node,
                                         self.trap_state,
+                                        self.multi_trace,
                                         inherit_errexit=inherit_errexit)
         p = process.Process(thunk, self.job_control, self.job_list,
                             self.tracer)
@@ -201,17 +204,31 @@ class ShellExecutor(vm._Executor):
 
         builtin_func = self.builtins[builtin_id]
 
-        with vm.ctx_FlushStdout():
+        io_errors = []  # type: List[error.IOError_OSError]
+        with vm.ctx_FlushStdout(io_errors):
             # note: could be second word, like 'builtin read'
             with ui.ctx_Location(self.errfmt, cmd_val.arg_locs[0]):
                 try:
                     status = builtin_func.Run(cmd_val)
                     assert isinstance(status, int)
+                except (IOError, OSError) as e:
+                    self.errfmt.PrintMessage(
+                        '%s builtin I/O error: %s' %
+                        (cmd_val.argv[0], pyutil.strerror(e)),
+                        cmd_val.arg_locs[0])
+                    return 1
                 except error.Usage as e:
                     arg0 = cmd_val.argv[0]
                     # e.g. 'type' doesn't accept flag '-x'
                     self.errfmt.PrefixPrint(e.msg, '%r ' % arg0, e.location)
-                    status = 2  # consistent error code for usage error
+                    return 2  # consistent error code for usage error
+
+        if len(io_errors):  # e.g. disk full, ulimit
+            self.errfmt.PrintMessage(
+                '%s builtin I/O error: %s' %
+                (cmd_val.argv[0], pyutil.strerror(io_errors[0])),
+                cmd_val.arg_locs[0])
+            return 1
 
         return status
 
@@ -223,9 +240,6 @@ class ShellExecutor(vm._Executor):
         - YSH might have different, simpler rules.  No special builtins, etc.
         - YSH might have OILS_PATH = :| /bin /usr/bin | or something.
         - Interpreters might want to define all their own builtins.
-
-        Args:
-          call_procs: whether to look up procs.
         """
         argv = cmd_val.argv
         if len(cmd_val.arg_locs):
@@ -263,6 +277,11 @@ class ShellExecutor(vm._Executor):
         call_procs = not (run_flags & NO_CALL_PROCS)
         # Builtins like 'true' can be redefined as functions.
         if call_procs:
+            # TODO: Look shell functions in self.sh_funcs, but procs are
+            # value.Proc in the var namespace.
+            # Pitfall: What happens if there are two of the same name?  I guess
+            # that's why you have = and 'type' inspect them
+
             proc_node = self.procs.get(arg0)
             if proc_node is not None:
                 if self.exec_opts.strict_errexit():
@@ -321,7 +340,7 @@ class ShellExecutor(vm._Executor):
         else:
             argv0_path = self.search_path.CachedLookup(arg0)
         if argv0_path is None:
-            self.errfmt.Print_('%r not found' % arg0, arg0_loc)
+            self.errfmt.Print_('%r not found (OILS-ERR-100)' % arg0, arg0_loc)
             return 127
 
         # Normal case: ls /
@@ -683,3 +702,6 @@ class ShellExecutor(vm._Executor):
 
         # The CommandEvaluator could have a ProcessSubStack, which supports Push(),
         # Pop(), and Top() of VALUES rather than GC objects?
+
+
+# vim: sw=4
