@@ -2,7 +2,7 @@
 """expr_eval.py."""
 from __future__ import print_function
 
-from _devbuild.gen.id_kind_asdl import Id, Kind
+from _devbuild.gen.id_kind_asdl import Id
 from _devbuild.gen.syntax_asdl import (
     loc,
     loc_t,
@@ -10,7 +10,7 @@ from _devbuild.gen.syntax_asdl import (
     re_e,
     re_t,
     Token,
-    NameTok,
+    SimpleVarSub,
     word_part,
     SingleQuoted,
     DoubleQuoted,
@@ -27,11 +27,11 @@ from _devbuild.gen.syntax_asdl import (
     class_literal_term,
     class_literal_term_e,
     class_literal_term_t,
-    char_class_term,
     char_class_term_t,
     PosixClass,
     PerlClass,
     CharCode,
+    CharRange,
     ArgList,
     Eggex,
 )
@@ -42,6 +42,7 @@ from _devbuild.gen.runtime_asdl import (
     scope_t,
     part_value,
     part_value_t,
+    Piece,
 )
 from _devbuild.gen.value_asdl import (value, value_e, value_t, y_lvalue,
                                       y_lvalue_e, y_lvalue_t, IntBox, LeftName)
@@ -52,12 +53,11 @@ from core import pyutil
 from core import state
 from core import ui
 from core import vm
-from frontend import consts
+from data_lang import j8
 from frontend import lexer
 from frontend import match
 from frontend import typed_args
 from osh import braces
-from osh import word_compile
 from mycpp import mops
 from mycpp.mylib import log, NewDict, switch, tagswitch, print_stderr
 from ysh import func_proc
@@ -264,23 +264,64 @@ class ExprEvaluator(object):
             else:
                 raise AssertionError()
 
-    def _EvalLhsExpr(self, lhs):
-        # type: (y_lhs_t) -> y_lvalue_t
+    def _EvalLeftLocalOrGlobal(self, lhs, which_scopes):
+        # type: (expr_t, scope_t) -> value_t
+        """Evaluate the LEFT MOST part, respecting setvar/setglobal.
 
+        Consider this statement:
+
+            setglobal g[a[i]] = 42
+
+        - The g is always global, never local.  It's the thing to be mutated.
+        - The a can be local or global
+        """
+        UP_lhs = lhs
+        with tagswitch(lhs) as case:
+            if case(expr_e.Var):
+                lhs = cast(expr.Var, UP_lhs)
+
+                # respect setvar/setglobal with which_scopes
+                return LookupVar(self.mem, lhs.name, which_scopes, lhs.left)
+
+            elif case(expr_e.Subscript):
+                lhs = cast(Subscript, UP_lhs)
+
+                # recursive call
+                obj = self._EvalLeftLocalOrGlobal(lhs.obj, which_scopes)
+                index = self._EvalExpr(lhs.index)
+
+                return self._EvalSubscript(obj, index)
+
+            elif case(expr_e.Attribute):
+                lhs = cast(Attribute, UP_lhs)
+                assert lhs.op.id == Id.Expr_Dot
+
+                # recursive call
+                obj = self._EvalLeftLocalOrGlobal(lhs.obj, which_scopes)
+                return self._EvalDot(lhs, obj)
+
+            else:
+                # Shouldn't happen because of Transformer._CheckLhs
+                raise AssertionError()
+
+    def _EvalLhsExpr(self, lhs, which_scopes):
+        # type: (y_lhs_t, scope_t) -> y_lvalue_t
+        """
+        Handle setvar x, setvar a[i], ... setglobal x, setglobal a[i]
+        """
         UP_lhs = lhs
         with tagswitch(lhs) as case:
             if case(y_lhs_e.Var):
-                lhs = cast(NameTok, UP_lhs)
-                return LeftName(lhs.var_name, lhs.left)
+                lhs = cast(Token, UP_lhs)
+                return LeftName(lexer.LazyStr(lhs), lhs)
 
             elif case(y_lhs_e.Subscript):
                 lhs = cast(Subscript, UP_lhs)
                 # setvar mylist[0] = 42
                 # setvar mydict['key'] = 42
 
-                lval = self._EvalExpr(lhs.obj)
+                lval = self._EvalLeftLocalOrGlobal(lhs.obj, which_scopes)
                 index = self._EvalExpr(lhs.index)
-                #log('index %s', index)
                 return y_lvalue.Container(lval, index)
 
             elif case(y_lhs_e.Attribute):
@@ -288,7 +329,7 @@ class ExprEvaluator(object):
                 assert lhs.op.id == Id.Expr_Dot
 
                 # setvar mydict.key = 42
-                lval = self._EvalExpr(lhs.obj)
+                lval = self._EvalLeftLocalOrGlobal(lhs.obj, which_scopes)
 
                 attr = value.Str(lhs.attr_name)
                 return y_lvalue.Container(lval, attr)
@@ -305,11 +346,11 @@ class ExprEvaluator(object):
             val = self._EvalExpr(node)
         return val
 
-    def EvalLhsExpr(self, lhs):
-        # type: (y_lhs_t) -> y_lvalue_t
+    def EvalLhsExpr(self, lhs, which_scopes):
+        # type: (y_lhs_t, scope_t) -> y_lvalue_t
         """Public API for _EvalLhsExpr to ensure command_sub_errexit"""
         with state.ctx_YshExpr(self.mutable_opts):
-            lval = self._EvalLhsExpr(lhs)
+            lval = self._EvalLhsExpr(lhs, which_scopes)
         return lval
 
     def EvalExprSub(self, part):
@@ -320,7 +361,7 @@ class ExprEvaluator(object):
         with switch(part.left.id) as case:
             if case(Id.Left_DollarBracket):  # $[join(x)]
                 s = val_ops.Stringify(val, loc.WordPart(part))
-                return part_value.String(s, False, False)
+                return Piece(s, False, False)
 
             elif case(Id.Lit_AtLBracket):  # @[split(x)]
                 strs = val_ops.ToShellArray(val,
@@ -343,7 +384,7 @@ class ExprEvaluator(object):
             with state.ctx_Registers(self.mem):  # to sandbox globals
                 named_args = {}  # type: Dict[str, value_t]
                 arg_list = ArgList.CreateNull()  # There's no call site
-                rd = typed_args.Reader(pos_args, named_args, arg_list)
+                rd = typed_args.Reader(pos_args, named_args, None, arg_list)
 
                 try:
                     val = func_proc.CallUserFunc(func_val, rd, self.mem,
@@ -367,7 +408,7 @@ class ExprEvaluator(object):
             pos_args = [arg]
             named_args = {}  # type: Dict[str, value_t]
             arg_list = ArgList.CreateNull()  # There's no call site
-            rd = typed_args.Reader(pos_args, named_args, arg_list)
+            rd = typed_args.Reader(pos_args, named_args, None, arg_list)
             rd.SetFallbackLocation(convert_tok)
             try:
                 val = self._CallFunc(func_val, rd)
@@ -475,8 +516,8 @@ class ExprEvaluator(object):
 
         else:
             raise error.TypeErrVerbose(
-                'Binary operator expected numbers, got %s and %s' %
-                (ui.ValType(left), ui.ValType(right)), op)
+                'Binary operator expected numbers, got %s and %s (OILS-ERR-201)'
+                % (ui.ValType(left), ui.ValType(right)), op)
 
     def _ArithIntOnly(self, left, right, op):
         # type: (value_t, value_t, Token) -> value_t
@@ -783,7 +824,7 @@ class ExprEvaluator(object):
             if case(value_e.Func, value_e.BuiltinFunc):
                 to_call = func
                 pos_args, named_args = func_proc._EvalArgList(self, node.args)
-                rd = typed_args.Reader(pos_args, named_args, node.args)
+                rd = typed_args.Reader(pos_args, named_args, None, node.args)
 
             elif case(value_e.BoundFunc):
                 func = cast(value.BoundFunc, UP_func)
@@ -794,6 +835,7 @@ class ExprEvaluator(object):
                                                               me=func.me)
                 rd = typed_args.Reader(pos_args,
                                        named_args,
+                                       None,
                                        node.args,
                                        is_bound=True)
             else:
@@ -802,11 +844,8 @@ class ExprEvaluator(object):
 
         return self._CallFunc(to_call, rd)
 
-    def _EvalSubscript(self, node):
-        # type: (Subscript) -> value_t
-
-        obj = self._EvalExpr(node.obj)
-        index = self._EvalExpr(node.index)
+    def _EvalSubscript(self, obj, index):
+        # type: (value_t, value_t) -> value_t
 
         UP_obj = obj
         UP_index = index
@@ -880,6 +919,29 @@ class ExprEvaluator(object):
         raise error.TypeErr(obj, 'Subscript expected Str, List, or Dict',
                             loc.Missing)
 
+    def _EvalDot(self, node, obj):
+        # type: (Attribute, value_t) -> value_t
+        """ obj.attr on RHS or LHS
+
+        setvar x = obj.attr
+        setglobal g[obj.attr] = 42
+        """
+        UP_obj = obj
+        with tagswitch(obj) as case:
+            if case(value_e.Dict):
+                obj = cast(value.Dict, UP_obj)
+                attr_name = node.attr_name
+                try:
+                    result = obj.d[attr_name]
+                except KeyError:
+                    raise error.Expr('Dict entry %r not found' % attr_name,
+                                     node.op)
+
+            else:
+                raise error.TypeErr(obj, 'Dot operator expected Dict', node.op)
+
+        return result
+
     def _EvalAttribute(self, node):
         # type: (Attribute) -> value_t
 
@@ -927,21 +989,7 @@ class ExprEvaluator(object):
                             node.attr)
 
             elif case(Id.Expr_Dot):  # d.key is like d['key']
-                name = node.attr_name
-                with tagswitch(o) as case2:
-                    if case2(value_e.Dict):
-                        o = cast(value.Dict, UP_o)
-                        try:
-                            result = o.d[name]
-                        except KeyError:
-                            raise error.Expr('Dict entry %r not found' % name,
-                                             node.op)
-
-                    else:
-                        raise error.TypeErr(o, 'Dot operator expected Dict',
-                                            node.op)
-
-                return result
+                return self._EvalDot(node, o)
 
             else:
                 raise AssertionError(node.op)
@@ -980,8 +1028,16 @@ class ExprEvaluator(object):
                 else:
                     stdout_str = self.shell_ex.RunCommandSub(node)
                     if id_ == Id.Left_AtParen:  # @(seq 3)
-                        # TODO: Should use J8 lines
-                        strs = self.splitter.SplitForWordEval(stdout_str)
+                        # YSH splitting algorithm: does not depend on IFS
+                        try:
+                            strs = j8.SplitJ8Lines(stdout_str)
+                        except error.Decode as e:
+                            # status code 4 is special, for encode/decode errors.
+                            raise error.Structured(4, e.Message(),
+                                                   node.left_token)
+
+                        #strs = self.splitter.SplitForWordEval(stdout_str)
+
                         items = [value.Str(s)
                                  for s in strs]  # type: List[value_t]
                         return value.List(items)
@@ -1015,14 +1071,14 @@ class ExprEvaluator(object):
 
             elif case(expr_e.SingleQuoted):
                 node = cast(SingleQuoted, UP_node)
-                return value.Str(word_compile.EvalSingleQuoted(node))
+                return value.Str(node.sval)
 
             elif case(expr_e.BracedVarSub):
                 node = cast(BracedVarSub, UP_node)
                 return value.Str(self.word_ev.EvalBracedVarSubToString(node))
 
             elif case(expr_e.SimpleVarSub):
-                node = cast(NameTok, UP_node)
+                node = cast(SimpleVarSub, UP_node)
                 return value.Str(self.word_ev.EvalSimpleVarSubToString(node))
 
             elif case(expr_e.Unary):
@@ -1138,7 +1194,9 @@ class ExprEvaluator(object):
 
             elif case(expr_e.Subscript):
                 node = cast(Subscript, UP_node)
-                return self._EvalSubscript(node)
+                obj = self._EvalExpr(node.obj)
+                index = self._EvalExpr(node.index)
+                return self._EvalSubscript(obj, index)
 
             elif case(expr_e.Attribute):  # obj->method or mydict.key
                 node = cast(Attribute, UP_node)
@@ -1190,20 +1248,17 @@ class EggexEvaluator(object):
 
         with tagswitch(term) as case:
 
-            if case(class_literal_term_e.CharLiteral):
-                term = cast(class_literal_term.CharLiteral, UP_term)
+            if case(class_literal_term_e.CharCode):
+                term = cast(CharCode, UP_term)
 
-                # What about \0?
-                # At runtime, ERE should disallow it.  But we can also disallow it here.
-                out.append(word_compile.EvalCharLiteralForRegex(term.tok))
+                # What about \0?  At runtime, ERE should disallow it.  But we
+                # can also disallow it here.
+                out.append(term)
                 return
 
-            elif case(class_literal_term_e.Range):
-                term = cast(class_literal_term.Range, UP_term)
-
-                cp_start = word_compile.EvalCharLiteralForRegex(term.start)
-                cp_end = word_compile.EvalCharLiteralForRegex(term.end)
-                out.append(char_class_term.Range(cp_start, cp_end))
+            elif case(class_literal_term_e.CharRange):
+                term = cast(CharRange, UP_term)
+                out.append(term)
                 return
 
             elif case(class_literal_term_e.PosixClass):
@@ -1219,7 +1274,7 @@ class EggexEvaluator(object):
             elif case(class_literal_term_e.SingleQuoted):
                 term = cast(SingleQuoted, UP_term)
 
-                s = word_compile.EvalSingleQuoted(term)
+                s = term.sval
                 char_code_tok = term.left
 
             elif case(class_literal_term_e.Splice):
@@ -1239,7 +1294,7 @@ class EggexEvaluator(object):
                     "Use unquoted char literal for byte %d, which is >= 128"
                     " (avoid confusing a set of bytes with a sequence)" %
                     char_int, char_code_tok)
-            out.append(CharCode(char_int, False, char_code_tok))
+            out.append(CharCode(char_code_tok, char_int, False))
 
     def EvalE(self, node):
         # type: (re_t) -> re_t
@@ -1280,7 +1335,7 @@ class EggexEvaluator(object):
                 convert_func = None  # type: Optional[value_t]
                 convert_tok = None  # type: Optional[Token]
                 if node.func_name:
-                    func_name = lexer.TokenVal(node.func_name)
+                    func_name = lexer.LazyStr(node.func_name)
                     func_val = self.mem.GetValue(func_name)
                     with tagswitch(func_val) as case:
                         if case(value_e.Func, value_e.BuiltinFunc):
@@ -1307,44 +1362,11 @@ class EggexEvaluator(object):
                     self._EvalClassLiteralTerm(t, new_terms)
                 return re.CharClass(node.negated, new_terms)
 
-            elif case(re_e.Token):
-                node = cast(Token, UP_node)
-
-                id_ = node.id
-                tval = node.tval
-
-                if id_ == Id.Expr_Dot:
-                    return re.Primitive(Id.Re_Dot)
-
-                if id_ == Id.Arith_Caret:  # ^
-                    return re.Primitive(Id.Re_Start)
-
-                if id_ == Id.Expr_Dollar:  # $
-                    return re.Primitive(Id.Re_End)
-
-                if id_ == Id.Expr_Name:
-                    if tval == 'dot':
-                        return re.Primitive(Id.Re_Dot)
-                    raise NotImplementedError(tval)
-
-                if id_ == Id.Expr_Symbol:
-                    if tval == '%start':
-                        return re.Primitive(Id.Re_Start)
-                    if tval == '%end':
-                        return re.Primitive(Id.Re_End)
-                    raise NotImplementedError(tval)
-
-                # Must be Id.Char_{OneChar,Hex,Unicode4,Unicode8}
-                kind = consts.GetKind(id_)
-                assert kind == Kind.Char, id_
-                s = word_compile.EvalCStringToken(node)
-                return re.LiteralChars(s, node)
-
             elif case(re_e.SingleQuoted):
                 node = cast(SingleQuoted, UP_node)
 
-                s = word_compile.EvalSingleQuoted(node)
-                return re.LiteralChars(s, node.left)
+                s = node.sval
+                return re.LiteralChars(node.left, s)
 
             elif case(re_e.Splice):
                 node = cast(re.Splice, UP_node)
@@ -1354,8 +1376,8 @@ class EggexEvaluator(object):
                 with tagswitch(val) as case:
                     if case(value_e.Str):
                         val = cast(value.Str, UP_val)
-                        to_splice = re.LiteralChars(val.s,
-                                                    node.name)  # type: re_t
+                        to_splice = re.LiteralChars(node.name,
+                                                    val.s)  # type: re_t
 
                     elif case(value_e.Eggex):
                         val = cast(value.Eggex, UP_val)
@@ -1383,6 +1405,7 @@ class EggexEvaluator(object):
             else:
                 # These are evaluated at translation time
 
+                # case(re_e.Primitive)
                 # case(re_e.PosixClass)
                 # case(re_e.PerlClass)
                 return node
