@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
-"""Flag_def.py."""
+"""Flag parser defintions."""
+
 from __future__ import print_function
 
 from frontend import args
@@ -116,6 +117,7 @@ READ_SPEC.ShortFlag('-p', args.String)  # prompt
 READ_SPEC.ShortFlag('-0')  # until NUL, like -r -d ''
 READ_SPEC.LongFlag('--all')
 READ_SPEC.LongFlag('--line')
+READ_SPEC.LongFlag('--num-bytes', args.Int)
 # don't strip the trailing newline
 READ_SPEC.LongFlag('--with-eol')
 READ_SPEC.LongFlag('--json',
@@ -176,6 +178,26 @@ JOB_SPEC.ShortFlag('-l', help='long format')
 JOB_SPEC.ShortFlag('-p', help='prints PID only')
 JOB_SPEC.LongFlag('--debug', help='display debug info')
 
+ULIMIT_SPEC = FlagSpec('ulimit')
+
+ULIMIT_SPEC.ShortFlag('-a', help='Print all limits')
+ULIMIT_SPEC.LongFlag('--all', help='Alias for -a')
+ULIMIT_SPEC.ShortFlag('-H', help='Use hard limit')
+ULIMIT_SPEC.ShortFlag('-S', help='Use soft limit')
+
+_ULIMIT_RESOURCES = [
+    '-c',
+    '-d',
+    '-f',
+    '-n',
+    '-s',
+    '-t',
+    '-v',
+]
+
+for u_flag in _ULIMIT_RESOURCES:
+    ULIMIT_SPEC.ShortFlag(u_flag)
+
 #
 # FlagSpecAndMore
 #
@@ -330,7 +352,9 @@ def _DefineCompletionActions(spec):
     spec.Action('b', 'binding')
     spec.Action('c', 'command')
     spec.Action('d', 'directory')
+    spec.Action('e', 'export')
     spec.Action('f', 'file')
+    spec.Action('k', 'keyword')
     spec.Action('j', 'job')
     spec.Action('u', 'user')
     spec.Action('v', 'variable')
@@ -480,15 +504,6 @@ PUSH_REGISTERS_SPEC = FlagSpec('push-registers')
 
 FOPEN_SPEC = FlagSpec('fopen')
 
-#
-# Tea
-#
-
-TEA_MAIN_SPEC = FlagSpec('tea_main')
-TEA_MAIN_SPEC.ShortFlag('-n', args.Bool)  # Parse
-TEA_MAIN_SPEC.ShortFlag('-c', args.String)  # Command snippet
-TEA_MAIN_SPEC.LongFlag('--translate', args.Bool)
-
 #
 # JSON
 #
@@ -502,6 +517,7 @@ JSON_WRITE_SPEC.LongFlag('--pretty',
                          default=True,
                          help='Whitespace in output (default true)')
 
+# Unused:
 # JSON has the questionable decision of allowing (unpaired) surrogate like
 # \udc00.
 # When encoding, we try to catch the error on OUR side, rather than letting it
@@ -518,8 +534,3 @@ JSON_WRITE_SPEC.LongFlag('--indent',
                          help='Indent JSON by this amount')
 
 JSON_READ_SPEC = FlagSpec('json_read')
-# yajl has this option
-JSON_READ_SPEC.LongFlag('--validate',
-                        args.Bool,
-                        default=True,
-                        help='Validate UTF-8')
