@@ -1,33 +1,41 @@
 TOPICS = {'ARGV': 'special-var',
  'BASHOPTS': 'special-var',
  'BASHPID': 'special-var',
+ 'BASH_LINENO': 'special-var',
  'BASH_REMATCH': 'special-var',
- 'Better-Syntax': 'builtin-func',
- 'Bool': 'packle',
- 'Bytes': 'packle',
+ 'BASH_SOURCE': 'special-var',
+ 'BashArray': 'type-method',
+ 'BashAssoc': 'type-method',
+ 'Bool': 'type-method',
+ 'BoundFunc': 'type-method',
+ 'BuiltinFunc': 'type-method',
  'CDPATH': 'special-var',
  'COMPREPLY': 'special-var',
  'COMP_ARGV': 'special-var',
  'COMP_CWORD': 'special-var',
  'COMP_LINE': 'special-var',
  'COMP_POINT': 'special-var',
+ 'COMP_WORDBREAKS': 'special-var',
  'COMP_WORDS': 'special-var',
+ 'Command': 'type-method',
  'DEBUG': 'plugin',
- 'Dict': 'packle',
  'ENV': 'special-var',
  'ERR': 'plugin',
  'EUID': 'special-var',
  'EXIT': 'plugin',
- 'Float': 'packle',
+ 'Expr': 'type-method',
+ 'FUNCNAME': 'special-var',
+ 'Float': 'type-method',
  'GLOBIGNORE': 'special-var',
  'HISTFILE': 'special-var',
  'HOME': 'special-var',
+ 'HOSTNAME': 'special-var',
  'IFS': 'special-var',
- 'Int': 'packle',
+ 'Int': 'type-method',
  'LANG': 'special-var',
  'LINENO': 'special-var',
- 'List': 'packle',
- 'Null': 'packle',
+ 'List/append': 'type-method',
+ 'Null': 'type-method',
  'OILS_GC_ON_EXIT': 'special-var',
  'OILS_GC_STATS': 'special-var',
  'OILS_GC_STATS_FD': 'special-var',
@@ -37,9 +45,11 @@ TOPICS = {'ARGV': 'special-var',
  'OPTARG': 'special-var',
  'OPTERR': 'special-var',
  'OPTIND': 'special-var',
+ 'OSTYPE': 'special-var',
  'PATH': 'special-var',
  'PIPESTATUS': 'special-var',
  'PPID': 'special-var',
+ 'PROMPT_COMMAND': 'plugin',
  'PS1': 'plugin',
  'PS2': 'plugin',
  'PS3': 'plugin',
@@ -53,38 +63,46 @@ TOPICS = {'ARGV': 'special-var',
  'SHX_indent': 'special-var',
  'SHX_pid_str': 'special-var',
  'SHX_punct': 'special-var',
+ 'SIGINT': 'plugin',
+ 'SIGQUIT': 'plugin',
+ 'SIGTERM': 'plugin',
+ 'SIGTTIN': 'plugin',
+ 'SIGTTOU': 'plugin',
+ 'SIGWINCH': 'plugin',
  'UID': 'special-var',
- 'Unicode': 'packle',
  'YSH_HISTFILE': 'special-var',
- '_end()': 'builtin-func',
+ '_end': 'builtin-func',
  '_error': 'special-var',
- '_group()': 'builtin-func',
+ '_group': 'builtin-func',
  '_pipeline_status': 'special-var',
  '_process_sub_status': 'special-var',
  '_reply': 'special-var',
- '_start()': 'builtin-func',
+ '_start': 'builtin-func',
  '_status': 'special-var',
  '_this_dir': 'special-var',
- 'abs()': 'builtin-func',
- 'accum()': 'type-method',
+ 'abs': 'builtin-func',
+ 'accum': 'type-method',
  'alias': 'builtin-cmd',
- 'all()': 'builtin-func',
+ 'all': 'builtin-func',
  'ampersand': 'cmd-lang',
  'and': 'cmd-lang',
- 'any()': 'builtin-func',
- 'append': 'builtin-cmd',
- 'append()': 'type-method',
+ 'any': 'builtin-func',
+ 'append': 'index',
  'arg': 'builtin-cmd',
  'arith-context': 'mini-lang',
  'arith-sub': 'word-lang',
+ 'ascii-control-chars': 'front-end',
+ 'ascii-whitespace': 'front-end',
+ 'assign': 'expr-lang',
+ 'atom-literal': 'expr-lang',
+ 'aug-assign': 'expr-lang',
  'b-prefix': 'j8',
  'bang': 'cmd-lang',
  'block-arg': 'cmd-lang',
- 'block-literal': 'expr-lang',
- 'bool()': 'builtin-func',
+ 'block-expr': 'expr-lang',
+ 'bool': 'builtin-func',
  'bool-expr': 'mini-lang',
  'bool-infix': 'mini-lang',
- 'bool-literal': 'expr-lang',
  'bool-other': 'mini-lang',
  'bool-path': 'mini-lang',
  'bool-str': 'mini-lang',
@@ -92,120 +110,135 @@ TOPICS = {'ARGV': 'special-var',
  'braces': 'mini-lang',
  'break': 'cmd-lang',
  'builtin': 'builtin-cmd',
- 'call': 'cmd-lang',
- 'captureStdout()': 'type-method',
+ 'call': 'ysh-cmd',
+ 'captureStdout': 'type-method',
  'case': 'cmd-lang',
  'cat-em': 'front-end',
  'cd': 'builtin-cmd',
  'char-escapes': 'mini-lang',
- 'chr()': 'builtin-func',
+ 'chr': 'builtin-func',
  'class-literal': 'expr-lang',
+ 'cmd/append': 'builtin-cmd',
+ 'cmd/type': 'builtin-cmd',
  'colon': 'cmd-lang',
  'column-attrs': 'j8',
  'column-types': 'j8',
- 'com-sub': 'word-lang',
  'command': 'builtin-cmd',
- 'command_sub_errexit': 'option',
+ 'command-sub': 'word-lang',
  'comment': 'front-end',
  'compadjust': 'builtin-cmd',
  'compexport': 'builtin-cmd',
  'compgen': 'builtin-cmd',
- 'complete': 'builtin-cmd',
+ 'complete': 'plugin',
  'compopt': 'builtin-cmd',
  'concat': 'expr-lang',
  'config': 'front-end',
- 'const': 'cmd-lang',
+ 'const': 'ysh-cmd',
  'continue': 'cmd-lang',
- 'copy_env': 'option',
  'ctx': 'builtin-cmd',
  'dashglob': 'option',
- 'dbracket': 'mini-lang',
+ 'dbracket': 'cmd-lang',
  'declare': 'osh-assign',
- 'dict()': 'builtin-func',
+ 'dict': 'builtin-func',
  'dict-literal': 'expr-lang',
  'dirs': 'builtin-cmd',
  'doc-comment': 'front-end',
  'dparen': 'cmd-lang',
  'echo': 'builtin-cmd',
  'enable': 'builtin-cmd',
- 'end()': 'type-method',
- 'endsWith()': 'type-method',
- 'equal': 'cmd-lang',
- 'erase()': 'type-method',
+ 'end': 'type-method',
+ 'endsWith': 'type-method',
+ 'equal': 'ysh-cmd',
+ 'erase': 'type-method',
+ 'err-j8-lines-decode': 'errors',
+ 'err-j8-lines-encode': 'errors',
+ 'err-j8-str-decode': 'errors',
+ 'err-j8-str-encode': 'errors',
+ 'err-json-decode': 'errors',
+ 'err-json-encode': 'errors',
+ 'err-json8-decode': 'errors',
+ 'err-json8-encode': 'errors',
+ 'err-utf8-decode': 'errors',
+ 'err-utf8-encode': 'errors',
  'error': 'builtin-cmd',
- 'eval': 'builtin-cmd',
- 'eval()': 'type-method',
+ 'eval': 'type-method',
+ 'evalExpr': 'builtin-func',
+ 'evalHay': 'builtin-func',
+ 'eval_unsafe_arith': 'option',
  'exec': 'builtin-cmd',
  'exit': 'cmd-lang',
  'exit-codes': 'front-end',
  'export': 'osh-assign',
- 'expr-lit': 'expr-lang',
+ 'expr-literal': 'expr-lang',
  'expr-splice': 'word-lang',
  'expr-sub': 'word-lang',
- 'extend()': 'type-method',
+ 'extend': 'type-method',
  'extglob': 'mini-lang',
  'false': 'cmd-lang',
  'fat-arrow': 'expr-lang',
  'fg': 'builtin-cmd',
- 'find()': 'type-method',
+ 'find': 'type-method',
  'flag': 'builtin-cmd',
- 'float()': 'builtin-func',
+ 'float': 'builtin-func',
+ 'float-lit': 'expr-lang',
+ 'fopen': 'builtin-cmd',
  'for': 'cmd-lang',
  'for-expr-sh': 'cmd-lang',
  'fork': 'builtin-cmd',
  'forkwait': 'builtin-cmd',
- 'fromJson()': 'builtin-func',
- 'fromJson8()': 'builtin-func',
+ 'fromJson': 'builtin-func',
+ 'fromJson8': 'builtin-func',
+ 'func': 'ysh-cmd',
  'func-call': 'expr-lang',
- 'func-def': 'cmd-lang',
- 'get()': 'type-method',
+ 'func/type': 'builtin-func',
+ 'get': 'type-method',
+ 'getVar': 'builtin-func',
  'getopts': 'builtin-cmd',
- 'glob()': 'type-method',
+ 'glob': 'type-method',
  'glob-pat': 'mini-lang',
- 'group()': 'type-method',
+ 'group': 'type-method',
  'hash': 'builtin-cmd',
+ 'hay': 'builtin-cmd',
+ 'haynode': 'builtin-cmd',
  'help': None,
  'here-doc': 'cmd-lang',
  'history': 'builtin-cmd',
  'histsub': 'mini-lang',
  'if': 'cmd-lang',
- 'inc()': 'type-method',
- 'indexOf()': 'type-method',
- 'insert()': 'type-method',
- 'int()': 'builtin-func',
+ 'inc': 'type-method',
+ 'indexOf': 'type-method',
+ 'insert': 'type-method',
+ 'int': 'builtin-func',
  'int-literal': 'expr-lang',
  'is-main': 'builtin-cmd',
  'j8-escape': 'j8',
  'jobs': 'builtin-cmd',
- 'join()': 'builtin-func',
+ 'join': 'builtin-func',
  'json': 'builtin-cmd',
- 'json-decode-err': 'errors',
- 'json-encode-err': 'errors',
  'json-escape': 'j8',
  'json-string': 'j8',
  'json8': 'builtin-cmd',
  'json8-comment': 'j8',
- 'json8-decode-err': 'errors',
  'json8-dict': 'j8',
- 'json8-encode-err': 'errors',
  'json8-list': 'j8',
  'json8-num': 'j8',
  'json8-str': 'j8',
- 'keys()': 'type-method',
+ 'keys': 'type-method',
  'kill': 'builtin-cmd',
  'lazy-expr-arg': 'cmd-lang',
- 'leftMatch()': 'type-method',
- 'len()': 'builtin-func',
+ 'leftMatch': 'type-method',
+ 'len': 'builtin-func',
  'line-continuation': 'front-end',
- 'list()': 'builtin-func',
+ 'line-editing': 'front-end',
+ 'list': 'builtin-func',
  'list-literal': 'expr-lang',
  'local': 'osh-assign',
- 'lower()': 'type-method',
+ 'lower': 'type-method',
  'mapfile': 'builtin-cmd',
  'match-ops': 'expr-lang',
- 'max()': 'builtin-func',
- 'maybe()': 'builtin-func',
- 'min()': 'builtin-func',
+ 'max': 'builtin-func',
+ 'maybe': 'builtin-func',
+ 'min': 'builtin-func',
  'module': 'builtin-cmd',
  'multiline-command': 'front-end',
  'named-class': 'expr-lang',
@@ -214,34 +247,32 @@ TOPICS = {'ARGV': 'special-var',
  'oils-usage': None,
  'op-format': 'word-lang',
  'op-index': 'word-lang',
+ 'op-precedence': 'expr-lang',
  'op-replace': 'word-lang',
  'op-slice': 'word-lang',
  'op-strip': 'word-lang',
  'op-test': 'word-lang',
+ 'opts-internal': 'option',
+ 'opts-redefine': 'option',
  'or': 'cmd-lang',
- 'ord()': 'builtin-func',
+ 'ord': 'builtin-func',
  'osh-chapters': None,
  'osh-string': 'word-lang',
  'osh-usage': None,
- 'packle-decode-err': 'errors',
- 'packle-encode-err': 'errors',
- 'parseArgs()': 'builtin-cmd',
- 'parse_at': 'option',
- 'parse_brace': 'option',
- 'parse_equals': 'option',
+ 'parseArgs': 'builtin-cmd',
+ 'parseHay': 'builtin-func',
  'parse_ignored': 'option',
- 'parse_paren': 'option',
  'parse_triple_quote': 'option',
  'parse_ysh_string': 'option',
  'parser': 'builtin-cmd',
  'pipe': 'cmd-lang',
- 'pop()': 'type-method',
+ 'pop': 'type-method',
  'popd': 'builtin-cmd',
  'pp': 'builtin-cmd',
  'printf': 'builtin-cmd',
- 'proc-def': 'cmd-lang',
- 'process_sub_fail': 'option',
- 'promptVal()': 'type-method',
+ 'proc': 'ysh-cmd',
+ 'proc-sub': 'word-lang',
+ 'promptVal': 'type-method',
  'push-registers': 'builtin-cmd',
  'pushd': 'builtin-cmd',
  'pwd': 'builtin-cmd',
@@ -256,26 +287,26 @@ TOPICS = {'ARGV': 'special-var',
  're-splice': 'expr-lang',
  'read': 'builtin-cmd',
  'readarray': 'builtin-cmd',
+ 'readonly': 'osh-assign',
  'redir-desc': 'cmd-lang',
  'redir-file': 'cmd-lang',
  'regex': 'mini-lang',
- 'remove()': 'type-method',
- 'renderPrompt()': 'plugin',
- 'repeat()': 'builtin-func',
- 'replace()': 'type-method',
+ 'remove': 'type-method',
+ 'renderPrompt': 'plugin',
+ 'repeat': 'builtin-func',
+ 'replace': 'type-method',
  'rest': 'builtin-cmd',
  'return': 'cmd-lang',
- 'reverse()': 'type-method',
- 'round()': 'builtin-func',
- 'rune-literal': 'expr-lang',
- 'runes()': 'builtin-func',
+ 'reverse': 'type-method',
+ 'round': 'builtin-func',
+ 'runes': 'builtin-func',
  'runproc': 'builtin-cmd',
- 'search()': 'type-method',
+ 'search': 'type-method',
  'semicolon': 'cmd-lang',
  'set': 'builtin-cmd',
- 'setValue()': 'type-method',
- 'setglobal': 'cmd-lang',
- 'setvar': 'cmd-lang',
+ 'setValue': 'type-method',
+ 'setglobal': 'ysh-cmd',
+ 'setvar': 'ysh-cmd',
  'sh-append': 'osh-assign',
  'sh-arith': 'mini-lang',
  'sh-array': 'osh-assign',
@@ -286,81 +317,83 @@ TOPICS = {'ARGV': 'special-var',
  'sh-func': 'cmd-lang',
  'sh-logical': 'mini-lang',
  'sh-numbers': 'mini-lang',
- 'shSplit()': 'builtin-func',
+ 'shSplit': 'builtin-func',
+ 'sha1dc': 'builtin-func',
+ 'sha256': 'builtin-func',
  'shift': 'osh-assign',
  'shopt': 'builtin-cmd',
  'shvar': 'builtin-cmd',
- 'shvar_get()': 'builtin-func',
+ 'shvarGet': 'builtin-func',
  'sigpipe_status_ok': 'option',
  'simple-command': 'cmd-lang',
- 'simple_word_eval': 'option',
  'source': 'builtin-cmd',
- 'split()': 'builtin-func',
- 'start()': 'type-method',
- 'startsWith()': 'type-method',
+ 'split': 'builtin-func',
+ 'start': 'type-method',
+ 'startsWith': 'type-method',
  'startup': 'front-end',
- 'str()': 'builtin-func',
+ 'str': 'builtin-func',
  'str-template': 'expr-lang',
- 'strcmp()': 'builtin-func',
- 'strftime()': 'type-method',
+ 'strcmp': 'builtin-func',
+ 'strftime': 'type-method',
  'strict:all': 'option',
  'strict_control_flow': 'option',
  'strict_nameref': 'option',
  'strict_tilde': 'option',
- 'strict_word_eval': 'option',
  'subshell': 'cmd-lang',
- 'sum()': 'builtin-func',
+ 'sum': 'builtin-func',
  'surrogate-pair': 'j8',
  'tagged-str': 'word-lang',
  'test': 'builtin-cmd',
  'thin-arrow': 'expr-lang',
  'tilde-sub': 'word-lang',
- 'time': 'cmd-lang',
- 'time()': 'type-method',
+ 'time': 'type-method',
  'times': 'builtin-cmd',
- 'toJson()': 'builtin-func',
- 'toJson8()': 'builtin-func',
+ 'toJson': 'builtin-func',
+ 'toJson8': 'builtin-func',
  'trap': 'builtin-cmd',
- 'trim()': 'type-method',
- 'trimEnd()': 'type-method',
- 'trimStart()': 'type-method',
+ 'trim': 'type-method',
+ 'trimEnd': 'type-method',
+ 'trimStart': 'type-method',
  'triple-quoted': 'word-lang',
  'true': 'cmd-lang',
  'try': 'builtin-cmd',
- 'type': 'builtin-cmd',
- 'type()': 'builtin-func',
+ 'type': 'index',
  'typed-arg': 'cmd-lang',
  'typeset': 'osh-assign',
  'u-prefix': 'j8',
+ 'ulimit': 'builtin-cmd',
  'umask': 'builtin-cmd',
  'unalias': 'builtin-cmd',
+ 'unquoted-line': 'j8',
  'unset': 'osh-assign',
  'until': 'cmd-lang',
- 'upper()': 'type-method',
+ 'upper': 'type-method',
  'use': 'builtin-cmd',
- 'utf8-decode-err': 'errors',
- 'utf8-encode-err': 'errors',
- 'values()': 'type-method',
- 'var': 'cmd-lang',
+ 'values': 'type-method',
+ 'var': 'ysh-cmd',
  'var-splice': 'word-lang',
  'var-sub': 'word-lang',
  'wait': 'builtin-cmd',
  'while': 'cmd-lang',
  'write': 'builtin-cmd',
  'ysh-arith': 'expr-lang',
+ 'ysh-attr': 'expr-lang',
  'ysh-bitwise': 'expr-lang',
  'ysh-case': 'cmd-lang',
  'ysh-cd': 'builtin-cmd',
  'ysh-chapters': None,
  'ysh-compare': 'expr-lang',
+ 'ysh-echo': 'builtin-cmd',
+ 'ysh-equals': 'expr-lang',
  'ysh-for': 'cmd-lang',
  'ysh-format': 'word-lang',
  'ysh-if': 'cmd-lang',
+ 'ysh-in': 'expr-lang',
  'ysh-index': 'expr-lang',
  'ysh-logical': 'expr-lang',
  'ysh-printf': 'word-lang',
  'ysh-read': 'builtin-cmd',
- 'ysh-return': 'cmd-lang',
+ 'ysh-return': 'ysh-cmd',
  'ysh-shopt': 'builtin-cmd',
  'ysh-slice': 'expr-lang',
  'ysh-string': 'word-lang',
