@@ -40,20 +40,23 @@ class cmd_value_t(pybase.CompoundObj):
 class cmd_value(object):
   class Argv(cmd_value_t):
     _type_tag = 1
-    __slots__ = ('argv', 'arg_locs', 'typed_args', 'pos_args', 'named_args')
+    __slots__ = ('argv', 'arg_locs', 'typed_args', 'pos_args', 'named_args',
+                 'block_arg')
   
-    def __init__(self, argv, arg_locs, typed_args, pos_args, named_args):
-      # type: (List[str], List[CompoundWord], Optional[ArgList], Optional[List[value_t]], Optional[Dict[str, value_t]]) -> None
+    def __init__(self, argv, arg_locs, typed_args, pos_args, named_args,
+                 block_arg):
+      # type: (List[str], List[CompoundWord], Optional[ArgList], Optional[List[value_t]], Optional[Dict[str, value_t]], Optional[value_t]) -> None
       self.argv = argv
       self.arg_locs = arg_locs
       self.typed_args = typed_args
       self.pos_args = pos_args
       self.named_args = named_args
+      self.block_arg = block_arg
   
     @staticmethod
     def CreateNull(alloc_lists=False):
       # type: () -> cmd_value.Argv
-      return cmd_value.Argv([] if alloc_lists else cast('List[str]', None), [] if alloc_lists else cast('List[CompoundWord]', None), cast('Optional[ArgList]', None), cast('Optional[List[value_t]]', None), cast('Optional[Dict[str, value_t]]', None))
+      return cmd_value.Argv([] if alloc_lists else cast('List[str]', None), [] if alloc_lists else cast('List[CompoundWord]', None), cast('Optional[ArgList]', None), cast('Optional[List[value_t]]', None), cast('Optional[Dict[str, value_t]]', None), cast('Optional[value_t]', None))
   
     def PrettyTree(self, trav=None):
       # type: (Optional[TraversalState]) -> hnode_t
@@ -99,6 +102,10 @@ class cmd_value(object):
           x4.children.append(v4.PrettyTree(trav=trav))
         L.append(Field('named_args', x4))
   
+      if self.block_arg is not None:  # Optional
+        x5 = self.block_arg.PrettyTree(trav=trav)
+        L.append(Field('block_arg', x5))
+  
       return out_node
   
     def _AbbreviatedTree(self, trav=None):
@@ -144,6 +151,10 @@ class cmd_value(object):
           x4.children.append(v4.AbbreviatedTree(trav=trav))
         L.append(Field('named_args', x4))
   
+      if self.block_arg is not None:  # Optional
+        x5 = self.block_arg.AbbreviatedTree(trav=trav)
+        L.append(Field('block_arg', x5))
+  
       return out_node
   
     def AbbreviatedTree(self, trav=None):
@@ -246,14 +257,14 @@ class cmd_value(object):
   pass
 
 class part_value_e(object):
-  String = 1
+  String = 65
   Array = 2
   ExtGlob = 3
 
 _part_value_str = {
-  1: 'String',
   2: 'Array',
   3: 'ExtGlob',
+  65: 'String',
 }
 
 def part_value_str(tag, dot=True):
@@ -271,66 +282,6 @@ class part_value_t(pybase.CompoundObj):
   pass
 
 class part_value(object):
-  class String(part_value_t):
-    _type_tag = 1
-    __slots__ = ('s', 'quoted', 'do_split')
-  
-    def __init__(self, s, quoted, do_split):
-      # type: (str, bool, bool) -> None
-      self.s = s
-      self.quoted = quoted
-      self.do_split = do_split
-  
-    @staticmethod
-    def CreateNull(alloc_lists=False):
-      # type: () -> part_value.String
-      return part_value.String('', False, False)
-  
-    def PrettyTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('part_value.String')
-      L = out_node.fields
-  
-      x0 = NewLeaf(self.s, color_e.StringConst)
-      L.append(Field('s', x0))
-  
-      x1 = hnode.Leaf('T' if self.quoted else 'F', color_e.OtherConst)
-      L.append(Field('quoted', x1))
-  
-      x2 = hnode.Leaf('T' if self.do_split else 'F', color_e.OtherConst)
-      L.append(Field('do_split', x2))
-  
-      return out_node
-  
-    def _AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('part_value.String')
-      L = out_node.fields
-      x0 = NewLeaf(self.s, color_e.StringConst)
-      L.append(Field('s', x0))
-  
-      x1 = hnode.Leaf('T' if self.quoted else 'F', color_e.OtherConst)
-      L.append(Field('quoted', x1))
-  
-      x2 = hnode.Leaf('T' if self.do_split else 'F', color_e.OtherConst)
-      L.append(Field('do_split', x2))
-  
-      return out_node
-  
-    def AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      return self._AbbreviatedTree(trav=trav)
-  
   class Array(part_value_t):
     _type_tag = 2
     __slots__ = ('strs',)
@@ -1589,19 +1540,80 @@ class AssignArg(pybase.CompoundObj):
     # type: (Optional[TraversalState]) -> hnode_t
     return self._AbbreviatedTree(trav=trav)
 
-class VarSubState(pybase.CompoundObj):
+class Piece(part_value_t):
   _type_tag = 65
-  __slots__ = ('join_array', 'is_type_query')
+  __slots__ = ('s', 'quoted', 'do_split')
 
-  def __init__(self, join_array, is_type_query):
-    # type: (bool, bool) -> None
+  def __init__(self, s, quoted, do_split):
+    # type: (str, bool, bool) -> None
+    self.s = s
+    self.quoted = quoted
+    self.do_split = do_split
+
+  @staticmethod
+  def CreateNull(alloc_lists=False):
+    # type: () -> Piece
+    return Piece('', False, False)
+
+  def PrettyTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('Piece')
+    L = out_node.fields
+
+    x0 = NewLeaf(self.s, color_e.StringConst)
+    L.append(Field('s', x0))
+
+    x1 = hnode.Leaf('T' if self.quoted else 'F', color_e.OtherConst)
+    L.append(Field('quoted', x1))
+
+    x2 = hnode.Leaf('T' if self.do_split else 'F', color_e.OtherConst)
+    L.append(Field('do_split', x2))
+
+    return out_node
+
+  def _AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('Piece')
+    L = out_node.fields
+    x0 = NewLeaf(self.s, color_e.StringConst)
+    L.append(Field('s', x0))
+
+    x1 = hnode.Leaf('T' if self.quoted else 'F', color_e.OtherConst)
+    L.append(Field('quoted', x1))
+
+    x2 = hnode.Leaf('T' if self.do_split else 'F', color_e.OtherConst)
+    L.append(Field('do_split', x2))
+
+    return out_node
+
+  def AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    return self._AbbreviatedTree(trav=trav)
+
+class VarSubState(pybase.CompoundObj):
+  _type_tag = 66
+  __slots__ = ('join_array', 'is_type_query', 'has_test_op')
+
+  def __init__(self, join_array, is_type_query, has_test_op):
+    # type: (bool, bool, bool) -> None
     self.join_array = join_array
     self.is_type_query = is_type_query
+    self.has_test_op = has_test_op
 
   @staticmethod
   def CreateNull(alloc_lists=False):
     # type: () -> VarSubState
-    return VarSubState(False, False)
+    return VarSubState(False, False, False)
 
   def PrettyTree(self, trav=None):
     # type: (Optional[TraversalState]) -> hnode_t
@@ -1619,6 +1631,9 @@ class VarSubState(pybase.CompoundObj):
     x1 = hnode.Leaf('T' if self.is_type_query else 'F', color_e.OtherConst)
     L.append(Field('is_type_query', x1))
 
+    x2 = hnode.Leaf('T' if self.has_test_op else 'F', color_e.OtherConst)
+    L.append(Field('has_test_op', x2))
+
     return out_node
 
   def _AbbreviatedTree(self, trav=None):
@@ -1636,6 +1651,9 @@ class VarSubState(pybase.CompoundObj):
     x1 = hnode.Leaf('T' if self.is_type_query else 'F', color_e.OtherConst)
     L.append(Field('is_type_query', x1))
 
+    x2 = hnode.Leaf('T' if self.has_test_op else 'F', color_e.OtherConst)
+    L.append(Field('has_test_op', x2))
+
     return out_node
 
   def AbbreviatedTree(self, trav=None):
@@ -1643,7 +1661,7 @@ class VarSubState(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class Cell(pybase.CompoundObj):
-  _type_tag = 66
+  _type_tag = 67
   __slots__ = ('exported', 'readonly', 'nameref', 'val')
 
   def __init__(self, exported, readonly, nameref, val):
@@ -1712,7 +1730,7 @@ class Cell(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class VTestPlace(pybase.CompoundObj):
-  _type_tag = 67
+  _type_tag = 68
   __slots__ = ('name', 'index')
 
   def __init__(self, name, index):
@@ -1769,7 +1787,7 @@ class VTestPlace(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class RedirValue(pybase.CompoundObj):
-  _type_tag = 68
+  _type_tag = 69
   __slots__ = ('op_id', 'op_loc', 'loc', 'arg')
 
   def __init__(self, op_id, op_loc, loc, arg):
@@ -1842,7 +1860,7 @@ class RedirValue(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class StatusArray(pybase.CompoundObj):
-  _type_tag = 69
+  _type_tag = 70
   __slots__ = ('codes', 'locs')
 
   def __init__(self, codes, locs):
@@ -1911,7 +1929,7 @@ class StatusArray(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class CommandStatus(pybase.CompoundObj):
-  _type_tag = 70
+  _type_tag = 71
   __slots__ = ('check_errexit', 'show_code', 'pipe_negated', 'pipe_status',
                'pipe_locs')
 
@@ -2003,7 +2021,7 @@ class CommandStatus(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class HayNode(pybase.CompoundObj):
-  _type_tag = 71
+  _type_tag = 72
   __slots__ = ('children',)
 
   def __init__(self, children):
