@@ -472,19 +472,18 @@ class value_e(object):
   Eggex = 11
   Match = 67
   Expr = 13
-  Template = 14
-  Command = 15
-  Block = 16
-  Place = 17
-  Module = 18
-  IO = 19
-  Guts = 20
-  BuiltinFunc = 21
-  BoundFunc = 22
-  Proc = 23
-  Func = 24
-  Slice = 25
-  Range = 26
+  Command = 14
+  Block = 15
+  Place = 16
+  Module = 17
+  IO = 18
+  Guts = 19
+  BuiltinFunc = 20
+  BoundFunc = 21
+  Proc = 22
+  Func = 23
+  Slice = 24
+  Range = 25
 
 _value_str = {
   1: 'Undef',
@@ -499,19 +498,18 @@ _value_str = {
   10: 'Dict',
   11: 'Eggex',
   13: 'Expr',
-  14: 'Template',
-  15: 'Command',
-  16: 'Block',
-  17: 'Place',
-  18: 'Module',
-  19: 'IO',
-  20: 'Guts',
-  21: 'BuiltinFunc',
-  22: 'BoundFunc',
-  23: 'Proc',
-  24: 'Func',
-  25: 'Slice',
-  26: 'Range',
+  14: 'Command',
+  15: 'Block',
+  16: 'Place',
+  17: 'Module',
+  18: 'IO',
+  19: 'Guts',
+  20: 'BuiltinFunc',
+  21: 'BoundFunc',
+  22: 'Proc',
+  23: 'Func',
+  24: 'Slice',
+  25: 'Range',
   67: 'Match',
 }
 
@@ -1169,56 +1167,8 @@ class value(object):
       # type: (Optional[TraversalState]) -> hnode_t
       return self._AbbreviatedTree(trav=trav)
   
-  class Template(value_t):
-    _type_tag = 14
-    __slots__ = ('dq',)
-  
-    def __init__(self, dq):
-      # type: (DoubleQuoted) -> None
-      self.dq = dq
-  
-    @staticmethod
-    def CreateNull(alloc_lists=False):
-      # type: () -> value.Template
-      return value.Template(cast(DoubleQuoted, None))
-  
-    def PrettyTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.Template')
-      L = out_node.fields
-  
-      assert self.dq is not None
-      x0 = self.dq.PrettyTree(trav=trav)
-      L.append(Field('dq', x0))
-  
-      return out_node
-  
-    def _AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.Template')
-      L = out_node.fields
-      assert self.dq is not None
-      x0 = self.dq.AbbreviatedTree(trav=trav)
-      L.append(Field('dq', x0))
-  
-      return out_node
-  
-    def AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      return self._AbbreviatedTree(trav=trav)
-  
   class Command(value_t):
-    _type_tag = 15
+    _type_tag = 14
     __slots__ = ('c',)
   
     def __init__(self, c):
@@ -1266,7 +1216,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Block(value_t):
-    _type_tag = 16
+    _type_tag = 15
     __slots__ = ('block',)
   
     def __init__(self, block):
@@ -1314,7 +1264,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Place(value_t):
-    _type_tag = 17
+    _type_tag = 16
     __slots__ = ('lval', 'frame')
   
     def __init__(self, lval, frame):
@@ -1379,7 +1329,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Module(value_t):
-    _type_tag = 18
+    _type_tag = 17
     __slots__ = ('defs',)
   
     def __init__(self, defs):
@@ -1435,7 +1385,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class IO(value_t):
-    _type_tag = 19
+    _type_tag = 18
     __slots__ = ('cmd_ev', 'prompt_ev')
   
     def __init__(self, cmd_ev, prompt_ev):
@@ -1488,7 +1438,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Guts(value_t):
-    _type_tag = 20
+    _type_tag = 19
     __slots__ = ('vm',)
   
     def __init__(self, vm):
@@ -1534,7 +1484,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class BuiltinFunc(value_t):
-    _type_tag = 21
+    _type_tag = 20
     __slots__ = ('callable',)
   
     def __init__(self, callable):
@@ -1580,7 +1530,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class BoundFunc(value_t):
-    _type_tag = 22
+    _type_tag = 21
     __slots__ = ('me', 'func')
   
     def __init__(self, me, func):
@@ -1637,17 +1587,17 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Proc(value_t):
-    _type_tag = 23
-    __slots__ = ('name', 'name_tok', 'sig', 'body', 'defaults', 'dynamic_scope')
+    _type_tag = 22
+    __slots__ = ('name', 'name_tok', 'sig', 'body', 'defaults', 'sh_compat')
   
-    def __init__(self, name, name_tok, sig, body, defaults, dynamic_scope):
+    def __init__(self, name, name_tok, sig, body, defaults, sh_compat):
       # type: (str, Token, proc_sig_t, command_t, Optional[ProcDefaults], bool) -> None
       self.name = name
       self.name_tok = name_tok
       self.sig = sig
       self.body = body
       self.defaults = defaults
-      self.dynamic_scope = dynamic_scope
+      self.sh_compat = sh_compat
   
     @staticmethod
     def CreateNull(alloc_lists=False):
@@ -1683,8 +1633,8 @@ class value(object):
         x4 = self.defaults.PrettyTree(trav=trav)
         L.append(Field('defaults', x4))
   
-      x5 = hnode.Leaf('T' if self.dynamic_scope else 'F', color_e.OtherConst)
-      L.append(Field('dynamic_scope', x5))
+      x5 = hnode.Leaf('T' if self.sh_compat else 'F', color_e.OtherConst)
+      L.append(Field('sh_compat', x5))
   
       return out_node
   
@@ -1716,8 +1666,8 @@ class value(object):
         x4 = self.defaults.AbbreviatedTree(trav=trav)
         L.append(Field('defaults', x4))
   
-      x5 = hnode.Leaf('T' if self.dynamic_scope else 'F', color_e.OtherConst)
-      L.append(Field('dynamic_scope', x5))
+      x5 = hnode.Leaf('T' if self.sh_compat else 'F', color_e.OtherConst)
+      L.append(Field('sh_compat', x5))
   
       return out_node
   
@@ -1726,7 +1676,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Func(value_t):
-    _type_tag = 24
+    _type_tag = 23
     __slots__ = ('name', 'parsed', 'pos_defaults', 'named_defaults', 'module_')
   
     def __init__(self, name, parsed, pos_defaults, named_defaults, module_):
@@ -1832,7 +1782,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Slice(value_t):
-    _type_tag = 25
+    _type_tag = 24
     __slots__ = ('lower', 'upper')
   
     def __init__(self, lower, upper):
@@ -1889,7 +1839,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Range(value_t):
-    _type_tag = 26
+    _type_tag = 25
     __slots__ = ('lower', 'upper')
   
     def __init__(self, lower, upper):
@@ -1991,18 +1941,19 @@ class IntBox(pybase.CompoundObj):
 
 class ProcDefaults(pybase.CompoundObj):
   _type_tag = 65
-  __slots__ = ('for_word', 'for_typed', 'for_named')
+  __slots__ = ('for_word', 'for_typed', 'for_named', 'for_block')
 
-  def __init__(self, for_word, for_typed, for_named):
-    # type: (Optional[List[value_t]], Optional[List[value_t]], Optional[Dict[str, value_t]]) -> None
+  def __init__(self, for_word, for_typed, for_named, for_block):
+    # type: (Optional[List[value_t]], Optional[List[value_t]], Optional[Dict[str, value_t]], Optional[value_t]) -> None
     self.for_word = for_word
     self.for_typed = for_typed
     self.for_named = for_named
+    self.for_block = for_block
 
   @staticmethod
   def CreateNull(alloc_lists=False):
     # type: () -> ProcDefaults
-    return ProcDefaults(cast('Optional[List[value_t]]', None), cast('Optional[List[value_t]]', None), cast('Optional[Dict[str, value_t]]', None))
+    return ProcDefaults(cast('Optional[List[value_t]]', None), cast('Optional[List[value_t]]', None), cast('Optional[Dict[str, value_t]]', None), cast('Optional[value_t]', None))
 
   def PrettyTree(self, trav=None):
     # type: (Optional[TraversalState]) -> hnode_t
@@ -2038,6 +1989,10 @@ class ProcDefaults(pybase.CompoundObj):
         x2.children.append(v2.PrettyTree(trav=trav))
       L.append(Field('for_named', x2))
 
+    if self.for_block is not None:  # Optional
+      x3 = self.for_block.PrettyTree(trav=trav)
+      L.append(Field('for_block', x3))
+
     return out_node
 
   def _AbbreviatedTree(self, trav=None):
@@ -2073,6 +2028,10 @@ class ProcDefaults(pybase.CompoundObj):
         x2.children.append(v2.AbbreviatedTree(trav=trav))
       L.append(Field('for_named', x2))
 
+    if self.for_block is not None:  # Optional
+      x3 = self.for_block.AbbreviatedTree(trav=trav)
+      L.append(Field('for_block', x3))
+
     return out_node
 
   def AbbreviatedTree(self, trav=None):
