@@ -284,7 +284,7 @@ class Hay(vm._Builtin):
                 var_name = var_name[1:]
                 # TODO: This could be fatal?
 
-            cmd = typed_args.OptionalCommand(cmd_val)
+            cmd = typed_args.OptionalBlock(cmd_val)
             if not cmd:  # 'package foo' is OK
                 e_usage('eval expected a block', loc.Missing)
 
