@@ -1,15 +1,15 @@
 """
-lexer_def.py -- A lexer for both OSH and YSH.
+lexer_def.py - Lexing for OSH, YSH, and J8 Notation.
 
-It consists of a series of lexer modes, each with a regex -> Id mapping.
+The OSH/YSH lexer has lexer modes, each with a regex -> Id mapping.
 
 After changing this file, run:
 
-    build/dev.sh all
+    build/py.sh all
 
 or at least:
 
-    build/dev.sh fastlex
+    build/py.sh fastlex
 
 Input Handling
 --------------
@@ -18,14 +18,15 @@ Every line is NUL terminated:
 
     'one\n\0' 'last line\0'
 
-which means that no regexes below should match \0.  The core/lexer_gen.py code
-generator adds and extra rule for \0.
+which means that no regexes below should match \0.
 
 For example, use [^'\0]+ instead of [^']+ .
 
 If this rule isn't followed, we would read uninitialized memory past the
 sentinel.  Python's regex engine knows where the end of the input string is, so
 it doesn't require need a sentinel like \0.
+
+The frontend/lexer_gen.py generator adds a pattern mapping \0 to Id.Eol_Tok.
 """
 
 from _devbuild.gen.id_kind_asdl import Id, Id_t, Kind
@@ -59,6 +60,7 @@ def R(pat, tok_type):
 # We need the [^\0]* because the re2c translation assumes it's anchored like $.
 SHOULD_HIJACK_RE = r'#![^\0]*sh[ \t\r\n][^\0]*'
 
+# Separates words (\r it not whitespace here)
 _SIGNIFICANT_SPACE = R(r'[ \t]+', Id.WS_Space)
 
 _BACKSLASH = [
@@ -100,6 +102,9 @@ _LEFT_SUBS = [
     C('`', Id.Left_Backtick),
     C('$(', Id.Left_DollarParen),
     C('${', Id.Left_DollarBrace),
+    # Parse zsh syntax, but don't execute it.
+    # The examples we've seen so far are like ${(%):-} and ${(m)
+    R(r'\$\{\([^)\0]+\)', Id.Left_DollarBraceZsh),
     C('$((', Id.Left_DollarDParen),
     C('$[', Id.Left_DollarBracket),
 ]
@@ -135,10 +140,13 @@ LEXER_DEF = {}  # TODO: Should be a list so we enforce order.
 # docs, etc.
 LEXER_DEF[lex_mode_e.Comment] = [R(r'[^\n\0]*', Id.Ignored_Comment)]
 
-# A whitelist for efficiency.  The shell language says that "anything else" is
-# a literal character.  In other words, a single $ \ or ! is a literal, not a
-# syntax error.  It's defined negatively, but let's define positive runs here.
-# TODO: Add + here because it's never special?  It's different for YSH though.
+# A whitelist to make bigger Lit_Chars tokens.  We don't want one byte at a time.
+#
+# The shell language says that "anything other byte" is a literal character --
+# for example, unquoted $ \ ! are literal, not a syntax error.
+#
+# That is, a literal is defined NEGATIVELY, for a single characters.  But here
+# we define a SUBSET of literal chars POSITIVELY.
 
 # The range \x80-\xff makes sure that UTF-8 sequences are a single token.
 _LITERAL_WHITELIST_REGEX = r'[\x80-\xffa-zA-Z0-9_.\-]+'
@@ -160,7 +168,10 @@ _UNQUOTED = _BACKSLASH + _LEFT_SUBS + _LEFT_UNQUOTED + _LEFT_PROCSUB + _VARS + [
     C('&&', Id.Op_DAmp),
     C('||', Id.Op_DPipe),
     C(';', Id.Op_Semi),
+    # Case terminators
     C(';;', Id.Op_DSemi),
+    C(';&', Id.Op_SemiAmp),
+    C(';;&', Id.Op_DSemiAmp),
     C('(', Id.Op_LParen),
     C(')', Id.Op_RParen),
     R(r'[^\0]', Id.Lit_Other),  # any other single char is a literal
@@ -203,6 +214,7 @@ KEYWORDS = [
     C('setglobal', Id.KW_SetGlobal),
     C('call', Id.KW_Call),
     C('proc', Id.KW_Proc),
+    C('typed', Id.KW_Typed),
     C('func', Id.KW_Func),
 ]
 
@@ -224,24 +236,19 @@ EXPR_WORDS = [
     C('or', Id.Expr_Or),
     C('not', Id.Expr_Not),
     C('for', Id.Expr_For),
-    C('while', Id.Expr_While),
     C('is', Id.Expr_Is),
     C('in', Id.Expr_In),
     C('if', Id.Expr_If),
     C('else', Id.Expr_Else),
 
-    # for function literals
+    # Unused: could be for function literals, although we also have
+    # |x| x+1 lambdas
     C('func', Id.Expr_Func),
 
     # / <capture d+/
     C('capture', Id.Expr_Capture),
     # / <capture d+ as date> /
     C('as', Id.Expr_As),
-
-    # Tea Control Flow Operators
-    C('break', Id.Expr_Break),
-    C('continue', Id.Expr_Continue),
-    C('return', Id.Expr_Return),
 ]
 
 FD_VAR_NAME = r'\{' + VAR_NAME_RE + r'\}'
@@ -365,25 +372,37 @@ LEXER_DEF[lex_mode_e.ExtGlob] = \
 # TODO: For testing, write a script to extract and save regexes... and compile
 # them with regcomp.  I've only seen constant regexes.
 #
-# From code: ( | ) are treated special.
+# bash code: ( | ) are special
 
 LEXER_DEF[lex_mode_e.BashRegex] = _LEFT_SUBS + _LEFT_UNQUOTED + _VARS + [
-
-    # NOTE: bash accounts for spaces and non-word punctuation like ; inside ()
-    # and [].  We will avoid that and ask the user to extract a variable?
-    R(r'[a-zA-Z0-9_-]+', Id.Lit_Chars),  # not including period
+    # Like lex_mode_e.ShCommand
+    R(_LITERAL_WHITELIST_REGEX, Id.Lit_Chars),
 
     # Tokens for Tilde sub.  bash weirdness: RHS of [[ x =~ ~ ]] is expanded
     C('~', Id.Lit_Tilde),
     C('/', Id.Lit_Slash),
+
+    # Id.WS_Space delimits words.  In lex_mode_e.BashRegexFakeInner, we
+    # translate them to Id.Lit_Chars.
     _SIGNIFICANT_SPACE,
 
-    # Normally, \x evaluates to x.  But quoted regex metacharacters like \* should
-    # evaluate to \*.  Compare with ( | ).
-    R(r'\\[*+?.^$\[\]]', Id.Lit_RegexMeta),
+    # Analogous to Id.ExtGlob_* - we need to change lexer modes when we hit this
+    C('(', Id.BashRegex_LParen),
 
-    # NOTE: ( | and ) aren't operators!
-    R(r'[^\0]', Id.Lit_Other),  # Everything else is a literal
+    # Not special, this is like lex_mode_e.Outer
+    C(')', Id.Op_RParen),
+
+    # Copied and adapted from _UNQUOTED
+    # \n & ; < > are parse errors OUTSIDE a group   [[ s =~ ; ]]
+    #            but become allowed INSIDE a group  [[ s =~ (;) ]]
+    C('\n', Id.BashRegex_AllowedInParens),
+    C('&', Id.BashRegex_AllowedInParens),
+    C(';', Id.BashRegex_AllowedInParens),
+    C('>', Id.BashRegex_AllowedInParens),
+    C('<', Id.BashRegex_AllowedInParens),
+
+    # e.g. | is Id.Lit_Other, not pipe operator
+    R(r'[^\0]', Id.Lit_Other),  # like _UNQUOTED, any other byte is literal
 ] + _BACKSLASH  # These have to come after RegexMeta
 
 LEXER_DEF[lex_mode_e.DQ] = _DQ_BACKSLASH + [
@@ -403,7 +422,16 @@ _VS_ARG_COMMON = [
     C('$', Id.Lit_Dollar),  # completion of var names relies on this
 ]
 
-# Kind.{LIT,IGNORED,VS,LEFT,RIGHT,Eof}
+# We don't execute zsh var subs, but to find the closing } properly, we need to
+# to recognize \} and '}' and "}" $'}' etc.
+LEXER_DEF[lex_mode_e.VSub_Zsh] = \
+  _BACKSLASH + _LEFT_SUBS + _LEFT_UNQUOTED + _LEFT_PROCSUB + \
+  [
+    C('}', Id.Right_DollarBrace),  # For var sub "${a}"
+    R(r'[^\0]', Id.Lit_Other),  # e.g. "$", must be last
+]
+
+# Kind.{Lit,Ignored,VSub,Left,Right,Eof}
 LEXER_DEF[lex_mode_e.VSub_ArgUnquoted] = \
   _BACKSLASH + _VS_ARG_COMMON + _LEFT_SUBS + _LEFT_UNQUOTED + _LEFT_PROCSUB + \
   _VARS + _EXTGLOB_BEGIN + [
@@ -419,7 +447,7 @@ LEXER_DEF[lex_mode_e.VSub_ArgUnquoted] = \
     R(r'[^\0]', Id.Lit_Other),  # e.g. "$", must be last
 ]
 
-# Kind.{LIT,IGNORED,VS,LEFT,RIGHT,Eof}
+# Kind.{Lit,Ignored,VSub,Left,Right,Eof}
 LEXER_DEF[lex_mode_e.VSub_ArgDQ] = \
   _DQ_BACKSLASH +  _VS_ARG_COMMON + _LEFT_SUBS + _VARS + [
 
@@ -497,13 +525,13 @@ ECHO_E_DEF = _C_STRING_COMMON + [
     C(r'\c', Id.Char_Stop),
 
     # e.g. 'foo', anything that's not a backslash escape
-    R(r'[^\\\0]+', Id.Char_Literals),
+    R(r'[^\\\0]+', Id.Lit_Chars),
 ]
 
 # https://json.org/
 
 # Note that [0-9] has to come second, because Python chooses the first match.
-_JSON_INT = r'([1-9][0-9]*|[0-9])'  # Numbers can't start with leading 0
+_JSON_INT = r'-?([1-9][0-9]*|[0-9])'  # Numbers can't start with leading 0
 _JSON_FRACTION = r'(\.[0-9]+)?'
 _JSON_EXP = r'([eE][-+]?[0-9]+)?'
 
@@ -535,12 +563,16 @@ J8_SYMBOL_RE = (
     r'[a-zA-Z0-9' + J8_SYMBOL_CHARS + ']*')
 # yapf: enable
 
-J8_DEF = [
+_J8_LEFT = [
     C('"', Id.Left_DoubleQuote),  # JSON string
+    C('j"', Id.Left_JDoubleQuote),  # JSON string with explicit J8 prefix
     # Three left quotes that are J8 only
     C("u'", Id.Left_USingleQuote),  # unicode string
     C("'", Id.Left_USingleQuote),  # '' is alias for u'' in data, not in code
     C("b'", Id.Left_BSingleQuote),  # byte string
+]
+
+J8_DEF = _J8_LEFT + [
     C('[', Id.J8_LBracket),
     C(']', Id.J8_RBracket),
     C('{', Id.J8_LBrace),
@@ -565,10 +597,10 @@ J8_DEF = [
     # be either Symbol or plain Identifier, but field names can only be
     # Identifier.  JSON8 only has Identifier.
     #R(J8_SYMBOL_RE, Id.J8_Symbol),  # NIL8 only
-    R(r'[~!@$%^&*+=|:;./<>?-]+', Id.J8_Operator),  # NIL8 only
-
-    # TODO: emit Id.Ignored_Newline to count lines for error messages?
-    R(r'[ \r\n\t]+', Id.Ignored_Space),
+    R(r'[~!@$%^&*+=|;./<>?-]+', Id.J8_Operator),  # NIL8 only
+    R(r'[ \r\t]+', Id.Ignored_Space),
+    # A separate token, to count lines for error messages
+    C('\n', Id.Ignored_Newline),
     # comment is # until end of line
     # // comments are JavaScript style, but right now we might want them as
     # symbols?
@@ -578,30 +610,55 @@ J8_DEF = [
     R(r'[^\0]', Id.Unknown_Tok),
 ]
 
-# Exclude control characters 0x00-0x1f, aka 0-31 in J8 data
-# But \n has to be allowed in multi-line strings
+# Exclude control characters 0x00-0x1f, aka 0-31 in J8 data only (not YSH code)
 _ASCII_CONTROL = R(r'[\x01-\x1F]', Id.Char_AsciiControl)
 
+J8_LINES_DEF = _J8_LEFT + [
+    # not sure if we want \r here - same with lex_mode_e.Expr
+    R(r'[ \r\t]+', Id.WS_Space),
+    R(r'[\n]', Id.J8_Newline),
+
+    # doesn't match \t, which means tabs are allowed in the middle of unquoted
+    # lines
+    _ASCII_CONTROL,
+
+    # not space or ' or " or ASCII control or EOF
+    R(r'''[^ \t\r\n'"\x00-\x1F]+''', Id.Lit_Chars),
+]
+
 # https://json.org list of chars, plus '
 _JSON_ONE_CHAR = R(r'\\[\\"/bfnrt]', Id.Char_OneChar)
 
-# Union of escapes that "" u"" b"" accept.  Validation is separate.
-J8_STR_DEF = [
+# b'' u'' strings - what's common between code and data.
+_J8_STR_COMMON = [
     C("'", Id.Right_SingleQuote),  # end for J8
     _JSON_ONE_CHAR,
-    C("\\'", Id.Char_OneChar),
-
-    # osh/word_parse.py relies on this.  It has to match $'', which uses _C_STRING_COMMON
-    C('\\', Id.Unknown_Backslash),
+    C("\\'", Id.Char_OneChar),  # since ' ends, allow \'
     R(r'\\y[0-9a-fA-F]{2}', Id.Char_YHex),  # \yff - J8 only
     _U_BRACED_CHAR,  # \u{123456} - J8 only
-    _ASCII_CONTROL,
 
-    # Note: This will match INVALID UTF-8.  UTF-8 validation is another step.
-    R(r'''[^\\'\0]+''', Id.Char_Literals),
+    # osh/word_parse.py relies on this.  It has to be consistent with $''
+    # lexing, which uses _C_STRING_COMMON
+    C('\\', Id.Unknown_Backslash),
 ]
 
-# For "JSON strings \" \u1234"
+# Lexer for J8 strings in CODE.
+LEXER_DEF[lex_mode_e.J8_Str] = _J8_STR_COMMON + [
+    # Don't produce Char_AsciiControl tokens - that's only for data
+
+    # will match invalid UTF-8 - we have a separate validation step
+    R(r"[^\\'\0]+", Id.Lit_Chars),
+]
+
+# Lexer for J8 string data.
+# ASCII control characters are disallowed in DATA, but not CODE!
+J8_STR_DEF = _J8_STR_COMMON + [
+    _ASCII_CONTROL,
+    # will match invalid UTF-8 - we have a separate validation step
+    R(r"[^\\'\x00-\x1F]+", Id.Lit_Chars),
+]
+
+# Lexer for JSON string data - e.g. "json \" \u1234"
 JSON_STR_DEF = [
     C('"', Id.Right_DoubleQuote),  # end for JSON
     _JSON_ONE_CHAR,
@@ -613,15 +670,13 @@ JSON_STR_DEF = [
     R(
         r'\\u[dD][89aAbB][0-9a-fA-F][0-9a-fA-F]\\u[dD][cCdDeEfF][0-9a-fA-F][0-9a-fA-F]',
         Id.Char_SurrogatePair),
+    C('\\', Id.Unknown_Backslash),  # e.g. the \ before bad \z
     _ASCII_CONTROL,
 
     # Note: This will match INVALID UTF-8.  UTF-8 validation is another step.
-    R(r'[^\\"\0]+', Id.Char_Literals),
-    R(r'[^\0]', Id.Unknown_Tok),
+    R(r'[^\\"\x00-\x1F]+', Id.Lit_Chars),
 ]
 
-LEXER_DEF[lex_mode_e.J8_Str] = J8_STR_DEF
-
 OCTAL3_RE = r'\\[0-7]{1,3}'
 
 # https://www.gnu.org/software/bash/manual/html_node/Controlling-the-PromptEvaluator.html#Controlling-the-PromptEvaluator
@@ -654,13 +709,13 @@ LEXER_DEF[lex_mode_e.SQ_C] = _C_STRING_COMMON + [
     C(r'\"', Id.Char_OneChar),
 
     # e.g. 'foo', anything that's not a backslash escape or '
-    R(r"[^\\'\0]+", Id.Char_Literals),
+    R(r"[^\\'\0]+", Id.Lit_Chars),
     C("'", Id.Right_SingleQuote),
 ]
 
 LEXER_DEF[lex_mode_e.PrintfOuter] = _C_STRING_COMMON + [
     R(OCTAL3_RE, Id.Char_Octal3),
-    R(r"[^%\\\0]+", Id.Char_Literals),
+    R(r"[^%\\\0]+", Id.Lit_Chars),
     C('%%', Id.Format_EscapedPercent),
     C('%', Id.Format_Percent),
 ]
@@ -700,7 +755,7 @@ LEXER_DEF[lex_mode_e.VSub_1] = [
 LEXER_DEF[lex_mode_e.VSub_2] = \
     ID_SPEC.LexerPairs(Kind.VTest) + \
     ID_SPEC.LexerPairs(Kind.VOp0) + \
-    ID_SPEC.LexerPairs(Kind.VOpOil) + \
+    ID_SPEC.LexerPairs(Kind.VOpYsh) + \
     ID_SPEC.LexerPairs(Kind.VOp1) + \
     ID_SPEC.LexerPairs(Kind.VOp2) + \
     ID_SPEC.LexerPairs(Kind.VOp3) + [
@@ -836,15 +891,19 @@ YSH_LEFT_SUBS = [
 # Used by ysh/grammar_gen.py
 
 YSH_LEFT_UNQUOTED = [
+    # Double quoted
     C('"', Id.Left_DoubleQuote),
-    # In expression mode, we add the r'' and c'' prefixes for '' and $''.
+    C('$"', Id.Left_DollarDoubleQuote),  # $"" is synonym for ""
+    C('j"', Id.Left_JDoubleQuote),  # for printing ERROR
+    # Single quoted
     C("'", Id.Left_SingleQuote),
     C("r'", Id.Left_RSingleQuote),
     C("u'", Id.Left_USingleQuote),
     C("b'", Id.Left_BSingleQuote),
-    C("$'", Id.Left_DollarSingleQuote),
+    C("$'", Id.Left_DollarSingleQuote),  # legacy
     C('^"', Id.Left_CaretDoubleQuote),
     C('"""', Id.Left_TDoubleQuote),
+    C('$"""', Id.Left_DollarTDoubleQuote),
     # In expression mode, we add the r'' and c'' prefixes for '' and $''.
     C("'''", Id.Left_TSingleQuote),
     C("r'''", Id.Left_RTSingleQuote),
@@ -888,10 +947,11 @@ EXPR_OPS = [
 _EXPR_NEWLINE_COMMENT = [
     C('\n', Id.Op_Newline),
     R(r'#[^\n\0]*', Id.Ignored_Comment),
+    # Like lex_mode_e.Arith, \r is whitespace even without \n
     R(r'[ \t\r]+', Id.Ignored_Space),
 ]
 
-_WHITESPACE = r'[ \t\r\n]*'  # not including legacy \f \v
+_WHITESPACE = r'[ \t\r\n]*'  # ASCII whitespace doesn't have legacy \f \v
 
 # Python allows 0 to be written 00 or 0_0_0, which is weird.  But let's be
 # consistent, and avoid '00' turning into a float!
