@@ -22,7 +22,7 @@ except ImportError:
     posix = os
 
 from typing import (Tuple, List, Dict, Optional, Iterator, Any, TypeVar,
-                    TYPE_CHECKING)
+                    Generic, cast, TYPE_CHECKING)
 
 # For conditional translation
 CPP = False
@@ -172,6 +172,10 @@ class Writer:
         # type: () -> bool
         raise NotImplementedError()
 
+    def close(self):
+        # type: () -> None
+        raise NotImplementedError()
+
 
 class BufWriter(Writer):
     """Mimic StringIO API, but add clear() so we can reuse objects.
@@ -201,6 +205,13 @@ class BufWriter(Writer):
         # type: () -> None
         del self.parts[:]
 
+    def close(self):
+        # type: () -> None
+
+        # No-op for now - we could invalidate write()?
+
+        pass
+
 
 def Stdout():
     # type: () -> Writer
@@ -286,6 +297,43 @@ class tagswitch(object):
         return self.tag in cases
 
 
+if TYPE_CHECKING:
+    # Doesn't work
+    T = TypeVar('T')
+
+    class StackArray(Generic[T]):
+
+        def __init__(self):
+            self.items = []  # type: List[T]
+
+        def append(self, item):
+            # type: (T) -> None
+            self.items.append(item)
+
+        def pop(self):
+            # type: () -> T
+            return self.items.pop()
+
+    # Doesn't work, this is only for primitive types
+    #StackArray = NewType('StackArray', list)
+
+
+def MakeStackArray(item_type):
+    # type: (TypeVar) -> StackArray[item_type]
+    """
+    Convenience "constructor" used like this:
+
+        myarray = MakeStackArray(int)
+
+    The idiom could also be
+
+        myarray = cast('StackArray[int]', [])
+
+    But that's uglier.
+    """
+    return cast('StackArray[item_type]', [])
+
+
 if TYPE_CHECKING:
     K = TypeVar('K')
     V = TypeVar('V')
@@ -427,6 +475,13 @@ class UniqueObjects(object):
     # Note: self.memo.clear() doesn't appear to be used
 
 
+def probe(provider, name, *args):
+    # type: (str, str, Any) -> None
+    """Create a probe for use with profilers like linux perf and ebpf or dtrace."""
+    # Noop. Just a marker for mycpp to emit a DTRACE_PROBE()
+    return
+
+
 if 0:
     # Prototype of Unix file descriptor I/O, compared with FILE* libc I/O.
     # Doesn't seem like we need this now.
