@@ -40,7 +40,7 @@ from data_lang import j8_lite
 from frontend import location
 from frontend import match
 from mycpp import mylib
-from mycpp.mylib import log, print_stderr, tagswitch, iteritems
+from mycpp.mylib import log, print_stderr, probe, tagswitch, iteritems
 
 import posix_ as posix
 from posix_ import (
@@ -466,10 +466,13 @@ class FdState(object):
 
                 thunk = _HereDocWriterThunk(write_fd, arg.body)
 
-                # TODO: Use PIPE_SIZE to save a process in the case of small here docs,
-                # which are the common case.  (dash does this.)
-                start_process = True
-                #start_process = False
+                # Use PIPE_SIZE to save a process in the case of small here
+                # docs, which are the common case.  (dash does this.)
+
+                # Note: could instrument this to see how often it happens.
+                # Though strace -ff can also work.
+                start_process = len(arg.body) > 4096
+                #start_process = True
 
                 if start_process:
                     here_proc = Process(thunk, self.job_control, self.job_list,
@@ -675,6 +678,7 @@ class ExternalProgram(object):
 
         Called by:   ls /   exec ls /   ( ls / )
         """
+        probe('process', 'ExternalProgram_Exec', argv0_path)
         self._Exec(argv0_path, cmd_val.argv, cmd_val.arg_locs[0], environ,
                    True)
         assert False, "This line should never execute"  # NO RETURN
@@ -795,11 +799,17 @@ class ExternalThunk(Thunk):
 class SubProgramThunk(Thunk):
     """A subprogram that can be executed in another process."""
 
-    def __init__(self, cmd_ev, node, trap_state, inherit_errexit=True):
-        # type: (CommandEvaluator, command_t, trap_osh.TrapState, bool) -> None
+    def __init__(self,
+                 cmd_ev,
+                 node,
+                 trap_state,
+                 multi_trace,
+                 inherit_errexit=True):
+        # type: (CommandEvaluator, command_t, trap_osh.TrapState, dev.MultiTracer, bool) -> None
         self.cmd_ev = cmd_ev
         self.node = node
         self.trap_state = trap_state
+        self.multi_trace = multi_trace
         self.inherit_errexit = inherit_errexit  # for bash errexit compatibility
 
     def UserString(self):
@@ -815,6 +825,7 @@ class SubProgramThunk(Thunk):
     def Run(self):
         # type: () -> None
         #self.errfmt.OneLineErrExit()  # don't quote code in child processes
+        probe('process', 'SubProgramThunk_Run')
 
         # TODO: break circular dep.  Bit flags could go in ASDL or headers.
         from osh import cmd_eval
@@ -848,6 +859,8 @@ class SubProgramThunk(Thunk):
         # _exit()
         pyos.FlushStdout()
 
+        self.multi_trace.WriteDumps()
+
         # We do NOT want to raise SystemExit here.  Otherwise dev.Tracer::Pop()
         # gets called in BOTH processes.
         # The crash dump seems to be unaffected.
@@ -877,6 +890,7 @@ class _HereDocWriterThunk(Thunk):
     def Run(self):
         # type: () -> None
         """do_exit: For small pipelines."""
+        probe('process', 'HereDocWriterThunk_Run')
         #log('Writing %r', self.body_str)
         posix.write(self.w, self.body_str)
         #log('Wrote %r', self.body_str)
@@ -1062,7 +1076,7 @@ class Process(Job):
             pyos.Sigaction(SIGTTOU, SIG_DFL)
             pyos.Sigaction(SIGTTIN, SIG_DFL)
 
-            self.tracer.SetProcess(pid)
+            self.tracer.OnNewProcess(pid)
             # clear foreground pipeline for subshells
             self.thunk.Run()
             # Never returns
@@ -1143,7 +1157,8 @@ class Process(Job):
                     print_stderr('[%d] Done PID %d' % (self.job_id, self.pid))
 
                 self.job_list.RemoveJob(self.job_id)
-                self.job_list.RemoveChildProcess(self.pid)
+
+            self.job_list.RemoveChildProcess(self.pid)
 
             if not self.in_background:
                 self.job_control.MaybeTakeTerminal()
@@ -1708,7 +1723,7 @@ class JobList(object):
             return previous
 
         # TODO: Add support for job specs based on prefixes of process argv.
-        m = util.simple_regex_search(r'^%([0-9]+)$', job_spec)
+        m = util.RegexSearch(r'^%([0-9]+)$', job_spec)
         if m is not None:
             assert len(m) == 2
             job_id = int(m[1])
