@@ -8,6 +8,7 @@
 """
 sh_expr_eval.py -- Shell boolean and arithmetic expressions.
 """
+from __future__ import print_function
 
 from _devbuild.gen.id_kind_asdl import Id
 from _devbuild.gen.runtime_asdl import scope_t
@@ -15,7 +16,6 @@ from _devbuild.gen.syntax_asdl import (
     word_t,
     CompoundWord,
     Token,
-    NameTok,
     loc,
     loc_t,
     source,
@@ -51,6 +51,8 @@ from core import num
 from core import state
 from core import ui
 from frontend import consts
+from frontend import lexer
+from frontend import location
 from frontend import match
 from frontend import parse_lib
 from frontend import reader
@@ -513,8 +515,8 @@ class ArithEvaluator(object):
         # BASH_LINENO, arr (array name without strict_array), etc.
         if (val.tag() in (value_e.BashArray, value_e.BashAssoc) and
                 node.tag() == arith_expr_e.VarSub):
-            vsub = cast(NameTok, node)
-            if word_eval.ShouldArrayDecay(vsub.var_name, self.exec_opts):
+            vsub = cast(Token, node)
+            if word_eval.ShouldArrayDecay(lexer.LazyStr(vsub), self.exec_opts):
                 val = word_eval.DecayArray(val)
 
         i = self._ValToIntOrError(val, node)
@@ -544,10 +546,11 @@ class ArithEvaluator(object):
         UP_node = node
         with tagswitch(node) as case:
             if case(arith_expr_e.VarSub):  # $(( x ))  (can be array)
-                vsub = cast(NameTok, UP_node)
-                val = self.mem.GetValue(vsub.var_name)
+                vsub = cast(Token, UP_node)
+                var_name = lexer.LazyStr(vsub)
+                val = self.mem.GetValue(var_name)
                 if val.tag() == value_e.Undef and self.exec_opts.nounset():
-                    e_die('Undefined variable %r' % vsub.var_name, vsub.left)
+                    e_die('Undefined variable %r' % var_name, vsub)
                 return val
 
             elif case(arith_expr_e.Word):  # $(( $x )) $(( ${x}${y} )), etc.
@@ -656,7 +659,7 @@ class ArithEvaluator(object):
 
             elif case(arith_expr_e.Binary):
                 node = cast(arith_expr.Binary, UP_node)
-                op_id = node.op_id
+                op_id = node.op.id
 
                 # Short-circuit evaluation for || and &&.
                 if op_id == Id.Arith_DPipe:
@@ -730,12 +733,12 @@ class ArithEvaluator(object):
                     result = mops.Mul(lhs_big, rhs_big)
                 elif op_id == Id.Arith_Slash:
                     if mops.Equal(rhs_big, mops.ZERO):
-                        e_die('Divide by zero', loc.Arith(node.right))
+                        e_die('Divide by zero', node.op)
                     result = num.IntDivide(lhs_big, rhs_big)
 
                 elif op_id == Id.Arith_Percent:
                     if mops.Equal(rhs_big, mops.ZERO):
-                        e_die('Divide by zero', loc.Arith(node.right))
+                        e_die('Divide by zero', node.op)
                     result = num.IntRemainder(lhs_big, rhs_big)
 
                 elif op_id == Id.Arith_DStar:
@@ -792,8 +795,8 @@ class ArithEvaluator(object):
 
         raise AssertionError('for -Wreturn-type in C++')
 
-    def EvalWordToString(self, node):
-        # type: (arith_expr_t) -> str
+    def EvalWordToString(self, node, blame_loc=loc.Missing):
+        # type: (arith_expr_t, loc_t) -> str
         """
         Raises:
           error.FatalRuntime if the expression isn't a string
@@ -809,8 +812,12 @@ class ArithEvaluator(object):
             val = self.word_ev.EvalWordToString(w)
             return val.s
         else:
-            # TODO: location info for original
-            e_die("Associative array keys must be strings: $x 'x' \"$x\" etc.")
+            # A[x] is the "Parsing Bash is Undecidable" problem
+            # It is a string or var name?
+            # (It's parsed as arith_expr.VarSub)
+            e_die(
+                "Assoc array keys must be strings: $x 'x' \"$x\" etc. (OILS-ERR-101)",
+                blame_loc)
 
     def EvalShellLhs(self, node, which_scopes):
         # type: (sh_lhs_t, scope_t) -> sh_lvalue_t
@@ -835,7 +842,9 @@ class ArithEvaluator(object):
                 assert node.name is not None
 
                 if self.mem.IsBashAssoc(node.name):
-                    key = self.EvalWordToString(node.index)
+                    key = self.EvalWordToString(node.index,
+                                                blame_loc=node.left)
+                    # node.left points to A[ in A[x]=1
                     lval2 = sh_lvalue.Keyed(node.name, key, node.left)
                     lval = lval2
                 else:
@@ -850,13 +859,14 @@ class ArithEvaluator(object):
 
     def _VarNameOrWord(self, anode):
         # type: (arith_expr_t) -> Tuple[Optional[str], loc_t]
-        """Returns a variable name if the arith node can be interpreted that
-        way."""
+        """
+        Returns a variable name if the arith node can be interpreted that way.
+        """
         UP_anode = anode
         with tagswitch(anode) as case:
             if case(arith_expr_e.VarSub):
-                tok = cast(NameTok, UP_anode)
-                return (tok.var_name, tok.left)
+                tok = cast(Token, UP_anode)
+                return (lexer.LazyStr(tok), tok)
 
             elif case(arith_expr_e.Word):
                 w = cast(CompoundWord, UP_anode)
@@ -874,28 +884,30 @@ class ArithEvaluator(object):
         UP_anode = anode
         if anode.tag() == arith_expr_e.Binary:
             anode = cast(arith_expr.Binary, UP_anode)
-            if anode.op_id == Id.Arith_LBracket:
-                var_name, location = self._VarNameOrWord(anode.left)
+            if anode.op.id == Id.Arith_LBracket:
+                var_name, blame_loc = self._VarNameOrWord(anode.left)
 
                 # (( 1[2] = 3 )) isn't valid
                 if not match.IsValidVarName(var_name):
-                    e_die('Invalid variable name %r' % var_name, location)
+                    e_die('Invalid variable name %r' % var_name, blame_loc)
 
                 if var_name is not None:
                     if self.mem.IsBashAssoc(var_name):
-                        key = self.EvalWordToString(anode.right)
-                        return sh_lvalue.Keyed(var_name, key, location)
+                        arith_loc = location.TokenForArith(anode)
+                        key = self.EvalWordToString(anode.right,
+                                                    blame_loc=arith_loc)
+                        return sh_lvalue.Keyed(var_name, key, blame_loc)
                     else:
                         index = mops.BigTruncate(self.EvalToBigInt(
                             anode.right))
-                        return sh_lvalue.Indexed(var_name, index, location)
+                        return sh_lvalue.Indexed(var_name, index, blame_loc)
 
-        var_name, location = self._VarNameOrWord(anode)
+        var_name, blame_loc = self._VarNameOrWord(anode)
         if var_name is not None:
-            return LeftName(var_name, location)
+            return LeftName(var_name, blame_loc)
 
         # e.g. unset 'x-y'.  status 2 for runtime parse error
-        e_die_status(2, 'Invalid LHS to modify', location)
+        e_die_status(2, 'Invalid LHS to modify', blame_loc)
 
 
 class BoolEvaluator(ArithEvaluator):
