@@ -445,7 +445,14 @@ def Main(
         trace_f = debug_f
     else:
         trace_f = util.DebugFile(mylib.Stderr())
-    tracer = dev.Tracer(parse_ctx, exec_opts, mutable_opts, mem, trace_f)
+
+    trace_dir = environ.get('OILS_TRACE_DIR', '')
+    dumps = environ.get('OILS_TRACE_DUMPS', '')
+    streams = environ.get('OILS_TRACE_STREAMS', '')
+    multi_trace = dev.MultiTracer(my_pid, trace_dir, dumps, streams, fd_state)
+
+    tracer = dev.Tracer(parse_ctx, exec_opts, mutable_opts, mem, trace_f,
+                        multi_trace)
     fd_state.tracer = tracer  # circular dep
 
     signal_safe = pyos.InitSignalSafe()
@@ -475,8 +482,7 @@ def Main(
     # TODO: This is instantiation is duplicated in osh/word_eval.py
     globber = glob_.Globber(exec_opts)
 
-    # This could just be OILS_DEBUG_STREAMS='debug crash' ?  That might be
-    # stuffing too much into one, since a .json crash dump isn't a stream.
+    # This could just be OILS_TRACE_DUMPS='crash:argv0'
     crash_dump_dir = environ.get('OILS_CRASH_DUMP_DIR', '')
     cmd_deps.dumper = dev.CrashDumper(crash_dump_dir, fd_state)
 
@@ -662,6 +668,7 @@ def Main(
     b[builtin_i.exec_] = process_osh.Exec(mem, ext_prog, fd_state, search_path,
                                           errfmt)
     b[builtin_i.umask] = process_osh.Umask()
+    b[builtin_i.ulimit] = process_osh.Ulimit()
     b[builtin_i.wait] = process_osh.Wait(waiter, job_list, mem, tracer, errfmt)
 
     b[builtin_i.jobs] = process_osh.Jobs(job_list)
@@ -833,6 +840,7 @@ def Main(
 
     _SetGlobalFunc(mem, 'glob', func_misc.Glob(globber))
     _SetGlobalFunc(mem, 'shvarGet', func_misc.Shvar_get(mem))
+    _SetGlobalFunc(mem, 'getVar', func_misc.GetVar(mem))
     _SetGlobalFunc(mem, 'assert_', func_misc.Assert())
 
     _SetGlobalFunc(mem, 'toJson8', func_misc.ToJson8(True))
@@ -1057,7 +1065,9 @@ def Main(
     tool_name = 'syntax-tree' if exec_opts.noexec() else flag.tool
 
     if len(tool_name):
-        arena.SaveTokens()
+        # Don't save tokens becaues it's slow
+        if tool_name != 'syntax-tree':
+            arena.SaveTokens()
 
         try:
             node = main_loop.ParseWholeFile(c_parser)
@@ -1104,5 +1114,7 @@ def Main(
     mut_status = IntParamBox(status)
     cmd_ev.MaybeRunExitTrap(mut_status)
 
+    multi_trace.WriteDumps()
+
     # NOTE: We haven't closed the file opened with fd_state.Open
     return mut_status.i
