@@ -19,25 +19,77 @@ from core import ui
 from core import error
 from core.error import e_die, e_strict
 from mycpp.mylib import log
+from mycpp import mylib
 from osh import glob_
 
 import libc
+import fastfunc
 
 from typing import List, Tuple
 
 _ = log
 
-# TODO: Add details of the invalid character/byte here?
-
-INCOMPLETE_CHAR = 'Incomplete UTF-8 character'
-INVALID_CONT = 'Invalid UTF-8 continuation byte'
-INVALID_START = 'Invalid start of UTF-8 character'
+# Error types returned by fastfunc.Utf8DecodeOne
+# Derived from Utf8Error enum from data_lang/utf8.h
+UTF8_ERR_OVERLONG = -1  # Encodes a codepoint in more bytes than necessary
+UTF8_ERR_SURROGATE = -2  # Encodes a codepoint in the surrogate range (0xD800 to 0xDFFF)
+UTF8_ERR_TOO_LARGE = -3  # Encodes a value greater than the max codepoint U+10FFFF
+UTF8_ERR_BAD_ENCODING = -4  # Encoding doesn't conform to the UTF-8 bit patterns
+UTF8_ERR_TRUNCATED_BYTES = -5  # It looks like there is another codepoint, but it has been truncated
 
 
-def _CheckContinuationByte(byte):
-    # type: (str) -> None
-    if (ord(byte) >> 6) != 0b10:
-        e_strict(INVALID_CONT, loc.Missing)
+def Utf8Error_str(error):
+    # type: (int) -> str
+    if error == UTF8_ERR_OVERLONG:
+        return "UTF-8 Error: Decoded Overlong"
+    if error == UTF8_ERR_SURROGATE:
+        return "UTF-8 Error: Decoded Surrogate"
+    if error == UTF8_ERR_TOO_LARGE:
+        return "UTF-8 Error: Decoded Invalid Codepoint"
+    if error == UTF8_ERR_BAD_ENCODING:
+        return "UTF-8 Error: Bad Encoding"
+    if error == UTF8_ERR_TRUNCATED_BYTES:
+        return "UTF-8 Error: Truncated Bytes"
+
+    raise AssertionError(0)
+
+
+def DecodeUtf8Char(s, start):
+    # type: (str, int) -> int
+    """Given a string and start index, decode the Unicode char immediately
+    following the start index. The start location is in bytes and should be
+    found using a function like NextUtf8Char or PreviousUtf8Char.
+
+    If the codepoint in invalid, we raise an `error.Expr`. (This is different
+    from {Next,Previous}Utf8Char which raises an `error.Strict` on encoding
+    errors.)
+    """
+    codepoint_or_error, _bytes_read = fastfunc.Utf8DecodeOne(s, start)
+    if codepoint_or_error < 0:
+        raise error.Expr(
+            "%s at byte index %d in string of length %d" %
+            (Utf8Error_str(codepoint_or_error), start, len(s)), loc.Missing)
+    return codepoint_or_error
+
+
+def NextUtf8Char(s, i):
+    # type: (str, int) -> int
+    """Given a string and a byte offset, returns the byte position after the
+    character at this position.  Usually this is the position of the next
+    character, but for the last character in the string, it's the position just
+    past the end of the string.
+
+    Validates UTF-8.
+    """
+    codepoint_or_error, bytes_read = fastfunc.Utf8DecodeOne(s, i)
+    if codepoint_or_error < 0:
+        e_strict(
+            "%s at byte index %d in string of length %d" %
+            (Utf8Error_str(codepoint_or_error), i, len(s)), loc.Missing)
+    return i + bytes_read
+
+
+_INVALID_START = 'Invalid start of UTF-8 sequence'
 
 
 def _Utf8CharLen(starting_byte):
@@ -51,115 +103,7 @@ def _Utf8CharLen(starting_byte):
     elif (starting_byte >> 3) == 0b11110:
         return 4
     else:
-        e_strict(INVALID_START, loc.Missing)
-
-
-def _ReadOneUnit(s, cursor):
-    # type: (str, int) -> Tuple[int, int]
-    """Helper for DecodeUtf8Char"""
-    if cursor >= len(s):
-        raise error.Expr(INCOMPLETE_CHAR, loc.Missing)
-
-    b = ord(s[cursor])
-    cursor += 1
-
-    if b & 0b11000000 != 0b10000000:
-        raise error.Expr(INVALID_CONT, loc.Missing)
-
-    return b & 0b00111111, cursor
-
-
-def DecodeUtf8Char(s, start):
-    # type: (str, int) -> int
-    """Given a string and start index, decode the Unicode char immediately
-    following the start index. The start location is in bytes and should be
-    found using a function like NextUtf8Char or PreviousUtf8Char.
-
-    If the codepoint in invalid, we raise an `error.Expr`. (This is different
-    from {Next,Previous}Utf8Char which raises an `error.Strict` on encoding
-    errors.)
-
-    Known Issues:
-    - Doesn't raise issue on surrogate pairs
-    - Doesn't raise issue on non-shortest form encodings
-    - Isn't very performant and allocates one-byte-strings for each byte
-    """
-    # We use table 3.6 (reproduced below) from [0]. Note that table 3.6 is not
-    # sufficient for validating UTF-8 as it allows surrogate pairs and
-    # non-shortest form encodings. A correct decoder should follow the
-    # encodings in table 3.7 from [0].
-    #
-    # | Scalar Value               | 1st Byte | 2nd Byte | 3rd Byte | 4th Byte |
-    # +----------------------------+----------+----------+----------+----------+
-    # | 00000000 0xxxxxxx          | 0xxxxxxx |          |          |          |
-    # | 00000yyy yyxxxxxx          | 110yyyyy | 10xxxxxx |          |          |
-    # | zzzzyyyy yyxxxxxx          | 1110zzzz | 10yyyyyy | 10xxxxxx |          |
-    # | 000uuuuu zzzzyyyy yyxxxxxx | 11110uuu | 10uuzzzz | 10yyyyyy | 10xxxxxx |
-    #
-    # [0] https://www.unicode.org/versions/Unicode15.0.0/ch03.pdf
-    assert 0 <= start < len(s)
-
-    b = ord(s[start])
-    cursor = start + 1
-
-    if b & 0b10000000 == 0:
-        return b & 0b01111111
-
-    if b & 0b11100000 == 0b11000000:
-        y = b & 0b00011111
-        y <<= 6
-
-        x, cursor = _ReadOneUnit(s, cursor)
-
-        return y | x
-
-    if b & 0b11110000 == 0b11100000:
-        z = b & 0b00001111
-        z <<= 12
-
-        y, cursor = _ReadOneUnit(s, cursor)
-        y <<= 6
-
-        x, cursor = _ReadOneUnit(s, cursor)
-
-        return z | x | y
-
-    if b & 0b11111000 == 0b11110000:
-        u = b & 0b00000111
-        u <<= 18
-
-        z, cursor = _ReadOneUnit(s, cursor)
-        z <<= 12
-
-        y, cursor = _ReadOneUnit(s, cursor)
-        y <<= 6
-
-        x, cursor = _ReadOneUnit(s, cursor)
-
-        return u | z | x | y
-
-    raise error.Expr(INVALID_START, loc.Missing)
-
-
-def NextUtf8Char(s, i):
-    # type: (str, int) -> int
-    """Given a string and a byte offset, returns the byte position after the
-    character at this position.  Usually this is the position of the next
-    character, but for the last character in the string, it's the position just
-    past the end of the string.
-
-    Validates UTF-8.
-    """
-    n = len(s)
-    assert i < n, i  # should always be in range
-    byte_as_int = ord(s[i])
-    length = _Utf8CharLen(byte_as_int)
-    for j in xrange(i + 1, i + length):
-        if j >= n:
-            e_strict(INCOMPLETE_CHAR, loc.Missing)
-        _CheckContinuationByte(s[j])
-
-    return i + length
+        e_strict(_INVALID_START, loc.Missing)
 
 
 def PreviousUtf8Char(s, i):
@@ -197,17 +141,17 @@ def PreviousUtf8Char(s, i):
 
     while i > 0:
         i -= 1
-        byte_as_int = ord(s[i])
+        byte_as_int = mylib.ByteAt(s, i)
         if (byte_as_int >> 6) != 0b10:
             offset = orig_i - i
             if offset != _Utf8CharLen(byte_as_int):
                 # Leaving a generic error for now, but if we want to, it's not
                 # hard to calculate the position where things go wrong.  Note
                 # that offset might be more than 4, for an invalid utf-8 string.
-                e_strict(INVALID_START, loc.Missing)
+                e_strict(_INVALID_START, loc.Missing)
             return i
 
-    e_strict(INVALID_START, loc.Missing)
+    e_strict(_INVALID_START, loc.Missing)
 
 
 def CountUtf8Chars(s):
