@@ -4,11 +4,10 @@ tdop.py - Library for expression parsing.
 
 from _devbuild.gen.id_kind_asdl import Id, Id_t
 from _devbuild.gen.syntax_asdl import (loc, arith_expr, arith_expr_e,
-                                       arith_expr_t, word_t, CompoundWord,
-                                       NameTok)
+                                       arith_expr_t, word_e, word_t,
+                                       CompoundWord, Token)
 from core.error import p_die
 from core import ui
-from frontend import lexer
 from mycpp import mylib
 from mycpp.mylib import tagswitch
 from osh import word_
@@ -48,7 +47,7 @@ def CheckLhsExpr(node, blame_word):
     UP_node = node
     if node.tag() == arith_expr_e.Binary:
         node = cast(arith_expr.Binary, UP_node)
-        if node.op_id == Id.Arith_LBracket and IsIndexable(node.left):
+        if node.op.id == Id.Arith_LBracket and IsIndexable(node.left):
             return
         # But a[0][0] = 1 is NOT valid.
 
@@ -74,7 +73,7 @@ def NullConstant(p, w, bp):
     # type: (TdopParser, word_t, int) -> arith_expr_t
     name_tok = word_.LooksLikeArithVar(w)
     if name_tok:
-        return NameTok(name_tok, lexer.TokenVal(name_tok))
+        return name_tok
 
     # Id.Word_Compound in the spec ensures this cast is valid
     return cast(CompoundWord, w)
@@ -117,8 +116,11 @@ def LeftError(p, t, left, rbp):
 def LeftBinaryOp(p, w, left, rbp):
     # type: (TdopParser, word_t, arith_expr_t, int) -> arith_expr_t
     """Normal binary operator like 1+2 or 2*3, etc."""
-    # TODO: w should be a Token, and we should extract the token from it.
-    return arith_expr.Binary(word_.ArithId(w), left, p.ParseUntil(rbp))
+
+    assert w.tag() == word_e.Operator, w
+    tok = cast(Token, w)
+
+    return arith_expr.Binary(tok, left, p.ParseUntil(rbp))
 
 
 def LeftAssign(p, w, left, rbp):
@@ -259,7 +261,7 @@ class TdopParser(object):
     def CurrentId(self):
         # type: () -> Id_t
         """Glue used by the WordParser to check for extra tokens."""
-        return word_.CommandId(self.cur_word)
+        return word_.ArithId(self.cur_word)
 
     def AtToken(self, token_type):
         # type: (Id_t) -> bool
