@@ -21,22 +21,21 @@ from _devbuild.gen.syntax_asdl import (
     source,
     source_e,
 )
-from _devbuild.gen.value_asdl import (value, value_e, value_t, value_str)
+from _devbuild.gen.value_asdl import (value_t, value_str)
 from asdl import format as fmt
 from data_lang import pretty
 from frontend import lexer
 from frontend import location
 from mycpp import mylib
 from mycpp.mylib import print_stderr, tagswitch, log
-from data_lang import j8
 from data_lang import j8_lite
+import libc
 
 from typing import List, Optional, Any, cast, TYPE_CHECKING
 if TYPE_CHECKING:
     from _devbuild.gen import arg_types
     from core import error
     from core.error import _ErrorWithLocation
-    from mycpp.mylib import Writer
 
 _ = log
 
@@ -92,12 +91,26 @@ def PrettyDir(dir_name, home_dir):
 
 
 def _PrintCodeExcerpt(line, col, length, f):
-    # type: (str, int, int, Writer) -> None
+    # type: (str, int, int, mylib.Writer) -> None
 
     buf = mylib.BufWriter()
 
     buf.write('  ')
+
+    # TODO: Be smart about horizontal space when printing code snippet
+    # - Accept max_width param, which is terminal width or perhaps 100
+    #   when there's no terminal
+    # - If 'length' of token is greater than max_width, then perhaps print 10
+    #   chars on each side
+    # - If len(line) is less than max_width, then print everything normally
+    # - If len(line) is greater than max_width, then print up to max_width
+    #   but make sure to include the entire token, with some context
+    #   Print > < or ... to show truncation
+    #
+    #   ^col 80  ^~~~~ error
+
     buf.write(line.rstrip())
+
     buf.write('\n  ')
     # preserve tabs
     for c in line[:col]:
@@ -111,6 +124,34 @@ def _PrintCodeExcerpt(line, col, length, f):
     f.write(buf.getvalue())
 
 
+def _PrintTokenTooLong(loc_tok, f):
+    # type: (loc.TokenTooLong, mylib.Writer) -> None
+    line = loc_tok.line
+    col = loc_tok.col
+
+    buf = mylib.BufWriter()
+
+    buf.write('  ')
+    # Only print 10 characters, since it's probably very long
+    buf.write(line.content[:col + 10].rstrip())
+    buf.write('\n  ')
+
+    # preserve tabs, like _PrintCodeExcerpt
+    for c in line.content[:col]:
+        buf.write('\t' if c == '\t' else ' ')
+
+    buf.write('^\n')
+
+    source_str = GetLineSourceString(loc_tok.line, quote_filename=True)
+    buf.write(
+        '%s:%d: Token starting at column %d is too long: %d bytes (%s)\n' %
+        (source_str, line.line_num, loc_tok.col, loc_tok.length,
+         Id_str(loc_tok.id)))
+
+    # single write() call
+    f.write(buf.getvalue())
+
+
 def GetLineSourceString(line, quote_filename=False):
     # type: (SourceLine, bool) -> str
     """Returns a human-readable string for dev tools.
@@ -148,6 +189,8 @@ def GetLineSourceString(line, quote_filename=False):
         elif case(source_e.ArgvWord):
             src = cast(source.ArgvWord, UP_src)
 
+            # Note: _PrintWithLocation() uses this more specifically
+
             # TODO: check loc.Missing; otherwise get Token from loc_t, then line
             blame_tok = location.TokenFor(src.location)
             if blame_tok is None:
@@ -159,7 +202,6 @@ def GetLineSourceString(line, quote_filename=False):
                     line, quote_filename=quote_filename)
                 s = '[ %s word at line %d of %s ]' % (src.what, line_num,
                                                       outer_source)
-            # Note: _PrintCodeExcerpt called above
 
         elif case(source_e.Variable):
             src = cast(source.Variable, UP_src)
@@ -222,6 +264,10 @@ def _PrintWithLocation(prefix, msg, blame_loc, show_code):
     - and turn on "stack" tracing?  For 'source' and more?
     """
     f = mylib.Stderr()
+    if blame_loc.tag() == loc_e.TokenTooLong:
+        _PrintTokenTooLong(cast(loc.TokenTooLong, blame_loc), f)
+        return
+
     blame_tok = location.TokenFor(blame_loc)
     if blame_tok is None:  # When does this happen?
         f.write('[??? no location ???] %s%s\n' % (prefix, msg))
@@ -234,26 +280,55 @@ def _PrintWithLocation(prefix, msg, blame_loc, show_code):
 
     if show_code:
         UP_src = src
-        # LValue/backticks is the only case where we don't print this
-        if src.tag() == source_e.Reparsed:
-            src = cast(source.Reparsed, UP_src)
-            span2 = src.left_token
-            line_num = span2.line.line_num
 
-            # We want the excerpt to look like this:
-            #   a[x+]=1
-            #       ^
-            # Rather than quoting the internal buffer:
-            #   x+
-            #     ^
-            line2 = span2.line.content
-            lbracket_col = span2.col + span2.length
-            # NOTE: The inner line number is always 1 because of reparsing.  We
-            # overwrite it with the original span.
-            _PrintCodeExcerpt(line2, orig_col + lbracket_col, 1, f)
+        with tagswitch(src) as case:
+            if case(source_e.Reparsed):
+                # Special case for LValue/backticks
 
-        else:
-            _PrintCodeExcerpt(line, blame_tok.col, blame_tok.length, f)
+                # We want the excerpt to look like this:
+                #   a[x+]=1
+                #       ^
+                # Rather than quoting the internal buffer:
+                #   x+
+                #     ^
+
+                # Show errors:
+                #   test/parse-errors.sh text-arith-context
+
+                src = cast(source.Reparsed, UP_src)
+                tok2 = src.left_token
+                line_num = tok2.line.line_num
+
+                line2 = tok2.line.content
+                lbracket_col = tok2.col + tok2.length
+                # NOTE: The inner line number is always 1 because of reparsing.
+                # We overwrite it with the original token.
+                _PrintCodeExcerpt(line2, orig_col + lbracket_col, 1, f)
+
+            elif case(source_e.ArgvWord):
+                src = cast(source.ArgvWord, UP_src)
+                # Special case for eval, unset, printf -v, etc.
+
+                # Show errors:
+                #   test/runtime-errors.sh test-assoc-array
+
+                #print('OUTER blame_loc', blame_loc)
+                #print('OUTER tok', blame_tok)
+                #print('INNER src.location', src.location)
+
+                # Print code and location for MOST SPECIFIC location
+                _PrintCodeExcerpt(line, blame_tok.col, blame_tok.length, f)
+                source_str = GetLineSourceString(blame_tok.line,
+                                                 quote_filename=True)
+                f.write('%s:%d\n' % (source_str, line_num))
+                f.write('\n')
+
+                # Now print OUTER location, with error message
+                _PrintWithLocation(prefix, msg, src.location, show_code)
+                return
+
+            else:
+                _PrintCodeExcerpt(line, blame_tok.col, blame_tok.length, f)
 
     source_str = GetLineSourceString(blame_tok.line, quote_filename=True)
 
@@ -387,12 +462,35 @@ def PrintAst(node, flag):
         print_stderr('AST not printed.')
         if 0:
             from _devbuild.gen.id_kind_asdl import Id_str
-            from frontend.lexer import ID_HIST
+            from frontend.lexer import ID_HIST, LAZY_ID_HIST
+
+            print(LAZY_ID_HIST)
+            print(len(LAZY_ID_HIST))
+
             for id_, count in ID_HIST.most_common(10):
                 print('%8d %s' % (count, Id_str(id_)))
             print()
             total = sum(ID_HIST.values())
-            print('%8d total tokens returned' % total)
+            uniq = len(ID_HIST)
+            print('%8d total tokens' % total)
+            print('%8d unique tokens IDs' % uniq)
+            print()
+
+            for id_, count in LAZY_ID_HIST.most_common(10):
+                print('%8d %s' % (count, Id_str(id_)))
+            print()
+            total = sum(LAZY_ID_HIST.values())
+            uniq = len(LAZY_ID_HIST)
+            print('%8d total tokens' % total)
+            print('%8d tokens with LazyVal()' % total)
+            print('%8d unique tokens IDs' % uniq)
+            print()
+
+        if 0:
+            from osh.word_parse import WORD_HIST
+            #print(WORD_HIST)
+            for desc, count in WORD_HIST.most_common(20):
+                print('%8d %s' % (count, desc))
 
     else:  # text output
         f = mylib.Stdout()
@@ -424,39 +522,16 @@ def PrettyPrintValue(val, f):
     # type: (value_t, mylib.Writer) -> None
     """For the = keyword"""
 
-    ysh_type = ValType(val)
-    id_str = j8.ValueIdString(val)
+    printer = pretty.PrettyPrinter()
+    printer.SetUseStyles(f.isatty())
+    try:
+        width = libc.get_terminal_width()
+        if width > 0:
+            printer.SetMaxWidth(width)
+    except (IOError, OSError):
+        pass
 
-    UP_val = val
-    with tagswitch(val) as case:
-        # "JSON" data types will use J8 serialization
-        if case(value_e.Null, value_e.Bool, value_e.Int, value_e.Float,
-                value_e.Str, value_e.List, value_e.Dict):
-            # Use () instead of <> as a hint that it's a "JSON value"
-            f.write('(%s%s)   ' % (ysh_type, id_str))
-
-            # Unused STUB
-            doc = pretty.FromValue(val)
-
-            buf = mylib.BufWriter()
-
-            # TODO: Wrap lines, and show color.  Use core/ansi.py
-            p = j8.InstancePrinter(buf, -1, j8.SHOW_CYCLES | j8.SHOW_NON_DATA)
-
-            # error.Encode should be impossible - we show cycles and non-data
-            p.Print(val)
-
-            f.write(buf.getvalue())
-            f.write('\n')
-
-        elif case(value_e.Range):
-            val = cast(value.Range, UP_val)
-
-            # Printing Range values more nicely.  Note that pp line (x) doesn't
-            # have this.
-            f.write('(%s)   %d .. %d\n' % (ysh_type, val.lower, val.upper))
-
-        else:
-            # Just print object and ID.  Use <> to show that it's more like
-            # a reference type.
-            f.write('<%s%s>\n' % (ysh_type, id_str))
+    buf = mylib.BufWriter()
+    printer.PrintValue(val, buf)
+    f.write(buf.getvalue())
+    f.write('\n')
