@@ -5,7 +5,7 @@ word_eval.py - Evaluator for the word language.
 from _devbuild.gen.id_kind_asdl import Id, Kind, Kind_str
 from _devbuild.gen.syntax_asdl import (
     Token,
-    NameTok,
+    SimpleVarSub,
     loc,
     loc_t,
     BracedVarSub,
@@ -38,6 +38,7 @@ from _devbuild.gen.runtime_asdl import (
     a_index_e,
     VTestPlace,
     VarSubState,
+    Piece,
 )
 from _devbuild.gen.option_asdl import option_i
 from _devbuild.gen.value_asdl import (
@@ -53,6 +54,7 @@ from core import pyutil
 from core import state
 from core import ui
 from core import util
+from data_lang import j8
 from data_lang import j8_lite
 from core.error import e_die
 from frontend import consts
@@ -64,7 +66,6 @@ from osh import braces
 from osh import glob_
 from osh import string_ops
 from osh import word_
-from osh import word_compile
 from ysh import expr_eval
 from ysh import val_ops
 
@@ -169,7 +170,7 @@ def _SplitAssignArg(arg, blame_word):
     """
     # Note: it would be better to cache regcomp(), but we don't have an API for
     # that, and it probably isn't a bottleneck now
-    m = util.simple_regex_search(ASSIGN_ARG_RE, arg)
+    m = util.RegexSearch(ASSIGN_ARG_RE, arg)
     if m is None:
         e_die("Assignment builtin expected NAME=value, got %r" % arg,
               blame_word)
@@ -178,7 +179,8 @@ def _SplitAssignArg(arg, blame_word):
     # m[2] is used for grouping; ERE doesn't have non-capturing groups
 
     op = m[3]
-    if op is not None and len(op):  # declare NAME=
+    assert op is not None, op
+    if len(op):  # declare NAME=
         val = value.Str(m[4])  # type: Optional[value_t]
         append = op[0] == '+'
     else:  # declare NAME
@@ -211,11 +213,11 @@ def _ValueToPartValue(val, quoted, part_loc):
         if case(value_e.Undef):
             # This happens in the case of ${undef+foo}.  We skipped _EmptyStrOrError,
             # but we have to append to the empty string.
-            return part_value.String('', quoted, not quoted)
+            return Piece('', quoted, not quoted)
 
         elif case(value_e.Str):
             val = cast(value.Str, UP_val)
-            return part_value.String(val.s, quoted, not quoted)
+            return Piece(val.s, quoted, not quoted)
 
         elif case(value_e.BashArray):
             val = cast(value.BashArray, UP_val)
@@ -231,7 +233,7 @@ def _ValueToPartValue(val, quoted, part_loc):
         elif case(value_e.Null, value_e.Bool, value_e.Int, value_e.Float,
                   value_e.Eggex, value_e.List):
             s = val_ops.Stringify(val, loc.Missing)
-            return part_value.String(s, quoted, not quoted)
+            return Piece(s, quoted, not quoted)
 
         else:
             raise error.TypeErr(val, "Can't substitute into word",
@@ -241,7 +243,7 @@ def _ValueToPartValue(val, quoted, part_loc):
 
 
 def _MakeWordFrames(part_vals):
-    # type: (List[part_value_t]) -> List[List[Tuple[str, bool, bool]]]
+    # type: (List[part_value_t]) -> List[List[Piece]]
     """A word evaluates to a flat list of part_value (String or Array).  frame
     is a portion that results in zero or more args.  It can never be joined.
     This idea exists because of arrays like "$@" and "${a[@]}".
@@ -260,10 +262,21 @@ def _MakeWordFrames(part_vals):
       [ ('2 3', True, False) ]
       [ ('4', True, False), ('y', False, True) ]
 
-    Note: A frame is a 3-tuple that's identical to part_value.String()?  Maybe we
+    Note: A frame is a 3-tuple that's identical to Piece()?  Maybe we
     should make that top level type.
+
+    TODO:
+    - Instead of List[List[Piece]], where List[Piece] is a Frame
+    - Change this representation to
+      Frames = (List[Piece] pieces, List[int] break_indices) 
+      # where break_indices are the end
+
+      Consider a common case like "$x" or "${x}" - I think this a lot more
+      efficient?
+
+    And then change _EvalWordFrame(pieces: List[Piece], start: int, end: int)
     """
-    current = []  # type: List[Tuple[str, bool, bool]]
+    current = []  # type: List[Piece]
     frames = [current]
 
     for p in part_vals:
@@ -271,8 +284,8 @@ def _MakeWordFrames(part_vals):
 
         with tagswitch(p) as case:
             if case(part_value_e.String):
-                p = cast(part_value.String, UP_p)
-                current.append((p.s, p.quoted, p.do_split))
+                p = cast(Piece, UP_p)
+                current.append(p)
 
             elif case(part_value_e.Array):
                 p = cast(part_value.Array, UP_p)
@@ -284,12 +297,12 @@ def _MakeWordFrames(part_vals):
 
                     # Arrays parts are always quoted; otherwise they would have decayed to
                     # a string.
-                    portion = (s, True, False)
+                    piece = Piece(s, True, False)
                     if is_first:
-                        current.append(portion)
+                        current.append(piece)
                         is_first = False
                     else:
-                        current = [portion]
+                        current = [piece]
                         frames.append(current)  # singleton frame
 
             else:
@@ -307,7 +320,7 @@ def _DecayPartValuesToString(part_vals, join_char):
         UP_p = p
         with tagswitch(p) as case:
             if case(part_value_e.String):
-                p = cast(part_value.String, UP_p)
+                p = cast(Piece, UP_p)
                 out.append(p.s)
             elif case(part_value_e.Array):
                 p = cast(part_value.Array, UP_p)
@@ -621,6 +634,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                     is_falsey = False
             elif case(value_e.BashArray):
                 val = cast(value.BashArray, UP_val)
+                # TODO: allow undefined
                 is_falsey = len(val.strs) == 0
             elif case(value_e.BashAssoc):
                 val = cast(value.BashAssoc, UP_val)
@@ -706,7 +720,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
             raise AssertionError(tok.id)
 
     def _Length(self, val, token):
-        # type: (value_t, Token) -> value_t
+        # type: (value_t, Token) -> int
         """Returns the length of the value, for ${#var}"""
         UP_val = val
         with tagswitch(val) as case:
@@ -732,7 +746,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                         # NOTE: Doesn't make the command exit with 1; it just returns a
                         # length of -1.
                         self.errfmt.PrettyPrintError(e, prefix='warning: ')
-                        return value.Str('-1')
+                        return -1
 
             elif case(value_e.BashArray):
                 val = cast(value.BashArray, UP_val)
@@ -750,7 +764,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 raise error.TypeErr(
                     val, "Length op expected Str, BashArray, BashAssoc", token)
 
-        return value.Str(str(length))
+        return length
 
     def _Keys(self, val, token):
         # type: (value_t, Token) -> value_t
@@ -1020,29 +1034,26 @@ class AbstractWordEvaluator(StringWordEvaluator):
             UP_val = val
             with tagswitch(val) as case2:
                 if case2(value_e.Undef):
-                    val = self._EmptyBashArrayOrError(part.token)
+                    if not vsub_state.has_test_op:
+                        val = self._EmptyBashArrayOrError(part.token)
                 elif case2(value_e.Str):
-                    val = cast(value.Str, UP_val)
-                    e_die("Can't index string with @", loc.WordPart(part))
+                    if self.exec_opts.strict_array():
+                        e_die("Can't index string with @", loc.WordPart(part))
                 elif case2(value_e.BashArray):
-                    val = cast(value.BashArray, UP_val)
-                    # TODO: Is this a no-op?  Just leave 'val' alone.
-                    val = value.BashArray(val.strs)
+                    pass  # no-op
 
         elif op_id == Id.Arith_Star:
             vsub_state.join_array = True  # both ${a[*]} and "${a[*]}" decay
             UP_val = val
             with tagswitch(val) as case2:
                 if case2(value_e.Undef):
-                    val = self._EmptyBashArrayOrError(part.token)
+                    if not vsub_state.has_test_op:
+                        val = self._EmptyBashArrayOrError(part.token)
                 elif case2(value_e.Str):
-                    val = cast(value.Str, UP_val)
-                    e_die("Can't index string with *", loc.WordPart(part))
+                    if self.exec_opts.strict_array():
+                        e_die("Can't index string with *", loc.WordPart(part))
                 elif case2(value_e.BashArray):
-                    val = cast(value.BashArray, UP_val)
-                    # TODO: Is this a no-op?  Just leave 'val' alone.
-                    # ${a[*]} or "${a[*]}" :  vsub_state.join_array is always true
-                    val = value.BashArray(val.strs)
+                    pass  # no-op
 
         else:
             raise AssertionError(op_id)  # unknown
@@ -1079,7 +1090,11 @@ class AbstractWordEvaluator(StringWordEvaluator):
 
             elif case2(value_e.BashAssoc):
                 assoc_val = cast(value.BashAssoc, UP_val)
-                key = self.arith_ev.EvalWordToString(anode)
+                # Location could also be attached to bracket_op?  But
+                # arith_expr.VarSub works OK too
+                key = self.arith_ev.EvalWordToString(
+                    anode, blame_loc=location.TokenForArith(anode))
+
                 vtest_place.index = a_index.Str(key)  # out param
                 s = assoc_val.d.get(key)
 
@@ -1115,7 +1130,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         # of (DoubleQuoted [Literal '']).  This is better but it means we
         # have to check for it.
         if len(parts) == 0:
-            v = part_value.String('', True, False)
+            v = Piece('', True, False)
             part_vals.append(v)
             return
 
@@ -1223,9 +1238,9 @@ class AbstractWordEvaluator(StringWordEvaluator):
     def _EvalBracedVarSub(self, part, part_vals, quoted):
         # type: (BracedVarSub, List[part_value_t], bool) -> None
         """
-    Args:
-      part_vals: output param to append to.
-    """
+        Args:
+          part_vals: output param to append to.
+        """
         # We have different operators that interact in a non-obvious order.
         #
         # 1. bracket_op: value -> value, with side effect on vsub_state
@@ -1282,8 +1297,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                         part_vals.append(part_value.Array(names))
                     else:
                         sep = self.splitter.GetJoinChar()
-                        part_vals.append(
-                            part_value.String(sep.join(names), quoted, True))
+                        part_vals.append(Piece(sep.join(names), quoted, True))
                     return  # EARLY RETURN
 
             var_name = part.var_name
@@ -1298,41 +1312,43 @@ class AbstractWordEvaluator(StringWordEvaluator):
             # $* decays
             val = self._EvalSpecialVar(part.token.id, quoted, vsub_state)
 
-        # Type query ${array@a} is a STRING, not an array
-        # NOTE: ${array@Q} is ${array[0]@Q} in bash, which is different than
-        # ${array[@]@Q}
-        # TODO: An IR for ${} might simplify these lengthy conditions
         suffix_op_ = part.suffix_op
-        if (suffix_op_ and suffix_op_.tag() == suffix_op_e.Nullary and
-                cast(Token, suffix_op_).id == Id.VOp0_a):
-            vsub_state.is_type_query = True
+        if suffix_op_:
+            UP_op = suffix_op_
+            with tagswitch(suffix_op_) as case:
+                if case(suffix_op_e.Nullary):
+                    suffix_op_ = cast(Token, UP_op)
+
+                    # Type query ${array@a} is a STRING, not an array
+                    # NOTE: ${array@Q} is ${array[0]@Q} in bash, which is different than
+                    # ${array[@]@Q}
+                    if suffix_op_.id == Id.VOp0_a:
+                        vsub_state.is_type_query = True
+
+                elif case(suffix_op_e.Unary):
+                    suffix_op_ = cast(suffix_op.Unary, UP_op)
+
+                    # Do the _EmptyStrOrError/_EmptyBashArrayOrError up front, EXCEPT in
+                    # the case of Kind.VTest
+                    if consts.GetKind(suffix_op_.op.id) == Kind.VTest:
+                        vsub_state.has_test_op = True
 
         # 2. Bracket Op
         val = self._EvalBracketOp(val, part, quoted, vsub_state, vtest_place)
 
-        # Do the _EmptyStrOrError up front here, EXCEPT in the case of Kind.VTest
-        suffix_is_test = False
-        UP_op = suffix_op_
-        if suffix_op_ is not None and suffix_op_.tag() == suffix_op_e.Unary:
-            suffix_op_ = cast(suffix_op.Unary, UP_op)
-            if consts.GetKind(suffix_op_.op.id) == Kind.VTest:
-                suffix_is_test = True
-
         if part.prefix_op:
             if part.prefix_op.id == Id.VSub_Pound:  # ${#var} for length
-                if not suffix_is_test:  # undef -> '' BEFORE length
+                if not vsub_state.has_test_op:  # undef -> '' BEFORE length
                     val = self._EmptyStrOrError(val, part.token)
 
-                val = self._Length(val, part.token)
-                # assume it's not quoted
-                part_val = _ValueToPartValue(val, False, part)
-                part_vals.append(part_val)
+                n = self._Length(val, part.token)
+                part_vals.append(Piece(str(n), quoted, False))
                 return  # EARLY EXIT: nothing else can come after length
 
             elif part.prefix_op.id == Id.VSub_Bang:
-                if part.bracket_op and part.bracket_op.tag(
-                ) == bracket_op_e.WholeArray:
-                    if suffix_is_test:
+                if (part.bracket_op and
+                        part.bracket_op.tag() == bracket_op_e.WholeArray):
+                    if vsub_state.has_test_op:
                         # ${!a[@]-'default'} is a non-fatal runtime error in bash.  Here
                         # it's fatal.
                         op_tok = cast(suffix_op.Unary, UP_op).op
@@ -1354,14 +1370,14 @@ class AbstractWordEvaluator(StringWordEvaluator):
                     val = self._EvalVarRef(val, part.token, quoted, vsub_state,
                                            vtest_place)
 
-                    if not suffix_is_test:  # undef -> '' AFTER indirection
+                    if not vsub_state.has_test_op:  # undef -> '' AFTER indirection
                         val = self._EmptyStrOrError(val, part.token)
 
             else:
                 raise AssertionError(part.prefix_op)
 
         else:
-            if not suffix_is_test:  # undef -> '' if no prefix op
+            if not vsub_state.has_test_op:  # undef -> '' if no prefix op
                 val = self._EmptyStrOrError(val, part.token)
 
         quoted2 = False  # another bit for @Q
@@ -1411,19 +1427,19 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 val = array_val
 
         # For example, ${a} evaluates to value.Str(), but we want a
-        # part_value.String().
+        # Piece().
         part_val = _ValueToPartValue(val, quoted or quoted2, part)
         part_vals.append(part_val)
 
     def _ConcatPartVals(self, part_vals, location):
         # type: (List[part_value_t], loc_t) -> str
-        """Helper."""
+
         strs = []  # type: List[str]
         for part_val in part_vals:
             UP_part_val = part_val
             with tagswitch(part_val) as case:
                 if case(part_value_e.String):
-                    part_val = cast(part_value.String, UP_part_val)
+                    part_val = cast(Piece, UP_part_val)
                     s = part_val.s
 
                 elif case(part_value_e.Array):
@@ -1457,16 +1473,15 @@ class AbstractWordEvaluator(StringWordEvaluator):
         return self._ConcatPartVals(part_vals, part.left)
 
     def _EvalSimpleVarSub(self, part, part_vals, quoted):
-        # type: (NameTok, List[part_value_t], bool) -> None
+        # type: (SimpleVarSub, List[part_value_t], bool) -> None
 
-        # TODO: use name
-        token = part.left
-        var_name = part.var_name
+        token = part.tok
 
         vsub_state = VarSubState.CreateNull()
 
         # 1. Evaluate from (var_name, var_num, Token) -> defined, value
         if token.id == Id.VSub_DollarName:
+            var_name = lexer.LazyStr(token)
             # TODO: Special case for LINENO
             val = self.mem.GetValue(var_name)
             if val.tag() in (value_e.BashArray, value_e.BashAssoc):
@@ -1479,7 +1494,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                         % var_name, token)
 
         elif token.id == Id.VSub_Number:
-            var_num = int(var_name)
+            var_num = int(lexer.LazyStr(token))
             val = self._EvalVarNum(var_num)
 
         else:
@@ -1499,14 +1514,14 @@ class AbstractWordEvaluator(StringWordEvaluator):
         part_vals.append(v)
 
     def EvalSimpleVarSubToString(self, node):
-        # type: (NameTok) -> str
+        # type: (SimpleVarSub) -> str
         """For double quoted strings in YSH expressions.
 
         Example: var x = "$foo-${foo}"
         """
         part_vals = []  # type: List[part_value_t]
         self._EvalSimpleVarSub(node, part_vals, False)
-        return self._ConcatPartVals(part_vals, node.left)
+        return self._ConcatPartVals(part_vals, node.tok)
 
     def _EvalExtGlob(self, part, part_vals):
         # type: (word_part.ExtGlob, List[part_value_t]) -> None
@@ -1515,17 +1530,16 @@ class AbstractWordEvaluator(StringWordEvaluator):
         if op.id == Id.ExtGlob_Comma:
             op_str = '@('
         else:
-            op_str = lexer.TokenVal(op)
+            op_str = lexer.LazyStr(op)
         # Do NOT split these.
-        part_vals.append(part_value.String(op_str, False, False))
+        part_vals.append(Piece(op_str, False, False))
 
         for i, w in enumerate(part.arms):
             if i != 0:
-                part_vals.append(part_value.String('|', False,
-                                                   False))  # separator
+                part_vals.append(Piece('|', False, False))  # separator
             # FLATTEN the tree of extglob "arms".
             self._EvalWordToParts(w, part_vals, EXTGLOB_NESTED)
-        part_vals.append(part_value.String(')', False, False))  # closing )
+        part_vals.append(Piece(')', False, False))  # closing )
 
     def _TranslateExtGlob(self, part_vals, w, glob_parts, fnmatch_parts):
         # type: (List[part_value_t], CompoundWord, List[str], List[str]) -> None
@@ -1538,7 +1552,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
             UP_part_val = part_val
             with tagswitch(part_val) as case:
                 if case(part_value_e.String):
-                    part_val = cast(part_value.String, UP_part_val)
+                    part_val = cast(Piece, UP_part_val)
                     if part_val.quoted and not self.exec_opts.noglob():
                         s = glob_.GlobEscape(part_val.s)
                     else:
@@ -1565,18 +1579,9 @@ class AbstractWordEvaluator(StringWordEvaluator):
 
     def _EvalWordPart(self, part, part_vals, flags):
         # type: (word_part_t, List[part_value_t], int) -> None
-        """Evaluate a word part.
+        """Evaluate a word part, appending to part_vals
 
         Called by _EvalWordToParts, EvalWordToString, and _EvalDoubleQuoted.
-
-        Args:
-          part: What to evaluate
-          part_vals: Output parameter.
-          quoted: was the part quoted like "$x"
-          is_subst: do_split
-
-        Returns:
-          None
         """
         quoted = bool(flags & QUOTED)
         is_subst = bool(flags & IS_SUBST)
@@ -1595,18 +1600,17 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 part = cast(Token, UP_part)
                 # Split if it's in a substitution.
                 # That is: echo is not split, but ${foo:-echo} is split
-                v = part_value.String(part.tval, quoted, is_subst)
+                v = Piece(lexer.LazyStr(part), quoted, is_subst)
                 part_vals.append(v)
 
             elif case(word_part_e.EscapedLiteral):
                 part = cast(word_part.EscapedLiteral, UP_part)
-                v = part_value.String(part.ch, True, False)
+                v = Piece(part.ch, True, False)
                 part_vals.append(v)
 
             elif case(word_part_e.SingleQuoted):
                 part = cast(SingleQuoted, UP_part)
-                s = word_compile.EvalSingleQuoted(part)
-                v = part_value.String(s, True, False)
+                v = Piece(part.sval, True, False)
                 part_vals.append(v)
 
             elif case(word_part_e.DoubleQuoted):
@@ -1630,7 +1634,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 part_vals.append(sv)
 
             elif case(word_part_e.SimpleVarSub):
-                part = cast(NameTok, UP_part)
+                part = cast(SimpleVarSub, UP_part)
                 self._EvalSimpleVarSub(part, part_vals, quoted)
 
             elif case(word_part_e.BracedVarSub):
@@ -1642,14 +1646,13 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 # We never parse a quoted string into a TildeSub.
                 assert not quoted
                 s = self.tilde_ev.Eval(part)
-                v = part_value.String(s, True,
-                                      False)  # NOT split even when unquoted!
+                v = Piece(s, True, False)  # NOT split even when unquoted!
                 part_vals.append(v)
 
             elif case(word_part_e.ArithSub):
                 part = cast(word_part.ArithSub, UP_part)
                 num = self.arith_ev.EvalToBigInt(part.anode)
-                v = part_value.String(mops.ToStr(num), quoted, not quoted)
+                v = Piece(mops.ToStr(num), quoted, not quoted)
                 part_vals.append(v)
 
             elif case(word_part_e.ExtGlob):
@@ -1663,6 +1666,14 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 self._EvalExtGlob(part, part_vals2)  # flattens tree
                 part_vals.append(part_value.ExtGlob(part_vals2))
 
+            elif case(word_part_e.BashRegexGroup):
+                part = cast(word_part.BashRegexGroup, UP_part)
+
+                part_vals.append(Piece('(', False, False))  # not quoted
+                if part.child:
+                    self._EvalWordToParts(part.child, part_vals, 0)
+                part_vals.append(Piece(')', False, False))
+
             elif case(word_part_e.Splice):
                 part = cast(word_part.Splice, UP_part)
                 val = self.mem.GetValue(part.var_name)
@@ -1675,6 +1686,11 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 part_val = self.expr_ev.EvalExprSub(part)
                 part_vals.append(part_val)
 
+            elif case(word_part_e.ZshVarSub):
+                part = cast(word_part.ZshVarSub, UP_part)
+                e_die("ZSH var subs are parsed, but can't be evaluated",
+                      part.left)
+
             else:
                 raise AssertionError(part.tag())
 
@@ -1685,7 +1701,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         UP_w = w
         with tagswitch(w) as case:
             if case(rhs_word_e.Empty):
-                part_vals.append(part_value.String('', quoted, not quoted))
+                part_vals.append(Piece('', quoted, not quoted))
 
             elif case(rhs_word_e.Compound):
                 w = cast(CompoundWord, UP_w)
@@ -1761,7 +1777,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
             UP_part_val = part_val
             with tagswitch(part_val) as case:
                 if case(part_value_e.String):
-                    part_val = cast(part_value.String, UP_part_val)
+                    part_val = cast(Piece, UP_part_val)
                     s = part_val.s
                     if part_val.quoted:
                         if eval_flags & QUOTE_FNMATCH:
@@ -1824,6 +1840,8 @@ class AbstractWordEvaluator(StringWordEvaluator):
             # Could we additionally optimize a=$b, if we know $b isn't an array
             # etc.?
 
+        # Note: these empty lists are hot in fib benchmark
+
         part_vals = []  # type: List[part_value_t]
         for p in w.parts:
             # this doesn't use eval_flags, which is slightly confusing
@@ -1915,18 +1933,18 @@ class AbstractWordEvaluator(StringWordEvaluator):
         return self.EvalWordToString(w)
 
     def _EvalWordFrame(self, frame, argv):
-        # type: (List[Tuple[str, bool, bool]], List[str]) -> None
+        # type: (List[Piece], List[str]) -> None
         all_empty = True
         all_quoted = True
         any_quoted = False
 
         #log('--- frame %s', frame)
 
-        for s, quoted, _ in frame:
-            if len(s):
+        for piece in frame:
+            if len(piece.s):
                 all_empty = False
 
-            if quoted:
+            if piece.quoted:
                 any_quoted = True
             else:
                 all_quoted = False
@@ -1938,7 +1956,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         # If every frag is quoted, e.g. "$a$b" or any part in "${a[@]}"x, then
         # don't do word splitting or globbing.
         if all_quoted:
-            tmp = [s for s, _, _ in frame]
+            tmp = [piece.s for piece in frame]
             a = ''.join(tmp)
             argv.append(a)
             return
@@ -1947,16 +1965,16 @@ class AbstractWordEvaluator(StringWordEvaluator):
 
         # Array of strings, some of which are BOTH IFS-escaped and GLOB escaped!
         frags = []  # type: List[str]
-        for frag, quoted, do_split in frame:
-            if will_glob and quoted:
-                frag = glob_.GlobEscape(frag)
+        for piece in frame:
+            if will_glob and piece.quoted:
+                frag = glob_.GlobEscape(piece.s)
             else:
                 # If we have a literal \, then we turn it into \\\\.
                 # Splitting takes \\\\ -> \\
                 # Globbing takes \\ to \ if it doesn't match
-                frag = _BackslashEscape(frag)
+                frag = _BackslashEscape(piece.s)
 
-            if do_split:
+            if piece.do_split:
                 frag = _BackslashEscape(frag)
             else:
                 frag = self.splitter.Escape(frag)
@@ -2000,7 +2018,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         argv = []  # type: List[str]
         for frame in frames:
             if len(frame):  # empty array gives empty frame!
-                tmp = [s for (s, _, _) in frame]
+                tmp = [piece.s for piece in frame]
                 argv.append(''.join(tmp))  # no split or glob
         #log('argv: %s', argv)
         return argv
@@ -2145,11 +2163,11 @@ class AbstractWordEvaluator(StringWordEvaluator):
             # disallows such expressions at parse time.
             for frame in frames:
                 if len(frame):  # empty array gives empty frame!
-                    tmp = [s for (s, _, _) in frame]
+                    tmp = [piece.s for piece in frame]
                     strs.append(''.join(tmp))  # no split or glob
                     locs.append(w)
 
-        return cmd_value.Argv(strs, locs, None, None, None)
+        return cmd_value.Argv(strs, locs, None, None, None, None)
 
     def EvalWordSequence2(self, words, allow_assign=False):
         # type: (List[CompoundWord], bool) -> cmd_value_t
@@ -2213,7 +2231,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 val0 = part_vals[0]
                 UP_val0 = val0
                 if val0.tag() == part_value_e.String:
-                    val0 = cast(part_value.String, UP_val0)
+                    val0 = cast(Piece, UP_val0)
                     if not val0.quoted:
                         builtin_id = consts.LookupAssignBuiltin(val0.s)
                         if builtin_id != consts.NO_INDEX:
@@ -2246,7 +2264,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         # A non-assignment command.
         # NOTE: Can't look up builtins here like we did for assignment, because
         # functions can override builtins.
-        return cmd_value.Argv(strs, locs, None, None, None)
+        return cmd_value.Argv(strs, locs, None, None, None, None)
 
     def EvalWordSequence(self, words):
         # type: (List[CompoundWord]) -> List[str]
@@ -2288,17 +2306,25 @@ class NormalWordEvaluator(AbstractWordEvaluator):
     def _EvalCommandSub(self, cs_part, quoted):
         # type: (CommandSub, bool) -> part_value_t
         stdout_str = self.shell_ex.RunCommandSub(cs_part)
+
         if cs_part.left_token.id == Id.Left_AtParen:
-            strs = self.splitter.SplitForWordEval(stdout_str)
+            # YSH splitting algorithm: does not depend on IFS
+            try:
+                strs = j8.SplitJ8Lines(stdout_str)
+            except error.Decode as e:
+                # status code 4 is special, for encode/decode errors.
+                raise error.Structured(4, e.Message(), cs_part.left_token)
+
+            #strs = self.splitter.SplitForWordEval(stdout_str)
             return part_value.Array(strs)
         else:
-            return part_value.String(stdout_str, quoted, not quoted)
+            return Piece(stdout_str, quoted, not quoted)
 
     def _EvalProcessSub(self, cs_part):
-        # type: (CommandSub) -> part_value.String
+        # type: (CommandSub) -> Piece
         dev_path = self.shell_ex.RunProcessSub(cs_part)
         # pretend it's quoted; no split or glob
-        return part_value.String(dev_path, True, False)
+        return Piece(dev_path, True, False)
 
 
 _DUMMY = '__NO_COMMAND_SUB__'
@@ -2337,12 +2363,12 @@ class CompletionWordEvaluator(AbstractWordEvaluator):
         if cs_part.left_token.id == Id.Left_AtParen:
             return part_value.Array([_DUMMY])
         else:
-            return part_value.String(_DUMMY, quoted, not quoted)
+            return Piece(_DUMMY, quoted, not quoted)
 
     def _EvalProcessSub(self, cs_part):
-        # type: (CommandSub) -> part_value.String
+        # type: (CommandSub) -> Piece
         # pretend it's quoted; no split or glob
-        return part_value.String('__NO_PROCESS_SUB__', True, False)
+        return Piece('__NO_PROCESS_SUB__', True, False)
 
 
 # vim: sw=4
