@@ -54,7 +54,7 @@ from _devbuild.gen.types_asdl import (lex_mode_t, lex_mode_e)
 from _devbuild.gen.syntax_asdl import (
     BoolParamBox,
     Token,
-    NameTok,
+    SimpleVarSub,
     loc,
     source,
     DoubleQuoted,
@@ -180,21 +180,35 @@ class WordParser(WordEmitter):
         # type: () -> None
         """Call this when you need to make a decision based on any of:
 
-        self.token_type self.token_kind self.cur_token  # contents
+        self.token_type
+        self.token_kind
+        self.cur_token
         """
-        if self.next_lex_mode != lex_mode_e.Undefined:
-            self.cur_token = self.lexer.Read(self.next_lex_mode)
-            self.token_type = self.cur_token.id
-            self.token_kind = consts.GetKind(self.token_type)
+        if self.next_lex_mode == lex_mode_e.Undefined:
+            return  # _SetNext() not called, so do nothing
 
-            # number of consecutive newlines, ignoring whitespace
-            if self.token_type == Id.Op_Newline:
-                self.newline_state += 1
-            elif self.token_kind != Kind.WS:
-                self.newline_state = 0
+        is_fake = self.next_lex_mode == lex_mode_e.BashRegexFakeInner
+        real_mode = (lex_mode_e.BashRegex if is_fake else self.next_lex_mode)
 
-            self.parse_ctx.trail.AppendToken(self.cur_token)  # For completion
-            self.next_lex_mode = lex_mode_e.Undefined
+        self.cur_token = self.lexer.Read(real_mode)
+
+        # MUTATE TOKEN for fake lexer mode.
+        # This is for crazy stuff bash allows, like [[ s =~ (< >) ]]
+        if (is_fake and self.cur_token.id
+                in (Id.WS_Space, Id.BashRegex_AllowedInParens)):
+            self.cur_token.id = Id.Lit_Chars
+
+        self.token_type = self.cur_token.id
+        self.token_kind = consts.GetKind(self.token_type)
+
+        # number of consecutive newlines, ignoring whitespace
+        if self.token_type == Id.Op_Newline:
+            self.newline_state += 1
+        elif self.token_kind != Kind.WS:
+            self.newline_state = 0
+
+        self.parse_ctx.trail.AppendToken(self.cur_token)  # For completion
+        self.next_lex_mode = lex_mode_e.Undefined
 
     def _SetNext(self, lex_mode):
         # type: (lex_mode_t) -> None
@@ -393,7 +407,7 @@ class WordParser(WordEmitter):
 
             part.suffix_op = suffix_op.Unary(tok, arg_word)
 
-        elif op_kind == Kind.VOpOil:
+        elif op_kind == Kind.VOpYsh:
             tok = self.cur_token
             arg_word = self._ReadVarOpArg(arg_lex_mode)
             if self.token_type != Id.Right_DollarBrace:
@@ -466,6 +480,17 @@ class WordParser(WordEmitter):
         # Now look for ops
         return part
 
+    def _ReadZshVarSub(self, left_token):
+        # type: (Token) -> word_part.ZshVarSub
+
+        self._SetNext(lex_mode_e.VSub_Zsh)  # Move past ${(foo)
+
+        # Can be empty
+        w = self._ReadCompoundWord3(lex_mode_e.VSub_Zsh, Id.Right_DollarBrace,
+                                    True)
+        self._GetToken()
+        return word_part.ZshVarSub(left_token, w, self.cur_token)
+
     def ReadBracedVarSub(self, left_token):
         # type: (Token) -> Tuple[BracedVarSub, Token]
         """   For YSH expressions like var x = ${x:-"default"}.  """
@@ -609,17 +634,20 @@ class WordParser(WordEmitter):
         """Internal method to read a word_part."""
         tokens = []  # type: List[Token]
         # In command mode, we never disallow backslashes like '\'
-        self.ReadSingleQuoted(lex_mode, left_token, tokens, False)
-        right_quote = self.cur_token
-        node = SingleQuoted(left_token, tokens, right_quote)
+        right_quote = self.ReadSingleQuoted(lex_mode, left_token, tokens,
+                                            False)
+        sval = word_compile.EvalSingleQuoted(left_token.id, tokens)
+        node = SingleQuoted(left_token, sval, right_quote)
         return node
 
-    def ReadSingleQuoted(self, lex_mode, left_token, tokens, is_ysh_expr):
+    def ReadSingleQuoted(self, lex_mode, left_token, out_tokens, is_ysh_expr):
         # type: (lex_mode_t, Token, List[Token], bool) -> Token
-        """Appends to tokens
+        """Appends to out_tokens; returns last token
 
         Used by expr_parse.py
         """
+        # TODO: Remove and use out_tokens
+        tokens = []  # type: List[Token]
 
         # echo '\' is allowed, but x = '\' is invalid, in favor of x = r'\'
         no_backslashes = is_ysh_expr and left_token.id == Id.Left_SingleQuote
@@ -633,12 +661,12 @@ class WordParser(WordEmitter):
             self._SetNext(lex_mode)
             self._GetToken()
 
-            # Kind.Char emitted in DOLLAR_SQ state
+            # Kind.Char emitted in lex_mode.SQ_C
             if self.token_kind in (Kind.Lit, Kind.Char):
                 tok = self.cur_token
                 # Happens in lex_mode_e.SQ: 'one\two' is ambiguous, should be
                 # r'one\two' or c'one\\two'
-                if no_backslashes and '\\' in tok.tval:
+                if no_backslashes and lexer.TokenContains(tok, '\\'):
                     p_die(
                         r"Strings with backslashes should look like r'\n' or u'\n' or b'\n'",
                         tok)
@@ -708,15 +736,8 @@ class WordParser(WordEmitter):
                 p_die(
                     r"%s escapes not allowed in u'' strings" %
                     lexer.TokenVal(tok), tok)
-            # \u{dc00} isn't valid
-            if tok.id == Id.Char_UBraced:
-                h = lexer.TokenSlice(tok, 3, -1)  # \u{123456}
-                i = int(h, 16)
-                if 0xD800 <= i and i < 0xE000:
-                    p_die(
-                        r"%s escape is illegal because it's in the surrogate range"
-                        % lexer.TokenVal(tok), tok)
 
+        out_tokens.extend(tokens)
         return self.cur_token
 
     def _ReadDoubleQuotedLeftParts(self):
@@ -762,7 +783,7 @@ class WordParser(WordEmitter):
 
         sq_part = self._ReadSingleQuoted(left_tok, lexer_mode)
 
-        if (len(sq_part.tokens) == 0 and self.lexer.ByteLookAhead() == "'"):
+        if (len(sq_part.sval) == 0 and self.lexer.ByteLookAhead() == "'"):
             self._SetNext(lex_mode_e.ShCommand)
             self._GetToken()
 
@@ -829,7 +850,7 @@ class WordParser(WordEmitter):
             # Got empty '' or r'' and there's a ' after
             # u'' and b'' are handled in _ReadYshSingleQuoted
             if (triple_left_id != Id.Undefined_Tok and
-                    triple_out is not None and len(sq_part.tokens) == 0 and
+                    triple_out is not None and len(sq_part.sval) == 0 and
                     self.lexer.ByteLookAhead() == "'"):
 
                 self._SetNext(lex_mode_e.ShCommand)
@@ -858,6 +879,9 @@ class WordParser(WordEmitter):
         if self.token_type == Id.Left_DollarBracket:
             return self._ReadExprSub(lex_mode_e.ShCommand)
 
+        if self.token_type == Id.Left_DollarBraceZsh:
+            return self._ReadZshVarSub(self.cur_token)
+
         raise AssertionError(self.cur_token)
 
     def _ReadExtGlob(self):
@@ -894,7 +918,7 @@ class WordParser(WordEmitter):
                 read_word = False
                 self._SetNext(lex_mode_e.ExtGlob)
 
-            # lex mode EXTGLOB should only produce these 4 kinds of tokens
+            # lex_mode_e.ExtGlob should only produce these 4 kinds of tokens
             elif self.token_kind in (Kind.Lit, Kind.Left, Kind.VSub,
                                      Kind.ExtGlob):
                 w = self._ReadCompoundWord(lex_mode_e.ExtGlob)
@@ -910,6 +934,40 @@ class WordParser(WordEmitter):
 
         return word_part.ExtGlob(left_token, arms, right_token)
 
+    def _ReadBashRegexGroup(self):
+        # type: () -> word_part.BashRegexGroup
+        """
+        Grammar:
+        BashRegexGroup = '(' WORD? ')
+        """
+        left_token = self.cur_token
+        assert left_token.id == Id.BashRegex_LParen, left_token
+
+        right_token = None  # type: Token
+        arms = []  # type: List[CompoundWord]
+
+        self.lexer.PushHint(Id.Op_RParen, Id.Right_BashRegexGroup)
+        self._SetNext(lex_mode_e.BashRegexFakeInner)  # advance past LEFT
+
+        self._GetToken()
+        if self.token_type == Id.Right_BashRegexGroup:  # empty ()
+            return word_part.BashRegexGroup(left_token, None, self.cur_token)
+
+        # lex_mode_e.BashRegex should only produce these 4 kinds of tokens
+        if self.token_kind in (Kind.Lit, Kind.Left, Kind.VSub, Kind.BashRegex):
+            # Fake lexer mode that translates Id.WS_Space to Id.Lit_Chars
+            # To allow bash style [[ s =~ (a b) ]]
+            w = self._ReadCompoundWord(lex_mode_e.BashRegexFakeInner)
+            arms.append(w)
+
+            self._GetToken()
+            if self.token_type != Id.Right_BashRegexGroup:
+                p_die('Expected ) to close bash regex group', self.cur_token)
+
+            return word_part.BashRegexGroup(left_token, w, self.cur_token)
+
+        p_die('Expected word after ( opening bash regex group', self.cur_token)
+
     def _ReadLikeDQ(self, left_token, is_ysh_expr, out_parts):
         # type: (Optional[Token], bool, List[word_part_t]) -> None
         """
@@ -920,7 +978,11 @@ class WordParser(WordEmitter):
           out_parts: list of word_part to append to
         """
         if left_token:
-            expected_end_tokens = 3 if left_token.id == Id.Left_TDoubleQuote else 1
+            if left_token.id in (Id.Left_TDoubleQuote,
+                                 Id.Left_DollarTDoubleQuote):
+                expected_end_tokens = 3
+            else:
+                expected_end_tokens = 1
         else:
             expected_end_tokens = 1000  # here doc will break
 
@@ -964,7 +1026,7 @@ class WordParser(WordEmitter):
 
             elif self.token_kind == Kind.VSub:
                 tok = self.cur_token
-                part = NameTok(tok, lexer.TokenSliceLeft(tok, 1))
+                part = SimpleVarSub(tok)
                 out_parts.append(part)
                 # NOTE: parsing "$f(x)" would BREAK CODE.  Could add a more for it
                 # later.
@@ -999,7 +1061,8 @@ class WordParser(WordEmitter):
             out_parts.pop()
 
         # Remove space from """ in both expression mode and command mode
-        if left_token and left_token.id == Id.Left_TDoubleQuote:
+        if (left_token and left_token.id
+                in (Id.Left_TDoubleQuote, Id.Left_DollarTDoubleQuote)):
             word_compile.RemoveLeadingSpaceDQ(out_parts)
 
         # Return nothing, since we appended to 'out_parts'
@@ -1189,8 +1252,8 @@ class WordParser(WordEmitter):
             UP_lhs = lhs
             with tagswitch(lhs) as case:
                 if case(y_lhs_e.Var):
-                    lhs = cast(NameTok, UP_lhs)
-                    var_checker.Check(kw_token.id, lhs.var_name, lhs.left)
+                    lhs = cast(Token, UP_lhs)
+                    var_checker.Check(kw_token.id, lexer.LazyStr(lhs), lhs)
 
                 # Note: this does not cover cases like
                 # setvar (a[0])[1] = v
@@ -1598,7 +1661,7 @@ class WordParser(WordEmitter):
 
         arg_list = ArgList.CreateNull(alloc_lists=True)
         arg_list.left = self.cur_token
-        self.parse_ctx.ParseYshArgList(self.lexer, arg_list, start_symbol)
+        self.parse_ctx.ParseProcCallArgs(self.lexer, arg_list, start_symbol)
         return arg_list
 
     def _MaybeReadWordPart(self, is_first, lex_mode, parts):
@@ -1744,9 +1807,7 @@ class WordParser(WordEmitter):
             elif self.token_kind == Kind.VSub:
                 vsub_token = self.cur_token
 
-                part = NameTok(vsub_token,
-                               lexer.TokenSliceLeft(vsub_token,
-                                                    1))  # type: word_part_t
+                part = SimpleVarSub(vsub_token)  # type: word_part_t
                 w.parts.append(part)
 
             elif self.token_kind == Kind.ExtGlob:
@@ -1771,6 +1832,14 @@ class WordParser(WordEmitter):
                     part = self._ReadExtGlob()
                 w.parts.append(part)
 
+            elif self.token_kind == Kind.BashRegex:
+                if self.token_type == Id.BashRegex_LParen:  # Opening (
+                    part = self._ReadBashRegexGroup()
+                    w.parts.append(part)
+                else:
+                    assert self.token_type == Id.BashRegex_AllowedInParens
+                    p_die('Invalid token in bash regex', self.cur_token)
+
             elif self.token_kind == Kind.Left:
                 try_triple_quote = (self.parse_opts.parse_triple_quote() and
                                     lex_mode == lex_mode_e.ShCommand and
@@ -1839,6 +1908,10 @@ class WordParser(WordEmitter):
             p_die('Unexpected parts after triple quoted string',
                   loc.WordPart(w.parts[-1]))
 
+        if 0:
+            from _devbuild.gen.syntax_asdl import word_part_str
+            word_key = ' '.join(word_part_str(p.tag()) for p in w.parts)
+            WORD_HIST[word_key] += 1
         return w
 
     def _ReadArithWord(self):
@@ -1876,7 +1949,7 @@ class WordParser(WordEmitter):
         """Helper function for ReadWord()."""
 
         # Change the pseudo lexer mode to a real lexer mode
-        if word_mode == lex_mode_e.ShCommandBrack:
+        if word_mode == lex_mode_e.ShCommandFakeBrack:
             lex_mode = lex_mode_e.ShCommand
         else:
             lex_mode = word_mode
@@ -1923,9 +1996,10 @@ class WordParser(WordEmitter):
             assert self.token_kind in (Kind.VSub, Kind.Lit, Kind.History,
                                        Kind.Left, Kind.KW, Kind.ControlFlow,
                                        Kind.BoolUnary, Kind.BoolBinary,
-                                       Kind.ExtGlob), 'Unhandled token kind'
+                                       Kind.ExtGlob,
+                                       Kind.BashRegex), 'Unhandled token kind'
 
-            if (word_mode == lex_mode_e.ShCommandBrack and
+            if (word_mode == lex_mode_e.ShCommandFakeBrack and
                     self.parse_opts.parse_bracket() and
                     self.token_type == Id.Lit_LBracket):
                 # Change [ from Kind.Lit -> Kind.Op
@@ -1974,24 +2048,26 @@ class WordParser(WordEmitter):
                     #
                     #     echo u'\u{3bc}' b'\yff' works
 
-                    if (self.parse_opts.parse_ysh_string() and
-                            self.cur_token.tval in ('r', 'u', 'b')):
-
-                        if self.cur_token.tval == 'r':
+                    tok = self.cur_token
+                    if self.parse_opts.parse_ysh_string():
+                        if lexer.TokenEquals(tok, 'r'):
                             left_id = Id.Left_RSingleQuote
-                        elif self.cur_token.tval == 'u':
+                        elif lexer.TokenEquals(tok, 'u'):
                             left_id = Id.Left_USingleQuote
-                        else:
+                        elif lexer.TokenEquals(tok, 'b'):
                             left_id = Id.Left_BSingleQuote
+                        else:
+                            left_id = Id.Undefined_Tok
 
-                        # skip the r, and then 'foo' will be read as normal
-                        self._SetNext(lex_mode_e.ShCommand)
+                        if left_id != Id.Undefined_Tok:
+                            # skip the r, and then 'foo' will be read as normal
+                            self._SetNext(lex_mode_e.ShCommand)
 
-                        self._GetToken()
-                        assert self.token_type == Id.Left_SingleQuote, self.token_type
+                            self._GetToken()
+                            assert self.token_type == Id.Left_SingleQuote, self.token_type
 
-                        # Read the word in a different lexer mode
-                        return self._ReadYshSingleQuoted(left_id)
+                            # Read the word in a different lexer mode
+                            return self._ReadYshSingleQuoted(left_id)
 
                 return self._ReadCompoundWord(lex_mode)
 
@@ -2057,7 +2133,8 @@ class WordParser(WordEmitter):
 
         This is a stateful wrapper for the stateless _ReadWord function.
         """
-        assert word_mode in (lex_mode_e.ShCommand, lex_mode_e.ShCommandBrack,
+        assert word_mode in (lex_mode_e.ShCommand,
+                             lex_mode_e.ShCommandFakeBrack,
                              lex_mode_e.DBracket, lex_mode_e.BashRegex)
 
         if self.buffered_word:  # For integration with pgen2
@@ -2106,3 +2183,10 @@ class WordParser(WordEmitter):
     def Multiline(self, b):
         # type: (bool) -> None
         self.multiline = b
+
+
+if 0:
+    import collections
+    WORD_HIST = collections.Counter()
+
+# vim: sw=4
