@@ -50,7 +50,7 @@ from _devbuild.gen.syntax_asdl import (
     loc,
     CompoundWord,
     Token,
-    NameTok,
+    SimpleVarSub,
     BracedVarSub,
     CommandSub,
     DoubleQuoted,
@@ -89,10 +89,20 @@ if TYPE_CHECKING:
     from _devbuild.gen.syntax_asdl import command_t
     from core import alloc
 
+_ = log
+
 
 class Cursor(object):
     """
     API to print/transform a complete source file, stored in a single arena.
+
+    In, core/alloc.py, SnipCodeBlock() and SnipCodeString work on lines.  They
+    don't iterate over tokens.
+
+    Or add a separate hash table of Token -> span ID?  That makes sense because
+    we need that kind of "address hash" for type checking anyway.
+
+    You use the hash table to go from next_token_id .. TokenId(until_token).
     """
 
     def __init__(self, arena, f):
@@ -101,7 +111,7 @@ class Cursor(object):
         self.f = f
         self.next_span_id = 0
 
-    def PrintUntilSpid(self, until_span_id):
+    def _PrintUntilSpid(self, until_span_id):
         # type: (int) -> None
 
         # Sometimes we add +1
@@ -115,12 +125,18 @@ class Cursor(object):
             if span.line is None:
                 continue
 
-            piece = span.line.content[span.col:span.col + span.length]
+            # Special case for recovering stripped leading space!
+            # See osh/word_compile.py
+            start_index = (0 if span.id == Id.Lit_CharsWithoutPrefix else
+                           span.col)
+            end_index = span.col + span.length
+
+            piece = span.line.content[start_index:end_index]
             self.f.write(piece)
 
         self.next_span_id = until_span_id
 
-    def SkipUntilSpid(self, next_span_id):
+    def _SkipUntilSpid(self, next_span_id):
         # type: (int) -> None
         """Skip everything before next_span_id.
 
@@ -133,19 +149,37 @@ class Cursor(object):
 
     def SkipUntil(self, tok):
         # type: (Token) -> None
-        self.SkipUntilSpid(tok.span_id)
+        span_id = self.arena.GetSpanId(tok)
+        self._SkipUntilSpid(span_id)
 
     def SkipPast(self, tok):
         # type: (Token) -> None
-        self.SkipUntilSpid(tok.span_id + 1)
+        span_id = self.arena.GetSpanId(tok)
+        self._SkipUntilSpid(span_id + 1)
 
     def PrintUntil(self, tok):
         # type: (Token) -> None
-        self.PrintUntilSpid(tok.span_id)
+        span_id = self.arena.GetSpanId(tok)
+
+        # Test invariant
+        if mylib.PYTHON:
+            arena_tok = self.arena.GetToken(span_id)
+            if tok != arena_tok:
+                raise AssertionError(
+                    '%s %d %d != %s %d %d' %
+                    (tok, span_id, id(tok), arena_tok,
+                     self.arena.GetSpanId(arena_tok), id(arena_tok)))
+
+        self._PrintUntilSpid(span_id)
 
     def PrintIncluding(self, tok):
         # type: (Token) -> None
-        self.PrintUntilSpid(tok.span_id + 1)
+        span_id = self.arena.GetSpanId(tok)
+        self._PrintUntilSpid(span_id + 1)
+
+    def PrintUntilEnd(self):
+        # type: () -> None
+        self._PrintUntilSpid(self.arena.LastSpanId())
 
 
 def LosslessCat(arena):
@@ -155,7 +189,7 @@ def LosslessCat(arena):
     doc.
     """
     cursor = Cursor(arena, mylib.Stdout())
-    cursor.PrintUntilSpid(arena.LastSpanId())
+    cursor.PrintUntilEnd()
 
 
 def PrintTokens(arena):
@@ -291,7 +325,7 @@ class YshPrinter(object):
     def End(self):
         # type: () -> None
         """Make sure we print until the end of the file."""
-        self.cursor.PrintUntilSpid(self.arena.LastSpanId())
+        self.cursor.PrintUntilEnd()
 
     def DoRedirect(self, node, local_symbols):
         # type: (Redir, Dict[str, bool]) -> None
@@ -442,6 +476,75 @@ class YshPrinter(object):
             if i != n - 1:
                 self.f.write(',')
 
+    def _DoSimple(self, node, local_symbols):
+        # type: (command.Simple, Dict[str, bool]) -> None
+
+        # How to preserve spaces between words?  Do you want to do it?
+        # Well you need to test this:
+        #
+        # echo foo \
+        #   bar
+
+        if len(node.more_env):
+            # We only need to transform the right side, not left side.
+            for pair in node.more_env:
+                self.DoRhsWord(pair.val, local_symbols)
+
+        if len(node.words):
+            first_word = node.words[0]
+            ok, val, quoted = word_.StaticEval(first_word)
+            word0_tok = location.LeftTokenForWord(first_word)
+            if ok and not quoted:
+                if val == '[' and len(node.words) >= 3:
+                    word2 = node.words[-2]
+                    last_word = node.words[-1]
+
+                    # Check if last word is ]
+                    ok, val, quoted = word_.StaticEval(last_word)
+                    if ok and not quoted and val == ']':
+                        # Replace [ with 'test'
+                        self.cursor.PrintUntil(word0_tok)
+                        self.cursor.SkipPast(word0_tok)
+                        self.f.write('test')
+
+                        for w in node.words[1:-1]:
+                            self.DoWordInCommand(w, local_symbols)
+
+                        # Now omit ]
+                        tok2 = location.RightTokenForWord(word2)
+                        rbrack_tok = location.LeftTokenForWord(last_word)
+
+                        # Skip the space token before ]
+                        self.cursor.PrintIncluding(tok2)
+                        # ] takes one spid
+                        self.cursor.SkipPast(rbrack_tok)
+                        return
+                    else:
+                        raise RuntimeError('Got [ without ]')
+
+                elif val == '.':
+                    self.cursor.PrintUntil(word0_tok)
+                    self.cursor.SkipPast(word0_tok)
+                    self.f.write('source')
+                    return
+
+        for w in node.words:
+            self.DoWordInCommand(w, local_symbols)
+
+        # It would be nice to convert here docs to multi-line strings
+        for r in node.redirects:
+            self.DoRedirect(r, local_symbols)
+
+        # TODO: Print the terminator.  Could be \n or ;
+        # Need to print env like PYTHONPATH = 'foo' && ls
+        # Need to print redirects:
+        # < > are the same.  << is here string, and >> is assignment.
+        # append is >+
+
+        # TODO: static_eval of simple command
+        # - [ -> "test".  Eliminate trailing ].
+        # - . -> source, etc.
+
     def DoCommand(self, node, local_symbols, at_top_level=False):
         # type: (command_t, Dict[str, bool], bool) -> None
 
@@ -451,8 +554,8 @@ class YshPrinter(object):
             if case(command_e.CommandList):
                 node = cast(command.CommandList, UP_node)
 
-                # TODO: How to distinguish between echo hi; echo bye; and on separate
-                # lines
+                # TODO: How to distinguish between echo hi; echo bye; and on
+                # separate lines
                 for child in node.children:
                     self.DoCommand(child,
                                    local_symbols,
@@ -461,69 +564,7 @@ class YshPrinter(object):
             elif case(command_e.Simple):
                 node = cast(command.Simple, UP_node)
 
-                # How to preserve spaces between words?  Do you want to do it?
-                # Well you need to test this:
-                #
-                # echo foo \
-                #   bar
-
-                if len(node.more_env):
-                    # We only need to transform the right side, not left side.
-                    for pair in node.more_env:
-                        self.DoRhsWord(pair.val, local_symbols)
-
-                if len(node.words):
-                    first_word = node.words[0]
-                    ok, val, quoted = word_.StaticEval(first_word)
-                    word0_tok = location.LeftTokenForWord(first_word)
-                    if ok and not quoted:
-                        if val == '[':
-                            last_word = node.words[-1]
-                            # Check if last word is ]
-                            ok, val, quoted = word_.StaticEval(last_word)
-                            if ok and not quoted and val == ']':
-                                # Replace [ with 'test'
-                                self.cursor.PrintUntil(word0_tok)
-                                self.cursor.SkipPast(word0_tok)
-                                self.f.write('test')
-
-                                for w in node.words[1:-1]:
-                                    self.DoWordInCommand(w, local_symbols)
-
-                                # Now omit ]
-                                rbrack_tok = location.LeftTokenForWord(
-                                    last_word)
-                                # Skip the space token before ]
-                                self.cursor.PrintUntilSpid(rbrack_tok.span_id -
-                                                           1)
-                                self.cursor.SkipPast(
-                                    rbrack_tok)  # ] takes one spid
-                                return
-                            else:
-                                raise RuntimeError('Got [ without ]')
-
-                        elif val == '.':
-                            self.cursor.PrintUntil(word0_tok)
-                            self.cursor.SkipPast(word0_tok)
-                            self.f.write('source')
-                            return
-
-                for w in node.words:
-                    self.DoWordInCommand(w, local_symbols)
-
-                # It would be nice to convert here docs to multi-line strings
-                for r in node.redirects:
-                    self.DoRedirect(r, local_symbols)
-
-                # TODO: Print the terminator.  Could be \n or ;
-                # Need to print env like PYTHONPATH = 'foo' && ls
-                # Need to print redirects:
-                # < > are the same.  << is here string, and >> is assignment.
-                # append is >+
-
-                # TODO: static_eval of simple command
-                # - [ -> "test".  Eliminate trailing ].
-                # - . -> source, etc.
+                self._DoSimple(node, local_symbols)
 
             elif case(command_e.ShAssignment):
                 node = cast(command.ShAssignment, UP_node)
@@ -678,26 +719,21 @@ class YshPrinter(object):
             elif case(command_e.If):
                 node = cast(command.If, UP_node)
 
-                if node.else_kw:
-                    else_spid = node.else_kw.span_id
-                else:
-                    else_spid = runtime.NO_SPID
-
                 # if foo; then -> if foo {
                 # elif foo; then -> } elif foo {
                 for i, arm in enumerate(node.arms):
-                    elif_spid = arm.spids[0]
-                    then_spid = arm.spids[1]
+                    elif_tok = arm.keyword
+                    then_tok = arm.then_tok
 
                     if i != 0:  # 'if' not 'elif' on the first arm
-                        self.cursor.PrintUntilSpid(elif_spid)
+                        self.cursor.PrintUntil(elif_tok)
                         self.f.write('} ')
 
                     cond = arm.cond
                     if cond.tag() == condition_e.Shell:
                         commands = cast(condition.Shell, cond).commands
-                        if len(commands) == 1 and commands[0].tag(
-                        ) == command_e.Sentence:
+                        if (len(commands) == 1 and
+                                commands[0].tag() == command_e.Sentence):
                             sentence = cast(command.Sentence, commands[0])
                             self.DoCommand(sentence, local_symbols)
 
@@ -708,8 +744,8 @@ class YshPrinter(object):
                             for child in commands:
                                 self.DoCommand(child, local_symbols)
 
-                    self.cursor.PrintUntilSpid(then_spid)
-                    self.cursor.SkipUntilSpid(then_spid + 1)
+                    self.cursor.PrintUntil(then_tok)
+                    self.cursor.SkipPast(then_tok)
                     self.f.write('{')
 
                     for child in arm.action:
@@ -719,7 +755,7 @@ class YshPrinter(object):
                 if len(node.else_action):
                     self.cursor.PrintUntil(node.else_kw)
                     self.f.write('} ')
-                    self.cursor.PrintUntilSpid(node.else_kw.span_id + 1)
+                    self.cursor.PrintIncluding(node.else_kw)
                     self.f.write(' {')
 
                     for child in node.else_action:
@@ -736,8 +772,6 @@ class YshPrinter(object):
                 to_match = None  # type: word_t
                 with tagswitch(node.to_match) as case:
                     if case(case_arg_e.YshExpr):
-                        #self.cursor.PrintUntilSpid(arms_end_spid)
-                        #self.cursor.SkipUntilSpid(arms_end_spid + 1)
                         return
                     elif case(case_arg_e.Word):
                         to_match = cast(case_arg.Word, node.to_match).w
@@ -749,7 +783,7 @@ class YshPrinter(object):
                 # Figure out the variable name, so we can translate
                 # - $var to (var)
                 # - "$var" to (var)
-                var_part = None  # type: NameTok
+                var_part = None  # type: SimpleVarSub
                 with tagswitch(to_match) as case:
                     if case(word_e.Compound):
                         w = cast(CompoundWord, to_match)
@@ -757,7 +791,7 @@ class YshPrinter(object):
 
                         with tagswitch(part0) as case2:
                             if case2(word_part_e.SimpleVarSub):
-                                var_part = cast(NameTok, part0)
+                                var_part = cast(SimpleVarSub, part0)
 
                             elif case2(word_part_e.DoubleQuoted):
                                 dq_part = cast(DoubleQuoted, part0)
@@ -769,12 +803,13 @@ class YshPrinter(object):
                                     # TODO: extract into a common function
                                     with tagswitch(dq_part0) as case3:
                                         if case3(word_part_e.SimpleVarSub):
-                                            var_part = cast(NameTok, dq_part0)
+                                            var_part = cast(
+                                                SimpleVarSub, dq_part0)
                                             #log("VAR PART %s", var_part)
 
                 if var_part:
                     self.f.write(' (')
-                    self.f.write(var_part.var_name)
+                    self.f.write(lexer.LazyStr(var_part.tok))
                     self.f.write(') ')
 
                 self.cursor.SkipPast(node.arms_start)  # Skip past 'in'
@@ -906,16 +941,11 @@ class YshPrinter(object):
                     # NOTE: In double quoted case, this is the begin and end quote.
                     # Do we need a HereDoc part?
 
-                    right_spid = dq_part.right.span_id
-
-                    # This is not set in the case of here docs?  Why not?
-                    assert right_spid != runtime.NO_SPID, right_spid
-
                     if len(dq_part.parts) == 1:
                         part0 = dq_part.parts[0]
                         if part0.tag() == word_part_e.SimpleVarSub:
-                            vsub_part = cast(NameTok, dq_part.parts[0])
-                            if vsub_part.left.id == Id.VSub_At:
+                            vsub_part = cast(SimpleVarSub, dq_part.parts[0])
+                            if vsub_part.tok.id == Id.VSub_At:
                                 self.cursor.PrintUntil(dq_part.left)
                                 self.cursor.SkipPast(
                                     dq_part.right)  # " then $@ then "
@@ -923,11 +953,11 @@ class YshPrinter(object):
                                 return  # Done replacing
 
                             # "$1" -> $1, "$foo" -> $foo
-                            if vsub_part.left.id in (Id.VSub_Number,
-                                                     Id.VSub_DollarName):
+                            if vsub_part.tok.id in (Id.VSub_Number,
+                                                    Id.VSub_DollarName):
                                 self.cursor.PrintUntil(dq_part.left)
                                 self.cursor.SkipPast(dq_part.right)
-                                self.f.write(lexer.TokenVal(vsub_part.left))
+                                self.f.write(lexer.TokenVal(vsub_part.tok))
                                 return
 
                         # Single arith sub, command sub, etc.
@@ -1000,17 +1030,7 @@ class YshPrinter(object):
 
             elif case(word_part_e.Literal):
                 node = cast(Token, UP_node)
-
-                # Print it literally.
-                # TODO: We might want to do it all on the word level though.  For
-                # example, foo"bar" becomes "foobar" in oil.
-                spid = node.span_id
-                if spid == runtime.NO_SPID:
-                    #raise RuntimeError('%s has no span_id' % node.token)
-                    # TODO: Fix word_.TildeDetect to construct proper tokens.
-                    log('WARNING: %s has no span_id' % node)
-                else:
-                    self.cursor.PrintIncluding(node)
+                self.cursor.PrintIncluding(node)
 
             elif case(word_part_e.SingleQuoted):
                 node = cast(SingleQuoted, UP_node)
@@ -1020,8 +1040,7 @@ class YshPrinter(object):
                 # $'\n' is '\n'
                 # TODO: Should print until right_spid
                 # left_spid, right_spid = node.spids
-                if len(node.tokens):  # Empty string has no tokens
-                    self.cursor.PrintIncluding(node.tokens[-1])
+                self.cursor.PrintUntil(node.right)
 
             elif case(word_part_e.DoubleQuoted):
                 node = cast(DoubleQuoted, UP_node)
@@ -1029,30 +1048,29 @@ class YshPrinter(object):
                     self.DoWordPart(part, local_symbols, quoted=True)
 
             elif case(word_part_e.SimpleVarSub):
-                node = cast(NameTok, UP_node)
+                node = cast(SimpleVarSub, UP_node)
 
-                spid = node.left.span_id
-                op_id = node.left.id
+                op_id = node.tok.id
 
                 if op_id == Id.VSub_DollarName:
-                    self.cursor.PrintIncluding(node.left)
+                    self.cursor.PrintIncluding(node.tok)
 
                 elif op_id == Id.VSub_Number:
-                    self.cursor.PrintIncluding(node.left)
+                    self.cursor.PrintIncluding(node.tok)
 
                 elif op_id == Id.VSub_At:  # $@ -- handled quoted case above
                     self.f.write('$[join(ARGV)]')
-                    self.cursor.SkipPast(node.left)
+                    self.cursor.SkipPast(node.tok)
 
                 elif op_id == Id.VSub_Star:  # $*
                     # PEDANTIC: Depends if quoted or unquoted
                     self.f.write('$[join(ARGV)]')
-                    self.cursor.SkipPast(node.left)
+                    self.cursor.SkipPast(node.tok)
 
                 elif op_id == Id.VSub_Pound:  # $#
                     # len(ARGV) ?
                     self.f.write('$Argc')
-                    self.cursor.SkipPast(node.left)
+                    self.cursor.SkipPast(node.tok)
 
                 else:
                     pass
